// Conversion.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef CONVERSION_H
#define CONVERSION_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//const Uint16 UNICODE_ENGLISH_START =  0x0021; // 012... (starts w/ '!')
//const Uint16 UNICODE_ENGLISH_START =  0x0030; // 012...
const Uint16 UNICODE_ENGLISH_START =  0x0020; // Starts with ' ' (space).
const Uint16 UNICODE_ENGLISH_END =    0x007E; // Ends with '~' (tilde).
const Uint16 UNICODE_HIRAGANA_START = 0x3040; // actually 0x3040 is the reserved start of it.
const Uint16 UNICODE_HIRAGANA_END =	  0x309F; // (0x3097-0x3098 not used)
const Uint16 UNICODE_KATAKANA_START = 0x30A0;
const Uint16 UNICODE_KATAKANA_END =   0x30FF;
///////////////////////////////////////////////////////////
Uint16 KANA_OFFSET = 0x0060;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsKatakana( Uint16 c ) {
	if( c >= UNICODE_KATAKANA_START && c <= UNICODE_KATAKANA_END ) {
		return( true );
	}
	return( false );
}
bool IsHiragana( Uint16 c ) {
	if( c >= UNICODE_HIRAGANA_START && c <= UNICODE_HIRAGANA_END ) {
		if( c >= 0x3041 && c <= 0x3096 ) {  // no extra characters.
			return( true );
		}
	}
	return( false );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsKatakana( wstring &str ) {
	int length = str.size();
	for( int i = 0; i < length; i++ ) {
		if( !IsKatakana( str[i] ) ) {
			return( false );
		}
	}
	return( true );
}
bool IsHiragana( wstring &str ) {
	int length = str.size();
	for( int i = 0; i < length; i++ ) {
		if( !IsHiragana( str[i] ) ) {
			return( false );
		}
	}
	return( true );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Is a Uint16 kana? */
bool IsKana( Uint16 c ) {
	if( IsHiragana(c) || IsKatakana(c) ) {
		return(true);
	}
	return(false);
}
/* Is a wstring made up of only kana? */
bool IsKana( wstring &str ) {
	for( int i = 0; i < str.size(); i++ ) {
		if( !(IsHiragana(str[i])||IsKatakana(str[i])) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
/* Is a wstring made up of only kana or wildcards? */
bool IsKana_WildcardsOkay( wstring &str ) {
	for( int i = 0; i < str.size(); i++ ) {
		if( !(IsHiragana(str[i])||IsKatakana(str[i])) ) {
			// If it's not kana, is it a wildcard?
			if( str[i] == WC_QUES || str[i] == WC_STAR ) {
				// If so, then we're still okay.
			} else {
				return(false);
			}
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Convert a Hiragana character to Katakana. */
Uint16 ToKatakana( Uint16 c ) {
	if( IsHiragana(c) ) {
		int offset = c - UNICODE_HIRAGANA_START;
		c = UNICODE_KATAKANA_START+offset;
	}
	return( c );
}
/* Convert a Katakana character to Hiragana. */
Uint16 ToHiragana( Uint16 c ) {
	if( IsKatakana(c) ) {
		int offset = c - UNICODE_KATAKANA_START;
		c = UNICODE_HIRAGANA_START+offset;
	}
	return( c );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Convert a wstring to Katakana. */
wstring ToKatakana( wstring &str ) {
	wstring katakana = str;
	for( int i = 0; i < str.length(); i++ ) {
		katakana[i] = ToKatakana( str[i] );
	}
	return(katakana);
}
/* Convert a wstring to Hiragana. */
wstring ToHiragana( wstring &str ) {
	wstring hiragana = str;
	for( int i = 0; i < str.length(); i++ ) {
		hiragana[i] = ToHiragana( str[i] );
	}
	return(hiragana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringsAreEqual_IgnoreKana( wstring &a, wstring &b ) {
	if( a.length() == b.length() ) {
		int len = a.length();
		for( int i = 0; i < len; i++ ) {
			if( ToHiragana(a[i]) == ToHiragana(b[i]) && ToKatakana(a[i]) == ToKatakana(b[i]) ) {
			} else {
				return( false );
			}
		}
	} else {
		return( false );
	}
	return( true );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsVowel( Uint16 c ) {
	if( c==L'a' || c==L'i' || c==L'u' || c==L'e' || c==L'o' ) {
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsRomaji( Uint16 c ) {
	if( c >= UNICODE_ENGLISH_START && c <= UNICODE_ENGLISH_END ) {
		return(true);
	} else {
		return(false);
	}
}

bool IsRomaji( wstring str ) {
	for( int i = 0; i < str.size(); i++ ) {
		if( !IsRomaji(str[i]) ) {
			return(false);
		}
	}
	return(true);
}


bool IsRomajiLetter( Uint16 c ) {
	if( (c >= L'A' && c <= L'Z') || (c >= L'a' && c <= L'z') ) {
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsUpper( Uint16 u ) {
	if( u >= L'A' && u <= L'Z' ) {
		return(true);
	}
	return(false);
}
///////////////////////////////////////////////////////////
bool IsLower( Uint16 u ) {
	if( u >= L'a' && u <= L'z' ) {
		return(true);
	}
	return(false);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsLower( wstring &text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsUpper(text[i]) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
bool IsUpper( wstring &text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsLower(text[i]) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Uint16 ToLower( Uint16 u ) {
	if( IsUpper(u) ) {
		u = u + (L'a'-L'A');
	}
	return(u);
}
///////////////////////////////////////////////////////////
Uint16 ToUpper( Uint16 u ) {
	if( IsLower(u) ) {
		u = u - (L'a'-L'A');
	}
	return(u);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring ToLower( wstring text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsUpper(text[i]) ) {
			text[i] = text[i] + (L'a'-L'A');
		}
	}
	return(text);
}
///////////////////////////////////////////////////////////
wstring ToUpper( wstring text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsLower(text[i]) ) {
			text[i] = text[i] - (L'a'-L'A');
		}
	}
	return(text);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Capital letters denote smaller kana:
Uint16 RomajiToKana_Single( wstring c ) {
	int offset = !IsLower(c)?KANA_OFFSET:0x0000;
	c = ToLower(c);
	Uint16 u = 0;
		  if( c==L"la" || c==L"xa") {		u = 0x3041 + offset;	}
	else if( c==L"a" )	{		u = 0x3042 + offset;	}
	else if( c==L"li" || c==L"xi" ) {		u = 0x3043 + offset;	}
	else if( c==L"i" ) {			u = 0x3044 + offset;	}
	else if( c==L"lu" || c==L"xu" ) {		u = 0x3045 + offset;	}
	else if( c==L"u" ) {			u = 0x3046 + offset;	}
	else if( c==L"le" || c==L"xe" ) {		u = 0x3047 + offset;	}
	else if( c==L"e" ) {			u = 0x3048 + offset;	}
	else if( c==L"lo" || c==L"xo" ) {		u = 0x3049 + offset;	}
	else if( c==L"o" ) {			u = 0x304A + offset;	}
	else if( c==L"ka" ) {		u = 0x304B + offset;	}
	else if( c==L"ga" ) {		u = 0x304C + offset;	}
	else if( c==L"ki" ) {		u = 0x304D + offset;	}
	else if( c==L"gi" ) {		u = 0x304E + offset;	}
	else if( c==L"ku" ) {		u = 0x304F + offset;	}
	else if( c==L"gu" ) {		u = 0x3050 + offset;	}
	else if( c==L"ke" ) {		u = 0x3051 + offset;	}
	else if( c==L"ge" ) {		u = 0x3052 + offset;	}
	else if( c==L"ko" ) {		u = 0x3053 + offset;	}
	else if( c==L"go" ) {		u = 0x3054 + offset;	}
	else if( c==L"sa" ) {		u = 0x3055 + offset;	}
	else if( c==L"za" ) {		u = 0x3056 + offset;	}
	else if( c==L"si" ) {		u = 0x3057 + offset;	}
	else if( c==L"zi" ) {		u = 0x3058 + offset;	}
	else if( c==L"su" ) {		u = 0x3059 + offset;	}
	else if( c==L"zu" ) {		u = 0x305A + offset;	}
	else if( c==L"se" ) {		u = 0x305B + offset;	}
	else if( c==L"ze" ) {		u = 0x305C + offset;	}
	else if( c==L"so" ) {		u = 0x305D + offset;	}
	else if( c==L"zo" ) {		u = 0x305E + offset;	}
	else if( c==L"ta" ) {		u = 0x305F + offset;	}
	else if( c==L"da" ) {		u = 0x3060 + offset;	}
	else if( c==L"ti" ) {		u = 0x3061 + offset;	}
	else if( c==L"di" ) {		u = 0x3062 + offset;	}
	else if( c==L"ltu"||c==L"xtu" ) {		u = 0x3063 + offset;	}
	else if( c==L"tu" ) {		u = 0x3064 + offset;	}
	else if( c==L"du" ) {		u = 0x3065 + offset;	}
	else if( c==L"te" ) {		u = 0x3066 + offset;	}
	else if( c==L"de" ) {		u = 0x3067 + offset;	}
	else if( c==L"to" ) {		u = 0x3068 + offset;	}
	else if( c==L"do" ) {		u = 0x3069 + offset;	}
	else if( c==L"na" ) {		u = 0x306A + offset;	}
	else if( c==L"ni" ) {		u = 0x306B + offset;	}
	else if( c==L"nu" ) {		u = 0x306C + offset;	}
	else if( c==L"ne" ) {		u = 0x306D + offset;	}
	else if( c==L"no" ) {		u = 0x306E + offset;	}
	else if( c==L"ha" ) {		u = 0x306F + offset;	}
	else if( c==L"ba" ) {		u = 0x3070 + offset;	}
	else if( c==L"pa" ) {		u = 0x3071 + offset;	}
	else if( c==L"hi" ) {		u = 0x3072 + offset;	}
	else if( c==L"bi" ) {		u = 0x3073 + offset;	}
	else if( c==L"pi" ) {		u = 0x3074 + offset;	}
	else if( c==L"hu" ) {		u = 0x3075 + offset;	}
	else if( c==L"bu" ) {		u = 0x3076 + offset;	}
	else if( c==L"pu" ) {		u = 0x3077 + offset;	}
	else if( c==L"he" ) {		u = 0x3078 + offset;	}
	else if( c==L"be" ) {		u = 0x3079 + offset;	}
	else if( c==L"pe" ) {		u = 0x307A + offset;	}
	else if( c==L"ho" ) {		u = 0x307B + offset;	}
	else if( c==L"bo" ) {		u = 0x307C + offset;	}
	else if( c==L"po" ) {		u = 0x307D + offset;	}
	else if( c==L"ma" ) {		u = 0x307E + offset;	}
	else if( c==L"mi" ) {		u = 0x307F + offset;	}
	else if( c==L"mu" ) {		u = 0x3080 + offset;	}
	else if( c==L"me" ) {		u = 0x3081 + offset;	}
	else if( c==L"mo" ) {		u = 0x3082 + offset;	}
	else if( c==L"lya" || c==L"xya" ) {		u = 0x3083 + offset;	}
	else if( c==L"ya" ) {		u = 0x3084 + offset;	}
	else if( c==L"lyu" || c==L"xyu" ) {		u = 0x3085 + offset;	}
	else if( c==L"yu" ) {		u = 0x3086 + offset;	}
	else if( c==L"lyo" || c==L"xyo" ) {		u = 0x3087 + offset;	}
	else if( c==L"yo" ) {		u = 0x3088 + offset;	}
	else if( c==L"ra" ) {		u = 0x3089 + offset;	}
	else if( c==L"ri" ) {		u = 0x308A + offset;	}
	else if( c==L"ru" ) {		u = 0x308B + offset;	}
	else if( c==L"re" ) {		u = 0x308C + offset;	}
	else if( c==L"ro" ) {		u = 0x308D + offset;	}
	else if( c==L"lwa" || c==L"xwa" ) {		u = 0x308E + offset;	}
	else if( c==L"wa" ) {		u = 0x308F + offset;	}
	else if( c==L"wi" ) {		u = 0x3090 + offset;	}
	else if( c==L"we" ) {		u = 0x3091 + offset;	}
	else if( c==L"wo" ) {		u = 0x3092 + offset;	}
	else if( c==L"n" ) {			u = 0x3093 + offset;	} else if( c==L"n'" ) {		u = 0x3093 + offset;	}
	else if( c==L"vu" ) {		u = 0x3094 + offset;	}
	else if( c==L"lka" || c==L"xka" ) {		u = 0x3095 + offset;	}
	else if( c==L"lke" || c==L"xke" ) {		u = 0x3096 + offset;	}
	// Katakana only:
	else if( c==L"va" ) {		u = 0x30F7;  }
	else if( c==L"vi" ) {		u = 0x30F8;  }
	else if( c==L"ve" ) {		u = 0x30F9;  }
	else if( c==L"vo" ) {		u = 0x30FA;  }
	else if( c==L"*" ) {			u = 0x30FB;  }
	else if( c==L"-" ) {			u = 0x30FC;  }

	return( u );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Uint16 RomajiToFullwidth( Uint16 c ) {
	if( c >= 0x0021 && c <= 0x007E ) {
		return( c + (0xFF01-0x0021) );
	}
	return( 0 );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring RomajiToKana( wstring text ) {
	wstring kana;

	Uint16 offset = !IsLower(text)?KANA_OFFSET:0x0000;

	text = ToLower(text);

	// Go over the kya patterns:
	if( text.size() == 3 && text[1] == L'y' && text[0] != L'l' ) {
		wstring ipart;
			ipart = text[0];
			ipart += L'i';
			//MessageBox(NULL,ipart.c_str(),text.c_str(),MB_OK );
		wstring ypart;
			ypart = L'l';
			ypart += text[1];
			ypart += text[2];
		kana = RomajiToKana_Single( ipart ) + offset;
		kana += RomajiToKana_Single( ypart ) + offset;
	} else {
		Uint16 c = RomajiToKana_Single( text );
		if( c != 0 ) {
			kana = c + offset;
		}
	}

	return(kana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring RomajiStringToKana( wstring text, bool guessend = true ) {
#ifdef DEBUG_CONVERSION
	static bool first = true;
	FILE *fcon = NULL;
	if( first ) {
		fcon = fopen("out.conversion.txt","wb");
		fwprintf(fcon,L"%c",0xFEFF );
		first = false;
	} else {
		fcon = fopen("out.conversion.txt","ab");
	}

	fwprintf(fcon,L"Conversion: %s\n",text.c_str() );
#endif


	wstring kana = L"";

	wstring temp;
	bool katakana = false;
	for( int i = 0; i < text.size(); i++ ) {
		Uint16 c = text[i];

		if( IsUpper(c) ) {
			katakana = true;
			c = ToLower(c);
		}

		//fwprintf(fcon,L"romaji: <<%c>> (0x%04X) %skatakana\n",c,c,katakana?L"":L"NOT ");

		// If it's ROMAji, then we're gonna convert it to kana:
		// (make sure that it's not a wildcard).
		//if( IsRomaji(c) && c != WC_QUES && c != WC_STAR ) {
		if( (IsRomajiLetter(c) || c == L'\'' || c == L'-' || c == L'*') && c != WC_QUES && c != WC_STAR ) {
			temp += c;

			// If it's NOT a vowel, then we're going to make sure it's okay.
			if( !IsVowel(c) ) {
				if( temp.size() == 2 ) {
					// Check for 'double consonants':
					if( temp[0] == temp[1] ) {
						// Check for n's:
						// For example, break the 'nn' in konniti to 'n' + 'n'...
						if( temp[0] == L'n' ) {
							if( katakana ) {  kana += RomajiToKana(L"N");	}
							else {	kana += RomajiToKana(L"n");	}
							temp = L'n';
						}
						// Check for tsu's:
						// In other words, every double consonant that isn't an 'n' is an 'ltu'...
						else {
							if( katakana ) {	kana += RomajiToKana(L"LTU");  }
							else {	kana += RomajiToKana(L"ltu");  }
							temp = temp[1];
						}
					} else {
					// If the consonants are NOT the same:
						// Check for n's:
						// For example, break the 'np' in konpyuta- to 'n' + 'p'...
						if( temp[0] == L'n' && temp[1] != L'\'' ) {
							if( katakana ) {  kana += RomajiToKana(L"N");	}
							else {	kana += RomajiToKana(L"n");	}
							temp = temp[1];
						}
					}
				}// (size==2)
				// Do (if size==1) AFTER ==2 because it may get SMALLER after that one:
				if( temp.size() == 1 ) {
						  /*if( temp == L"j" ) {  temp = L"zy";  }
					else*/ //if( temp == L"f" ) {	temp = L'h';  }
				} // (size==1)
			}// (!isvowel)

			// If it's a vowel or a special character, then it's time to get the kana for it:
			if( IsVowel(c) || c == L'\'' || c == L'-' || c == L'*' || i == text.size()-1 ) {
#ifdef DEBUG_CONVERSION
				fwprintf(fcon,L"   temp: <<%s>> ... ",temp.c_str() );
#endif



				// Even if it's a vowel, fix up the shoddy ROMAji:
				if( temp.size() == 2 ) {
						  if( temp==L"ja" ) {	temp=L"zya";	}
					else if( temp==L"ji" ) {	temp=L"zi";	}
					else if( temp==L"ju" ) {	temp=L"zyu";	}
					else if( temp==L"jo" ) {	temp=L"zyo";	}
					else if( temp == L"fa" ) {
						if( !katakana ) {	kana += RomajiToKana(L"hu") + RomajiToKana(L"la");  }
						else {				kana += RomajiToKana(L"HU") + RomajiToKana(L"LA");  }
						temp = L"";
					}
					else if( temp == L"fi" ) {
						if( !katakana ) {	kana += RomajiToKana(L"hu") + RomajiToKana(L"li");  }
						else {				kana += RomajiToKana(L"HU") + RomajiToKana(L"LI");  }
						temp = L"";
					}
					else if( temp == L"fu" ) {
						if( !katakana ) {	kana += RomajiToKana(L"hu");  }
						else {				kana += RomajiToKana(L"HU");  }
						temp = L"";
					}
					else if( temp == L"fe" ) {
						if( !katakana ) {	kana += RomajiToKana(L"hu") + RomajiToKana(L"le");  }
						else {				kana += RomajiToKana(L"HU") + RomajiToKana(L"LE");  }
						temp = L"";
					}
					else if( temp == L"fo" ) {
						if( !katakana ) {	kana += RomajiToKana(L"hu") + RomajiToKana(L"lo");  }
						else {				kana += RomajiToKana(L"HU") + RomajiToKana(L"LO");  }
						temp = L"";
					}
					else if( temp == L"va" ) {
						if( !katakana ) {	kana += RomajiToKana(L"vu") + RomajiToKana(L"la");  }
						else {				kana += RomajiToKana(L"VU") + RomajiToKana(L"LA");  }
						temp = L"";
					}
					else if( temp == L"vi" ) {
						if( !katakana ) {	kana += RomajiToKana(L"vu") + RomajiToKana(L"li");  }
						else {				kana += RomajiToKana(L"VU") + RomajiToKana(L"LI");  }
						temp = L"";
					}
					else if( temp == L"ve" ) {
						if( !katakana ) {	kana += RomajiToKana(L"vu") + RomajiToKana(L"le");  }
						else {				kana += RomajiToKana(L"VU") + RomajiToKana(L"LE");  }
						temp = L"";
					}
					else if( temp == L"vo" ) {
						if( !katakana ) {	kana += RomajiToKana(L"vu") + RomajiToKana(L"lo");  }
						else {				kana += RomajiToKana(L"VU") + RomajiToKana(L"LO");  }
						temp = L"";
					}
				}
				else if( temp.size() == 3) {
						  if( temp==L"shi" ) {	temp = L"si";  }
					else if( temp==L"sha" ) {	temp = L"sya";	}
					else if( temp==L"shu" ) {	temp = L"syu";	}
					else if( temp==L"sho" ) {	temp = L"syo";	}
					else if( temp==L"chi" ) {	temp = L"ti";	}
					else if( temp==L"cha" ) {	temp = L"tya";	}
					else if( temp==L"chu" ) {	temp = L"tyu";	}
					else if( temp==L"cho" ) {	temp = L"tyo";	}
					else if( temp==L"tsu" ) {  temp = L"tu";  }
					else if( temp==L"dzu" ) {  temp = L"du";  }
				}// (size==3)
				else if( temp.size() == 4 ) {
					if( temp==L"ltsu" || temp==L"xtsu" ) {	temp = L"ltu";	}
				}




				if( katakana ) {	temp = ToUpper(temp);	}
#ifdef DEBUG_CONVERSION
				fwprintf(fcon,L" <<%s>> ... ",temp.c_str() );
#endif
				wstring temp_old = temp;
				temp = RomajiToKana( temp );
				if( temp[0] != 0 ) {
#ifdef DEBUG_CONVERSION
					fwprintf(fcon,L"<<%s>>\n",temp.c_str() );
#endif
					// If do NOT have GuessEnd turned on,
					// then don't let it convert a lone 'n' to hiragana 'n'.
					if( !guessend && i == text.size()-1 && ToLower(temp_old) == L"n" ) {
						kana += temp_old;
					} else {
						kana += temp;
					}
				} else {
#ifdef DEBUG_CONVERSION
					fwprintf(fcon,L"(0)\n");
#endif
					// NEW !!!
					//return(kana);
					kana += temp_old;
				}

				temp = L"";
				katakana = false;
			}
		}// (isromaji)
		else {
			if( IsHiragana(c)||IsKatakana(c) ) {
				kana += c;
			} else {
				if( IsRomaji(c) ) {
					kana += RomajiToFullwidth(c);
				} else
			// It's probably a kanji character; add it, too:
				kana += c;
			}
		}
	}// for

#ifdef DEBUG_CONVERSION
	fwprintf(fcon,L"Converted to: <<%s>> (%i)\n\n",kana.c_str(), kana.size() );
	fclose(fcon);
#endif

	return(kana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// HIRAGANA DEFINES:
#define H_la	0x3041
#define H_a		0x3042
#define H_li	0x3043
#define H_i		0x3044
#define H_lu	0x3045
#define H_u		0x3046
#define H_le	0x3047
#define H_e		0x3048
#define H_lo	0x3049
#define H_o		0x304A
#define H_ka	0x304B
#define H_ga	0x304C
#define H_ki	0x304D
#define H_gi	0x304E
#define H_ku	0x304F
#define H_gu	0x3050
#define H_ke	0x3051
#define H_ge	0x3052
#define H_ko	0x3053
#define H_go	0x3054
#define H_sa	0x3055
#define H_za	0x3056
#define H_si	0x3057
#define H_zi	0x3058
#define H_su	0x3059
#define H_zu	0x305A
#define H_se	0x305B
#define H_ze	0x305C
#define H_so	0x305D
#define H_zo	0x305E
#define H_ta	0x305F
#define H_da	0x3060
#define H_ti	0x3061
#define H_di	0x3062
#define H_ltu	0x3063
#define H_tu	0x3063
#define H_du	0x3065
#define H_te	0x3066
#define H_de	0x3067
#define H_to	0x3068
#define H_do	0x3069
#define H_na	0x306A
#define H_ni	0x306B
#define H_nu	0x306C
#define H_ne	0x306D
#define H_no	0x306E
#define H_ha	0x306F
#define H_ba	0x3070
#define H_pa	0x3071
#define H_hi	0x3072
#define H_bi	0x3073
#define H_pi	0x3074
#define H_hu	0x3075
#define H_bu	0x3076
#define H_pu	0x3077
#define H_he	0x3078
#define H_be	0x3079
#define H_pe	0x307A
#define H_ho	0x307B
#define H_bo	0x307C
#define H_po	0x307D
#define H_ma	0x307E
#define H_mi	0x307F
#define H_mu	0x3080
#define H_me	0x3081
#define H_mo	0x3082
#define H_lya	0x3083
#define H_ya	0x3084
#define H_lyu	0x3085
#define H_yu	0x3086
#define H_lyo	0x3087
#define H_yo	0x3088
#define H_ra	0x3089
#define H_ri	0x308A
#define H_ru	0x308B
#define H_re	0x308C
#define H_ro	0x308D
#define H_lwa	0x308E
#define H_wa	0x308F
#define H_wi	0x3090
#define H_we	0x3091
#define H_wo	0x3092
#define H_n		0x3093
#define H_vu	0x3094
#define H_lka	0x3095
#define H_lke	0x3096
/*	// Katakana only:
	else if( c==0x30F7 ) {  r = L"va";  }
	else if( c==0x30F8 ) {  r = L"vi";  }
	else if( c==0x30F9 ) {  r = L"ve";  }
	else if( c==0x30FA ) {  r = L"vo";  }
	else if( c==0x30FB ) {  r = L"*";  }
	else if( c==0x30FC ) {  r = L"-";  }//*/
///////////////////////////////////////////////////////////
// Capital letters denote smaller kana:
wstring KanaToRomaji_Single( Uint16 c ) {
	bool katakana = IsKatakana( c );
	if( katakana ) {  c = c - KANA_OFFSET;  }
	wstring r = L"";
		  if( c==0x3041 ) {	r = L"la";	}
	else if( c==0x3042 ) {	r = L"a";	}
	else if( c==0x3043 ) {  r = L"li";  }
	else if( c==0x3044 ) {	r = L"i";	}
	else if( c==0x3045 ) {  r = L"lu";	} 
	else if( c==0x3046 ) {  r = L"u";	}
	else if( c==0x3047 ) {  r = L"le";	}
	else if( c==0x3048 ) {  r = L"e";	}
	else if( c==0x3049 ) {  r = L"lo";	}
	else if( c==0x304A ) {  r = L"o";	}
	else if( c==0x304B ) {  r = L"ka";	}
	else if( c==0x304C ) {  r = L"ga";	}
	else if( c==0x304D ) {  r = L"ki";	}
	else if( c==0x304E ) {  r = L"gi";	}
	else if( c==0x304F ) {  r = L"ku";	}
	else if( c==0x3050 ) {  r = L"gu";	}
	else if( c==0x3051 ) {  r = L"ke";	}
	else if( c==0x3052 ) {  r = L"ge";	}
	else if( c==0x3053 ) {  r = L"ko";	}
	else if( c==0x3054 ) {  r = L"go";	}
	else if( c==0x3055 ) {  r = L"sa";	}
	else if( c==0x3056 ) {  r = L"za";	}
	else if( c==0x3057 ) {  r = L"si";	}
	else if( c==0x3058 ) {  r = L"zi";	}
	else if( c==0x3059 ) {  r = L"su";	}
	else if( c==0x305A ) {  r = L"zu";	}
	else if( c==0x305B ) {  r = L"se";	}
	else if( c==0x305C ) {  r = L"ze";	}
	else if( c==0x305D ) {  r = L"so";	}
	else if( c==0x305E ) {  r = L"zo";	}
	else if( c==0x305F ) {  r = L"ta";	}
	else if( c==0x3060 ) {  r = L"da";	}
	else if( c==0x3061 ) {  r = L"ti";	}
	else if( c==0x3062 ) {  r = L"di";	}
	else if( c==0x3063 ) {  r = L"ltu";	}
	else if( c==0x3064 ) {  r = L"tu";	}
	else if( c==0x3065 ) {  r = L"du";	}
	else if( c==0x3066 ) {  r = L"te";	}
	else if( c==0x3067 ) {  r = L"de";	}
	else if( c==0x3068 ) {  r = L"to";	}
	else if( c==0x3069 ) {  r = L"do";	}
	else if( c==0x306A ) {  r = L"na";	}
	else if( c==0x306B ) {  r = L"ni";	}
	else if( c==0x306C ) {  r = L"nu";	}
	else if( c==0x306D ) {  r = L"ne";	}
	else if( c==0x306E ) {  r = L"no";	}
	else if( c==0x306F ) {  r = L"ha";	}
	else if( c==0x3070 ) {  r = L"ba";	}
	else if( c==0x3071 ) {  r = L"pa";	}
	else if( c==0x3072 ) {  r = L"hi";	}
	else if( c==0x3073 ) {  r = L"bi";	}
	else if( c==0x3074 ) {  r = L"pi";	}
	else if( c==0x3075 ) {  r = L"hu";	}
	else if( c==0x3076 ) {  r = L"bu";	}
	else if( c==0x3077 ) {  r = L"pu";	}
	else if( c==0x3078 ) {  r = L"he";	}
	else if( c==0x3079 ) {  r = L"be";	}
	else if( c==0x307A ) {  r = L"pe";	}
	else if( c==0x307B ) {  r = L"ho";	}
	else if( c==0x307C ) {  r = L"bo";	}
	else if( c==0x307D ) {  r = L"po";	}
	else if( c==0x307E ) {  r = L"ma";	}
	else if( c==0x307F ) {  r = L"mi";	}
	else if( c==0x3080 ) {  r = L"mu";	}
	else if( c==0x3081 ) {  r = L"me";	}
	else if( c==0x3082 ) {  r = L"mo";	}
	else if( c==0x3083 ) {  r = L"lya";	}
	else if( c==0x3084 ) {  r = L"ya";	}
	else if( c==0x3085 ) {  r = L"lyu";	}
	else if( c==0x3086 ) {  r = L"yu";	}
	else if( c==0x3087 ) {  r = L"lyo";	}
	else if( c==0x3088 ) {  r = L"yo";	}
	else if( c==0x3089 ) {  r = L"ra";	}
	else if( c==0x308A ) {  r = L"ri";	}
	else if( c==0x308B ) {  r = L"ru";	}
	else if( c==0x308C ) {  r = L"re";	}
	else if( c==0x308D ) {  r = L"ro";	}
	else if( c==0x308E ) {  r = L"lwa";	}
	else if( c==0x308F ) {  r = L"wa";	}
	else if( c==0x3090 ) {  r = L"wi";	}
	else if( c==0x3091 ) {  r = L"we";	}
	else if( c==0x3092 ) {  r = L"wo";	}
	else if( c==0x3093 ) {  r = L"n";	}
	else if( c==0x3094 ) {  r = L"vu";	}
	else if( c==0x3095 ) {  r = L"lka";	}
	else if( c==0x3096 ) {  r = L"lke";	}
	// Katakana only:
	if( katakana ) {
		c = c + KANA_OFFSET;
			  if( c==0x30F7 ) {  r = L"va";  }
		else if( c==0x30F8 ) {  r = L"vi";  }
		else if( c==0x30F9 ) {  r = L"ve";  }
		else if( c==0x30FA ) {  r = L"vo";  }
		else if( c==0x30FB ) {  r = L"*";  }
		else if( c==0x30FC ) {  r = L"-";  }
		else {
			r = ToUpper( r );
		}
	}

	// Fullwidth ROMAji:
	if( c >= 0xFF01 && c <= 0xFF5E ) {
		r = Uint16( c - (0xFF01-0x0021) );
	}


	return( r );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring KanaStringToRomaji( wstring kana ) {
	wstring r = L"";

	// Go through the katakana string:
	for( int k = 0; k < kana.size(); k++ ) {
		Uint16 c = kana[k];

		bool katakana = IsKatakana(c);

		Uint16 ch = c;
		if( katakana ) {
			ch = c - KANA_OFFSET;
		}

		switch( ch ) {
			// (ltu) ** patterns:
			case H_ltu: {
				if( k < kana.size()-1 && IsKana(kana[k+1]) ) {
					wstring r2 = KanaToRomaji_Single( kana[k+1] );
					r += r2[0];
				} else {
					r += KanaToRomaji_Single( c );
				}
				break;
			}
			// NN patterns:
			case H_n: {
				if( k < kana.size()-1 && IsKana(kana[k+1]) ) {
					Uint16 c2 = kana[k+1];
					if( (c2 >= H_la && c2 <= H_o) || (c2 >= H_na && c2 <= H_no) ) {
						r += KanaToRomaji_Single( c );
						r += L"'";
					} else {
						r += KanaToRomaji_Single( c );
					}
				} else {
					r += KanaToRomaji_Single( c );
				}
				break;
			}
			// *y{a,u,o} patterns:
			case H_ki:
			case H_gi:
			case H_si:
			case H_zi:
			case H_ti:
			case H_di:
			case H_ni:
			case H_hi:
			case H_bi:
			case H_pi:
			case H_mi:
			case H_ri: {
				if( k < kana.size()-1 && IsKana(kana[k+1]) ) {
					Uint16 c2 = kana[k+1];
					if( c2 == H_lya || c2 == H_lyu || c2 == H_lyo ) {
						r += KanaToRomaji_Single(c)[0] + KanaToRomaji_Single(c2).substr(1,-1);
						k++;
					} else if( c2 == H_li || c2 == H_le ) {
						r += KanaToRomaji_Single(c)[0] + L"y" + KanaToRomaji_Single(c2).substr(1,-1);
						k++;
					} else {
						r += KanaToRomaji_Single( c );
					}
				} else {
					r += KanaToRomaji_Single( c );
				}
				break;
			}
			// HU and VU patterns:
			case H_hu:
			case H_vu: {
				if( k < kana.size()-1 && IsKana(kana[k+1]) ) {
					Uint16 c2 = kana[k+1];
					if( c2 == H_la || c2 == H_li || c2 == H_lu || c2 == H_le || c2 == H_lo ) {
						r += KanaToRomaji_Single(c)[0] + KanaToRomaji_Single(c2).substr(1,-1);
						k++;
					} else {
						r += KanaToRomaji_Single( c );
					}
				} else {
					r += KanaToRomaji_Single( c );
				}
				break;
			}
			// DEFAULT:
			default: {
				r += KanaToRomaji_Single( c );
			}
		}//switch(ch)
	}

	return(r);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////