// Dialog_Options_Misc.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_MISC_H
#define DIALOG_OPTIONS_MISC_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HBRUSH hbrush_misc = GetSysColorBrush(COLOR_MENU);
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options_Misc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// Set the text of the font boxes:
			Uint16 str[1024];
				swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Misc\\Font\\Font"), registry.GetINT(".\\Misc\\Font\\Size") );
			SendDlgItemMessage( hWnd, IDC_MISC_HEADER, WM_SETTEXT, 0, (LPARAM)str );

			return(TRUE);
		}
		case WM_COMMAND: {
			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				case IDC_MISC_FONT_HEADER: {
				/*		The 'Font' button for header was clicked.
					We need to pop up a choose-font box so he can choose the font.
					If a font was chosen ('ok'), then we need to update the registry
					and update the font that the result window uses.
				*/
					switch( code ) {
						case BN_CLICKED: {
							CHOOSEFONT cf;
							ZeroMemory(&cf, sizeof(cf));
								cf.lStructSize = sizeof(cf);
								cf.hwndOwner = hWnd;
								cf.Flags = CF_FORCEFONTEXIST|CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT|CF_LIMITSIZE;
								//cf.Flags = 0;//CF_FORCEFONTEXIST|CF_SCREENFONTS|CF_LIMITSIZE;
								static LOGFONT lf;
								//LOGFONT lf;
									switch( identifier ) {
										case IDC_MISC_FONT_HEADER: {
											wcscpy( lf.lfFaceName, ToWString( registry.GetSTRING(".\\Misc\\Font\\Font") ).c_str() );
											lf.lfHeight = -MulDiv( registry.GetINT(".\\Misc\\Font\\Size"), GetDeviceCaps(GetDC(hwnd_searchwindow), LOGPIXELSY), 72);
											break;
										}
										default: {
											Log(L"ERROR: DlgProc_Options_Misc(): Unknown identifier\n");
										}
									}
									//lf.lfWidth = 0;
									//lf.lfEscapement = ??;
									//lf.lfOrientation = ??;
									lf.lfWeight = FW_NORMAL;
									lf.lfItalic = FALSE;
									lf.lfUnderline = FALSE;
									lf.lfStrikeOut = FALSE;
									lf.lfCharSet = SHIFTJIS_CHARSET;
									//lf.lfCharSet = DEFAULT_CHARSET;
									lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
									lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
									lf.lfQuality = DEFAULT_QUALITY;
									lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
								cf.lpLogFont = &lf;
								cf.nSizeMin = 6;
								cf.nSizeMax = 100;
							// Show the choose-font window:
							if( ChooseFont( &cf ) ) {
							// If the clicked 'ok'...
								switch( identifier ) {
									case IDC_MISC_FONT_HEADER: {
										// Set the new values:
										registry.SetSTRING(".\\Misc\\Font\\Font", ToString(lf.lfFaceName).c_str() );
										registry.SetINT(".\\Misc\\Font\\Size", (cf.iPointSize/10) );
								
										// Set the text of the box:
										Uint16 str[1024];
											swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Misc\\Font\\Font"), registry.GetINT(".\\Misc\\Font\\Size") );
										SendDlgItemMessage( hWnd, IDC_MISC_HEADER, WM_SETTEXT, 0, (LPARAM)str );


										// Kill the old font and make the new one:
										DeleteObject( hfont_msgothic12 );
										hfont_msgothic12 = CreateFontIndirect( &lf );


										// Update the affected areas:
										SendMessage( hwnd_resultlist[0],WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)true );
										SendMessage( hwnd_resultlist[1],WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)true );
										SendMessage( hwnd_kanjilist1,WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)true );
										SendMessage( hwnd_kanjilist2,WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)true );	
										SendMessage( hwnd_searchrebar_edit_resultsize, WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)true );		
										SendMessage( hwnd_searchrebar_combobox_encode, WM_SETFONT, (WPARAM)hfont_msgothic12, (LPARAM)TRUE );

										break;
									}
								}

								// Redraw the search window:
								RedrawWindow( hwnd_searchwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
			}
			return(TRUE);
		}
		case WM_CTLCOLORSTATIC: {
		/*		This message is sent when an edit box needs to be drawn.
			It gives you the ability to set the foreground and background colors.
			Also, you return the brush that you want use to color the rest
			(non-text) areas.
		*/
			/*HDC hdc_edit = (HDC)wParam;
			HWND hwnd_edit = (HWND)lParam;

			switch( GetDlgCtrlID(hwnd_edit) ) {
				case IDC_MISC_HEADER: {
					SetTextColor( hdc_edit, registry.GetINT(".\\Misc\\Font\\Color") );
					//SetBkColor( hdc_edit, registry.GetINT(".\\Search\\WindowBG") );
					return( (BOOL)hbrush_misc );
				}
			}
			//*/
			return(FALSE);
		}
	}
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateDialog_Options_Misc( HWND hwndParent ) {
	HWND hwnd = CreateDialog(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS_MISC ), 
		hwndParent,
		(DLGPROC)DlgProc_Options_Misc
	);
	return( hwnd );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
