// EDICT_Sort.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef EDICT_SORT_H
#define EDICT_SORT_H
///////////////////////////////////////////////////////////
#include "Result_List.h"

#include <Algorithm>
///////////////////////////////////////////////////////////
/* This is the SORT_BY structure.
   It's used by the sort functions to know how to actually
	sort everything.
   * window_id is used to know what set of attributes to use.
   * index is the index in the header that is window_id.
   * sortstyle is ASCENDING or DESCENDING. */
struct SORT_BYK {
	int window_id, index;
	int sortstyle;
};

// New school:
struct SORT_BY {
	int index;
	int sortstyle;
};

/* SortStyle:
   This is either ASCENDING or DESCENDING. */
enum {	ASCENDING = 0,
			DESCENDING = 1	};

/* This is the enumeration of PRIMARY and SECONDARY, for 
   use with the sort_by array. */
enum {	PRIMARY = 0,
			SECONDARY	};

/* sort_by[] keeps information on how to actually do the
   sort.  See the SORT_BY structure. */
SORT_BY sort_by[2] = { {-1,0},{-1,0} };
/* sort_on is used by the sort functions to pick what
   attribute to sort on.
   Those functions will use sort_by[sort_on] to do it. */
int sort_on = PRIMARY;

/* This determines if two entries are equal. */
bool operator==( const EDICT_Entry &a, const EDICT_Entry &b ) {
	switch( sort_by[sort_on].index ) {
		case HDR_RESULT_WORD: { // WORD
			return( a.word == b.word );
			break;
		}
		case HDR_RESULT_INFO: { // TYPE
			break;
		}
		case HDR_RESULT_WORD_L: { // LENGTH: WORD
			return( a.word.size() == b.word.size() );
			break;
		}
		case HDR_RESULT_KANA_L: { // LENGTH: KANA
			return( a.kana.size() == b.kana.size() );
			break;
		}
		case HDR_RESULT_KANA: { // KANA
			return( a.kana == b.kana );
			break;
		}
		case HDR_RESULT_MEANING: { // MEANING
			if( a.meaning.size() >= 1 && b.meaning.size() >= 1 ) {
				if( a.meaning[0].size() > 0 && b.meaning[0].size() > 0 ) {
					return( a.meaning[0][0] == b.meaning[0][0] );
				} else {
					Log(L"Sort: ==: ERROR: a.meaning[0].size or b.meaning[0].size < 1\n");
				}
			} else {
				Log(L"Sort: ==: ERROR: a.meaning.size or b.meaning.size < 1");
			}
			break;
		}
	}//*/
	return( a.word.size() == b.word.size() );
}

/* This uses sort_by[sort_on] to determine if one entry
   is 'greater' than another. */
bool operator>( const EDICT_Entry &a, const EDICT_Entry &b ) {
	switch( sort_by[sort_on].index ) {
		case HDR_RESULT_WORD: { // WORD
			if( sort_by[sort_on].sortstyle == ASCENDING ) {
				return( a.word > b.word );
			} else {
				return( !(a.word > b.word) );
			}
			break;
		}
		case HDR_RESULT_INFO: { // TYPE
			break;
		}
		case HDR_RESULT_WORD_L: { // LENGTH: WORD
			if( sort_by[sort_on].sortstyle == ASCENDING ) {
				return( a.word.size() > b.word.size() );
			} else {
				return( !(a.word.size() > b.word.size()) );
			}
			break;
		}
		case HDR_RESULT_KANA_L: { // LENGTH: KANA
			if( sort_by[sort_on].sortstyle == ASCENDING ) {
				return( a.kana.size() > b.kana.size() );
			} else {
				return( !(a.kana.size() > b.kana.size()) );
			}
			break;
		}
		case HDR_RESULT_KANA: { // KANA
			if( sort_by[sort_on].sortstyle == ASCENDING ) {
				return( a.kana > b.kana );
			} else {
				return( !(a.kana > b.kana) );
			}
			break;
		}
		case HDR_RESULT_MEANING: { // MEANING
			if( a.meaning.size() >= 1 && b.meaning.size() >= 1 ) {
				if( a.meaning[0].size() > 0 && b.meaning[0].size() > 0 ) {
					if( sort_by[sort_on].sortstyle == ASCENDING ) {
						return( a.meaning[0][0] > b.meaning[0][0] );
					} else {
						return( !(a.meaning[0][0] > b.meaning[0][0]) );
					}
				} else {
					Log(L"Sort: >: ERROR: a.meaning[0].size or b.meaning[0].size < 1\n");
				}
			} else {
				Log(L"Sort: >: ERROR: a.meaning.size or b.meaning.size < 1");
			}
			break;
		}
	}//*/
	return( a.word.size()>b.word.size() );
}

/* If it's not greater than an entry and it's not equal
   to it, then it must be less than it. */
bool operator<( const EDICT_Entry &a, const EDICT_Entry &b ) {
	return( !(a > b) && !(a == b) );
}

/* Sort the results by the primary and secondary sorts.
   If there is no secondary, it skips it.
	If there is no primary, it skips it.
//*/
void SortResults() {
#ifdef UBERDEBUG
	Log(L"(Not sorting EDICT results)\n");
	return;
#endif
	if( edict_results.size() > 0 ) {
		// If there is a secondary way to sort, do it:
		sort_on = SECONDARY;
			sort( edict_results.begin(), edict_results.end() );
		
		// Sort by the primary:
		sort_on = PRIMARY;
			sort( edict_results.begin(), edict_results.end() );
		
		RedrawWindow( hwnd_resultwindow, NULL, NULL, RDW_INVALIDATE );
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
