// Edict.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef EDICT_H
#define EDICT_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Used as info for the results: */
struct EDICT_Entry {
	wstring word;
	wstring kana;
	vector<wstring> meaning;
	vector<wstring> info;
	bool selected;

	EDICT_Entry() : selected(false) {};
};
///////////////////////////////////////////////////////////
// These are the results at any given time:
vector<EDICT_Entry> edict_results;

// Functions to parse the markers:
#include "Edict_markers.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This returns the number of lines in the file.
   Note: It resets the file-position pointer.
	This is used to find out how many entries are in an EDICT file. */
long CountLinesInFile( FILE *f ) {
	fseek(f, 0L, 0 );

	int edict_buffer = registry.GetINT(".\\Buffer\\EDICT");

	long lines = 0;
	char *line = new char[edict_buffer];
	while( !feof(f) ) {
			// Clear the line: (I'm not sure if this is necessary)...
			strnset( line, '\0', edict_buffer );
		fgets( line, edict_buffer-1, f );
			if( feof(f) ) {  break;  }
		if( line[strlen(line)-1] != '\n' ) {
			Log(L"CountLinesInFile: ERROR: Increase the edict buffer size.\n");
		}
		lines++;
	}
	delete [] line;
	fseek(f, 0L, 0 );
	return( lines );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This goes through the edict file and grabs the word,
   the kana, and the location of the line. */
void EDICT_Index_Load( Dictionary_EDICT *dictionary ) {
/*	Okay, here's how things work:
	1) If we use indexing, we load from the index.
		a) If the file exists,
			i) If it's the right format,
				* Load the index file.
				* return.
			Otherwise, 2)
	2) Load the file from EDICT
	3) If indexing is enabled,
		a) If we need to make an index,
			* Make the index file.
*/

	// Set some default values:
	dictionary->index_entries = NULL;
	dictionary->index_entries_size = 0;


	////////////////////
	///   INDEXING   ///
	////////////////////
	// Are we going to make a new index (as in, is one *not* there)?
	bool make_new_index = false;
	// Are we using Indexing?
	if( registry.GetBOOL(".\\Dictionaries\\Indexing") ) {
		/*Uint16 fname[1024];
			swprintf(fname,L"out.index.%S.txt",dictionary->filename.substr( dictionary->filename.find_last_of('\\')+1,-1 ).c_str() );
		FILE *f = _wfopen(fname,L"wb");
		if( !f ) {  MessageBox(NULL,L"Error in out.index.txt",L"!",MB_OK);  return;  }
			fwprintf(f,L"%c",0xFEFF);//*/

		// See if the index file is there:
		// NOTE: _wfopen() is only needed if the fileNAME is in wide-char format.
		char *file_index = new char[dictionary->filename.size()+4+1];
			sprintf(file_index,"%s.jli",dictionary->filename.c_str() );
		FILE *findex = fopen(file_index,"rb");//*/
		// If the index file *is* there...
		if( findex != NULL ) {
			// (The index file was opened successfully).

			// Get the first line (it should be the version.
			Uint16 header[100];
			fgetws( header, 100-1, findex );
			if( wcscmp(header,L"JLI3\n") == 0 ) {
				// The version is correct.  Continue.

				// Get how many entries there are:
				fread( &dictionary->index_entries_size, sizeof(int), 1, findex );
				dictionary->index_entries = new EDICT_Index_Entry[ dictionary->index_entries_size ];

				Log(L"EDICT_Index_Load: Number of entries: %i (in '%S')\n", dictionary->index_entries_size,dictionary->displayname.c_str() );

				// Load the entries in the index:
				for( int current = 0; current < dictionary->index_entries_size; current++ ) {
					// Format: <size><word><size><kana><fileloc>...
					Uint8 size; // (this is the length of the word/kana).
					// Read in the word:
					fread( &size, sizeof(Uint8), 1, findex );
						dictionary->index_entries[current].word = new Uint16[size+1];
					fread( dictionary->index_entries[current].word, sizeof(Uint16), size, findex );
					dictionary->index_entries[current].word[size] = L'\0';

					// Read in the kana:
					fread( &size, sizeof(Uint8), 1, findex );
						dictionary->index_entries[current].kana = new Uint16[size+1];
					fread( dictionary->index_entries[current].kana, sizeof(Uint16), size, findex );
					dictionary->index_entries[current].kana[size] = L'\0';
					
					// Read in the file location:
					long fileloc;
					fread( &fileloc, sizeof(long), 1, findex );
						dictionary->index_entries[current].fileloc = fileloc;

					/*if(1) {
						fwprintf(f,L"%s || %s || %li\n", dictionary->index_entries[current].word,dictionary->index_entries[current].kana,dictionary->index_entries[current].fileloc );
					}//*/
				}
				fclose(findex);

				//fclose(f);

				// At this point, we have loaded the index file.
				// We are done with this.
				////////////////////
				///   ALL DONE   ///
				////////////////////
				return;
			} else {
				// The file version is NOT correct.
				Log(L"WARNING: Index file (%S) is not correct.\n",file_index );

				Uint16 text[2000];
					swprintf(text,L"\
This file: %S, is not a correct index file.\n\
\n\
This could be because it is an older version.  In any case, the file will not work.\n\
\n\
JLookUp will NOT use it as an index file.  Do you want to erase the file and have it be rewritten correctly?", file_index );
				if( MessageBox(NULL,text,L"Note:",MB_YESNO ) == IDYES ) {
					make_new_index = true;
					Log(L"   Going to make new index file after EDICT load...\n");
				} else {
					Log(L"   Ignoring index file...\n");
				}

				fclose( findex );
			}
		} else {
			// There's no index as of now, so we have to do it manually:
			make_new_index = true; // Make a new index after this.
		}
		// Free up the string:
		delete [] file_index;
	}


	// Here, we're either *not* using Indexing, or, there is *no* index to use.
	// Open the EDICT file:
	FILE *fedict = fopen(dictionary->filename.c_str(),"rb");
	if( !fedict ) {
		Log(L"EDICT_Index_Load: ERROR: %S not opened.\n",dictionary->filename.c_str() );
		return;
	}

	int edict_buffer = registry.GetINT(".\\Buffer\\EDICT");

	// Get how many entries there are:
	long linesinfile = CountLinesInFile( fedict );
	// (The number of entries can't be *more* than that.)
	dictionary->index_entries = new EDICT_Index_Entry[linesinfile];


	Log(1,L"EDICT_Index_Load()\n");
	Log(1,L"   Lines in the EDICT file: %i\n",linesinfile );
	Log(1,L"   EDICT Buffer: %i\n",edict_buffer );

	char *line = new char[edict_buffer];
	int current = 0;
	while( !feof(fedict) ) {
		if( current == linesinfile ) {
			// Most likely there's not error here; it's just counting that last line.
			break;
		}

		// Set the information about the entry:
		dictionary->index_entries[current].fileloc = ftell( fedict );
		dictionary->index_entries[current].word = NULL;
		dictionary->index_entries[current].kana = NULL;
		//fprintf(g_flog,"   preentry #%i / %i-1: %s || %s || %li\n",current-1,dictionary->index_entries_size, dictionary->index_entries[current].word, dictionary->index_entries[current].kana, dictionary->index_entries[current].fileloc );

		// Clear the line:
		strnset( line, '\0', edict_buffer );

		// Read the line:
		fgets( line, edict_buffer-1, fedict );
			if( feof(fedict) ) {  break;  }
		if( line[strlen(line)-1] != '\n' ) {
			Log(L"EDICT_Index_Load: ERROR: (Loading Dictionary from EDICT) Increase the Edict_Buffer_Size for %S.\n",dictionary->filename.c_str() );
			break;
		}
		// Get rid of all of the whitespace until the final '/':
		while( line[strlen(line)-1] != '/' ) {
			line[strlen(line)-1] = '\0';
		}



		// Find the word and kana.
		char *word = NULL, *kana = NULL;
		word = line;
		int pos = ( strchr(word,' ') - word );
		if( pos > 0 ) {
			if( strlen(word) <= pos ) {
				Log(L"WARNING: Dictionary '%S' has an error on line %i (it's missing anything other than the word).\n", dictionary->filename.c_str(), current+1 );
			} else {
				word[pos] = '\0';
				kana = word+pos+1;

				if( kana[0] == '[' ) {
					kana = kana+1;

					// NOTE: the legal terms dictionary uses ['s and ]'s INSIDE of WORD and KANA
					// entries, for things like [~]wosuru, etc.  So, we have to make sure that
					// the kana is absolutely correct, meaning that the next character after the
					// final ] is a space.
					pos = strchr(kana,' ') - kana - 1; // -1 to get rid of the ].
					if( pos > 0 ) {
						if( kana[pos] != ']' ) {
							// This is technically an error in the dictionary, but we can work around it.
							Log(L"WARNING: Dictionary '%S' has an error on line %i (it's missing the ending kana bracket).\n", dictionary->filename.c_str(), current+1 );
							Log(L"   JLookUp is attempting to work around the problem.\n");
							// Move one to the right (this *should* be a space).
							pos++;
						}
						kana[pos] = '\0';
					} else { Log(L"EDICT_Index_Load(): God-awful error #2\n");  }
				} else {
					kana = NULL;
				}// if( kana[0] == '[' )
			}// if( strlen(word) <= pos )
		} else {  Log(L"EDICT_Index_Load(): God-awful error #1\n");  }


		char *encoding = (char*)dictionary->encoding.c_str();
		// Convert the word to Unicode:
		Uint16 *wd = (Uint16*)ConvertEncoding( word,strlen(word), encoding,"UTF-16LE" );
		if( wd ) {
			dictionary->index_entries[current].word = new Uint16[wcslen(wd)+1];
			wcscpy( dictionary->index_entries[current].word, wd );
			delete [] wd;
		} else {
			Log(L"ERROR: Could not convert word to Unicode: <<%S>>\n",word);
		}
		// Convert the kana to Unicode:
		if( kana ) {
			Uint16 *kn = (Uint16*)ConvertEncoding( kana,strlen(kana), encoding,"UTF-16LE" );
			if( kn ) {
				dictionary->index_entries[current].kana = new Uint16[wcslen(kn)+1];
				wcscpy( dictionary->index_entries[current].kana, kn );
				delete [] kn;
			} else {
				Log(L"ERROR: Could not convert kana to Unicode: <<%S>>\n",kana );
			}
		} else {
			dictionary->index_entries[current].kana = new Uint16[wcslen(dictionary->index_entries[current].word)+1];
			wcscpy( dictionary->index_entries[current].kana, dictionary->index_entries[current].word );
		}



		current++;
		// Set the number of entries as we go:
		dictionary->index_entries_size = current;

		/*
		if(true) {//if( current < 10 ) {
			//Log(L"(%i|%i) <%S>\n   ",strlen(word),strlen(kana),kana?kana+pos+1:word+pos+1 );

			//fprintf(g_flog,"Line: %s\n",line );
			Log(L"   Entry #%i: %s || %s || %li\n",current-1, dictionary->index_entries[current-1].word, dictionary->index_entries[current-1].kana, dictionary->index_entries[current-1].fileloc );
			//fprintf(g_flog,"   Entry #%i / %i: || %li\n",current-1,dictionary->index_entries_size-1, dictionary->index_entries[current-1].fileloc );
		}//*/
	}
	delete [] line;

	// See if everything's okay:
	if( dictionary->index_entries_size != current ) {
		Log(L"There seems to be an error.  index_entries_size != current.\n");
	}

	//Log(L"Lines in file / entries: %i / %i\n",linesinfile,current );

	fclose(fedict);


	//fprintf(g_flog,"%s: %i (%i) entries.\n",dictionary->filename.c_str(), dictionary->index_entries_size,(int)linesinfile );
	/*for( int i = 0; i < dictionary->index_entries_size; i++ ) {
		fprintf(g_flog,"%s || %s || %li\n",dictionary->index_entries[i].word,dictionary->index_entries[i].kana, dictionary->index_entries[i].fileloc );
	}//*/
	//fclose(g_flog);
	//g_flog = fopen("out.txt","at");


	////////////////////
	///   INDEXING   ///
	////////////////////
	// Write an index file to save time,
	// IF we're using Indexing AND we did load something to index...
	if( registry.GetBOOL(".\\Dictionaries\\Indexing") && dictionary->index_entries_size > 0 ) {//if( JLookUp_Options.edict_use_index && dictionary->index_entries_size > 0 ) {
		// If we don't have to make a new index, just quit:
		if( !make_new_index ) {  return;  }

		// Otherwise, make a new index:
		char *file_index = new char[dictionary->filename.size()+4+1];
			sprintf(file_index,"%s.jli",dictionary->filename.c_str() );
		FILE *findex = fopen(file_index,"wb");
		if( findex ) {
			// Set the status bar text:
			Uint16 sbtext[1024]; swprintf(sbtext,L"Making index: %S...",file_index );
			SetStatusBarText( sbtext );

			// Actually make the index:
			// Write the header line:
			fwprintf(findex,L"JLI3\n");
			// Number of entries:
			fwrite( &dictionary->index_entries_size, sizeof(int), 1, findex );
			for( int i = 0; i < dictionary->index_entries_size; i++ ) {
				// See if we can write the word/kana:
				if( wcslen(dictionary->index_entries[i].word) >= 256 || wcslen(dictionary->index_entries[i].kana) >= 256 ) {
					Log(L"WARNING: either word or kana is > 255 characters long!\n");
					Log(L"%s\n%s\n",dictionary->index_entries[i].word, dictionary->index_entries[i].kana );
				}
				if( wcslen(dictionary->index_entries[i].word) <= 0 || wcslen(dictionary->index_entries[i].kana) <= 0 ) {
					Log(L"WARNING: either word or kana is 0 characters long!\n");
					Log(L"%s\n%s\n",dictionary->index_entries[i].word, dictionary->index_entries[i].kana );
				}

				// Word:
				Uint8 size = wcslen(dictionary->index_entries[i].word);
				fwrite( &size, sizeof(Uint8), 1, findex );
				fwrite( dictionary->index_entries[i].word, sizeof(Uint16), size, findex );
				
				// Kana:
				size = wcslen(dictionary->index_entries[i].kana);
				fwrite( &size, sizeof(Uint8), 1, findex );
				fwrite( dictionary->index_entries[i].kana, sizeof(Uint16), size, findex );

				fwrite( &dictionary->index_entries[i].fileloc, sizeof(long), 1, findex );
			}
			fclose(findex);
		} else {
			Log(L"EDICT_Index_Load: ERROR: file pointer for (%S) is null.  Index not made.\n",file_index);
			//MessageBox(NULL,L"Index wasn't made.",L"Error",MB_OK );
		}
		delete [] file_index;
	}
	return;
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This uses the location of the line to fill in information
   in 'entry'. */
void LoadInfoInto( Dictionary_EDICT *dictionary, EDICT_Entry &entry, EDICT_Index_Entry &index ) {
	FILE *fedict = fopen( dictionary->filename.c_str(),"rb");
	if( !fedict ) {
		Log(L"ERROR: LoadInfoInto(): Could NOT open: %S\n",dictionary->filename.c_str() );
		return;
	}

	// Go to that entry's location:
	fseek( fedict, index.fileloc, 0 );


	// Get the buffer size:
	int edict_buffer = registry.GetINT(".\\Buffer\\EDICT");

	char *line = new char[edict_buffer];
		// Clear the line:
		strnset( line, '\0', edict_buffer );
	fgets( line, edict_buffer-1, fedict );
		// If it's the end of the file, stop:
		if( feof(fedict) ) {
			Log(L"ERROR: LoadInfoInto(): End of file found\n");
			fclose(fedict);
			delete [] line;
			return;
		}

	// Make sure that / is the last character.
	while( strlen(line) > 1 && line[strlen(line)-1] != '/' ) {
		line[strlen(line)-1] = '\0';
	}

	// Convert the line to Unicode:
	char *encoding = (char*)dictionary->encoding.c_str();
	wstring strline = L"";
	int status = ConvertEncoding( line,strlen(line), encoding,"UTF-16LE", strline );

	// Fill in the word & kana.
	// NOTE: EDICT_Entry's word & kana are wstrings.
	//       EDICT_Index_Entry's are Uint16*s.
	entry.word = index.word;
	entry.kana = index.kana;

	if( status < 0 ) {
		Log(L"ERROR: There was a problem converting the encoding to Unicode.\n");
		Log(L"   String: <<%S>>\n",line );
		// Fill in the entry's meaning with an error line:
		Uint16 str[1000];
			swprintf(str,L"ERROR in '%S' (%S).",dictionary->filename.c_str(), dictionary->displayname.c_str() );
		entry.meaning.push_back(str);
	} else {
		// Get the meanings:
		int pos = strline.find( '/',0 );
		strline = strline.substr( pos );
		while( strline.size() > 0 ) {
			if( strline[0] == L'/' ) {
				pos = strline.find( '/',1 );
				if( pos != -1 ) {
					if( pos != 1 ) {
					// If there is SOMETHING between the /'s...
						wstring strmeaning = strline.substr( 1,pos-1 );
						entry.meaning.push_back( strmeaning );
					}

					strline = strline.substr( pos,-1 );
				} else {
					break;
				}
			} else {
				break;
			}
		}
	}// status >= 0
	
	// We're done with the dictionary file; close it.
	fclose(fedict);

/*#ifdef DEBUGX
	Log(L"   WORD: %s\n",entry.word.c_str() );
	Log(L"   KANA: %s\n",entry.kana.c_str() );
	Log(L"   Meanings:\n");
	for( int i = 0; i < entry.meaning.size(); i++ ) {
	   Log(L"      <<%s>>\n",entry.meaning[i].c_str() );
	}
#endif//*/

	// We're done with the line; Delete it.
	delete [] line;
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Used with English searches;
      This uses the location of the line to fill in information
   in 'entry'. 
	   The reason that it takes a FILE* is that loading the file
	every time just takes too long.  We know that we're going
	to go through the whole thing, so just leave it open.
*/
void LoadInfoInto( FILE *fedict, Dictionary_EDICT *dictionary, EDICT_Entry &entry, EDICT_Index_Entry &index ) {
	Log(1,L"LoadInfoInto()\n");
	if( !fedict ) {  return;  }

	// Go to the entry's location:
	fseek( fedict, index.fileloc, 0 );


	int edict_buffer = registry.GetINT(".\\Buffer\\EDICT");


	char *line = new char[edict_buffer];
		// Clear the line: (I'm not sure if this is necessary)...
		strnset( line, '\0', edict_buffer );
	fgets( line, edict_buffer-1, fedict );
		if( feof(fedict) ) {
			fclose(fedict);
			delete [] line;
			Log(L"ERROR: LoadInfoInto(FILE*): End of file\n");
			return;
		}
	// Make sure that / is the last character.
	while( strlen(line) > 1 && line[strlen(line)-1] != '/' ) {
		line[strlen(line)-1] = '\0';
	}

	// Convert the line to Unicode:
	char *encoding = (char*)dictionary->encoding.c_str();
	wstring strline;
	int status = ConvertEncoding( line,strlen(line), encoding,"UTF-16LE", strline );//*/

	/*// Just make the thing a wide-character string:
	// (we won't be doing any Japanese-conversion with it,
	//  since we already know the word and kana).
	wstring strline = ToWString( line );//*/

	// Fill in the word & kana.
	// NOTE: EDICT_Entry's word & kana are wstrings.
	//       EDICT_Index_Entry's are Uint16*s.
	entry.word = index.word;
	entry.kana = index.kana;
	entry.meaning.clear();

	if( status < 0 ) {
		Log(L"ERROR: There was a problem converting the encoding to Unicode.\n");
		Log(L"   String: <<%S>>\n",line );
		// Fill in the entry's meaning with an error statement.
		Uint16 str[1000];
			swprintf(str,L"ERROR in '%S' (%S).",dictionary->filename.c_str(), dictionary->displayname.c_str() );
		entry.meaning.push_back(str);
	} else {
		// Get the meanings:
		// (Go to the first '/' and work from there).
		int pos = strline.find( '/',0 );
		strline = strline.substr( pos );
		while( strline.size() > 0 ) {
			if( strline[0] == L'/' ) {
				pos = strline.find( '/',1 );
				if( pos != -1 ) {
					wstring strmeaning = strline.substr( 1,pos-1 );
				
					entry.meaning.push_back( strmeaning );

					strline = strline.substr( pos,-1 );
				} else {
					break;
				}
			} else {
				break;
			}
		}
	}

	// We're done with the line; Delete it.
	delete [] line;

	// Here's all of the info that we got from this:
	Log(1,L"   WORD: %s\n",entry.word.c_str() );
	Log(1,L"   KANA: %s\n",entry.kana.c_str() );
	Log(1,L"   Meanings:\n");
	if( debug_level > 0 ) {
		for( int i = 0; i < entry.meaning.size(); i++ ) {
		   Log(1,L"      <<%s>>\n",entry.meaning[i].c_str() );
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* Used with English searches;
      This returns a char* line (NOT Unicode).
*/
char* GetEdictLine( FILE *fedict, EDICT_Index_Entry &index ) {
	if( !fedict ) {  return(NULL);  }
	
	// Go to the entry's location:
	fseek( fedict, index.fileloc, 0 );


	int edict_buffer = registry.GetINT(".\\Buffer\\EDICT");

	char *line = new char[edict_buffer];
		// Clear the line: (I'm not sure if this is necessary)...
		strnset( line, '\0', edict_buffer );
	fgets( line, edict_buffer-1, fedict );
		if( feof(fedict) ) {
			fclose(fedict);
			delete [] line;
			Log(L"ERROR: GetEdictLine(): End of file\n");
			return(NULL);
		}
	// Make sure that / is the last character.
	while( strlen(line) > 1 && line[strlen(line)-1] != '/' ) {
		line[strlen(line)-1] = '\0';
	}
	
	return( line );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
