// Edict_markers.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef EDICT_MARKERS_H
#define EDICT_MARKERS_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
struct EDICT_Marker {
	wstring marker;
	wstring meaning;
};
///////////////////////////////////////////////////////////
vector<EDICT_Marker> EDICT_Markers;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void LoadMarkers() {
	FILE *fin = fopen( "partsofspeech.jlf","rt" );
	if( !fin ) {
		Log(L"ERROR: LoadMarkers(): Could not open partsofspeech.jlf\n");
		return;
	}

	// Here's the line that we're going to read in:
	char line[1024];
	while( !feof(fin) ) {
		// Clear the line:
		strnset( line, '\0', 1024 );
		// Read in the line:
		fgets(line,1024-1,fin);
			if( feof(fin) ) {  break;  }
		// If it's a comment, keep going:
		if( strlen(line) > 0 && line[0] == '#' ) {  continue;  }

		// Strip off the end-line stuff:
		if( strlen(line) > 0 && line[strlen(line)-1] == '\n' ) {
			line[strlen(line)-1] = '\0';
		}

		string strline = line;
		int pos = strline.find(0x09); // (tab)

		if( pos != -1 ) {
			EDICT_Marker em;
				em.marker = ToWString( strline.substr(0,pos) );
				em.meaning = ToWString( strline.substr(pos+1,-1) );
			EDICT_Markers.push_back( em );

			//Log(L"Marker: <<%s>>\nMeaning: <<%s>>\n\n",em.marker.c_str(),em.meaning.c_str() );
		} else {
			Log(L"ERROR: LoadMarkers(): could not find the TAB character <<%S>>.\n",line);
		}
	}
	fclose(fin);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This returs a string of the long-version of the info
   based on the short-version. */
wstring GetInfo( wstring text ) {
	for( int i = 0; i < EDICT_Markers.size(); i++ ) {
		if( text == EDICT_Markers[i].marker ) {
			return( EDICT_Markers[i].meaning );
		}
	}
	return(L"");
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    Set the info for an entry. */
void SetInfo( EDICT_Entry &entry ) {
	// If there's nothing to set, quit:
	if( entry.meaning.size() == 0 ) {  return;  }
	if( entry.meaning[0].size() == 0 ) {  return;  }

#ifdef UBERDEBUG
	return;
#endif

	Log(1,L"SetInfo() :: Setting info...\n");
	// Go through every meaning and strip out markers.
	for( int i = 0; i < entry.meaning.size(); i++ ) {
		Log(2,L"   entry.meaning[%i]: <<%s>>\n",i,entry.meaning[i].c_str() );
		wstring line1 = entry.meaning[i];

		// Okay, 'line1' is the current meaning that we're playing with.

		bool changedline = false;
		for( int j = 0; j < line1.size(); j++ ) {
			if( line1[j] == L'(' ) {
				int end = line1.find( L')', j );
				if( end < 0 ) {
					Log(L"SetInfo(): ERROR: could not find ending ')'.\n");
				}
				// 'stuff' is the stuff INBETWEEN the parentheses.
				wstring stuff = line1.substr( j+1,(end)-(j+1) );
				Log(2,L"   stuff: %s\n",stuff.c_str() );

				// If 'foudninfo' is true, then we're going to change the entry's
				// meaning line to be the one without the markers.
				bool foundinfo = false;

				// Check for separate, internal info:
				int next = -1;
				while( true ) {
					// If there's a comma, then there are at least two parts.
					next = stuff.find( L',' );

					// 'part' is the part that we're working on now.
					wstring part = stuff.substr( 0, next );
					Log(2,L"   part: %s\n",part.c_str() );
					// Get the info for that part:
					wstring info = GetInfo( part );
					Log(2,L"   part: info: %s\n",info.c_str() );
					// If the info is valid...
					if( info != L"" ) {
						//entry.info.push_back(part);
						// Add the long-version to the info list:
						entry.info.push_back(info);
						// We did find info; we'll hve to change the entry.
						foundinfo = true;
					}
			
					if( next == -1 ) {
						break;
					} else {
						stuff = stuff.substr( next+1,-1 );
					}
				}

				if( foundinfo ) {
					changedline = true;
				
					// Take out the info stuff:
					line1 = line1.substr( 0,j ) + line1.substr( end+1,-1 );
					Log(2,L"   line1: %s\n",line1.c_str() );

					// If we started on a space, get rid of it:
					if( line1.size() > 0 && line1[0] == L' ' ) {  line1 = line1.substr(1,-1);  }

					j--;
				}
			}//if( line1[j]=='(' )
		}//for(j)
	// If we found something, take that info out of the display line:
	if( changedline ) {
		entry.meaning[i] = line1;
	}//*/
	}//for(i)


	if( entry.meaning[ entry.meaning.size()-1 ] == L"(P)" ) {
		entry.meaning.pop_back();
		//entry.info.push_back(L"(P)");
		entry.info.push_back(L"One of top 20,000 words");
	}

	Log(1,L"SetInfo(): All done.\n");
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////