// Header.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef HEADER_H
#define HEADER_H
///////////////////////////////////////////////////////////
#include <math.h>
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int Header_GetWidth( HWND hwndList ) {
	int numitems = Header_GetItemCount( hwndList );
	int w = 0;
	for( int i = 0; i < numitems; i++ ) {
		RECT rcHeaderItem;
			Header_GetItemRect( hwndList, i, &rcHeaderItem );
		w += rcHeaderItem.right-rcHeaderItem.left;
	}
	return(w);
}
///////////////////////////////////////////////////////////
//void Header_AddItem( HWND hwndList, TCHAR* text, int w ) {
void Header_AddItem( HWND hwndList, LPTSTR text, int w ) {
	HDITEM hdi;
		hdi.mask = HDI_TEXT | HDI_FORMAT | HDI_WIDTH | HDI_LPARAM;
		hdi.fmt = HDF_LEFT | HDF_STRING;
		hdi.pszText = text;
		hdi.cxy = w;
		hdi.lParam = w;
		hdi.cchTextMax = sizeof(hdi.pszText)/sizeof(hdi.pszText[0]); 
	int result = Header_InsertItem( hwndList, Header_GetItemCount(hwndList)+1, &hdi );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// This scales the header control to a certain width and height.
// NOTE: It assumes that the LPARAM of each item is its
//       percent width of the whole (as a double).
void Header_SetSizeScale( HWND hwndList, int w,int h ) {
	int width = Header_GetWidth(hwndList);
	int numitems = Header_GetItemCount(hwndList);
	int wsofar = 0;
	for( int i = 0; i < numitems; i++ ) {
		HDITEM hdi;
			hdi.mask = HDI_WIDTH|HDI_LPARAM;
		Header_GetItem( hwndList, i, &hdi );
			double percentage = double(hdi.lParam)/100.0;
			hdi.cxy = (int)(percentage*double(w));
			wsofar += hdi.cxy;

			// If it's the last item, fix things up:
			if( i == numitems-1 ) {
				if( wsofar < w ) {
					hdi.cxy += w-wsofar;
				} else if( wsofar > w ) {
					hdi.cxy -= wsofar-w;
				}
			}

		Header_SetItem( hwndList, i, &hdi );
	}
	SetWindowPos( hwndList, 0, 0,0, w,h, SWP_NOZORDER|SWP_NOMOVE|SWP_SHOWWINDOW );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This returns true if there is an item that has an LPARAM
   of 'lparam' in the header 'hwnd'. */
bool Header_IsItemWithLParamInHeader( HWND hwnd, int lParam ) {
	// Get how many items are in the desired header:
	int header_numitems = Header_GetItemCount(hwnd);
	for( int h = 0; h < header_numitems; h++ ) {
		HDITEM header_item;
		ZeroMemory( &header_item, sizeof(header_item) );
			header_item.mask = HDI_LPARAM;
		Header_GetItem( hwnd, h, &header_item );
		
		if( header_item.lParam == lParam ) {
			return(true);
		}
	}
	return(false);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This returns the LPARAM of an item whose zero-based
   index is 'index' in 'hwnd'. */
LPARAM Header_GetLParamOfIndex( HWND hwnd, int index ) {
	HDITEM hditem;
	ZeroMemory( &hditem, sizeof(hditem) );
		hditem.mask = HDI_LPARAM;
	Header_GetItem( hwnd, index, &hditem );
	return( hditem.lParam );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This returns the POSITION of the item ('index') in the
   header bar. */
int Header_GetPositionFromIndex( HWND hwnd, int index ) {
	int numitems = Header_GetItemCount( hwnd );
	if( numitems == 0 ) {  return(-1);  }

	int *array = new int[numitems];
	Header_GetOrderArray( hwnd, numitems, array );
	
	int getpos = -1;
	for( int pos = 0; pos < numitems; pos++ ) {
		if( array[pos] == index ) {
			getpos = pos;
		}
	}

	delete [] array;

	return(getpos);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// This returns the index of the item to the right of this
// one, excluding ones which are hidden.
int Header_GetItemToRight( HWND hwnd, int item ) {
	int numitems = Header_GetItemCount( hwnd );
	if( numitems == 0 ) {  return(-1);  }

	int *array = new int[numitems];
	Header_GetOrderArray( hwnd, numitems, array );
	
	int nextpos = -1;
	for( int pos = 0; pos < numitems; pos++ ) {
		if( array[pos] == item ) {
			if( pos < numitems-1 ) {
				nextpos = array[pos+1];
			}
			break;
		}
	}

	delete [] array;

	return(nextpos);
}
///////////////////////////////////////////////////////////
/*    This returns the zero-based index of the item that is
   to the left of 'item' (the zero-based index of the search). */
int Header_GetItemToLeft( HWND hwnd, int item ) {
	int numitems = Header_GetItemCount( hwnd );
	if( numitems == 0 ) {  return(-1);  }

	int *array = new int[numitems];
	Header_GetOrderArray( hwnd, numitems, array );
	
	int nextpos = -1;
	for( int pos = numitems-1; pos > 0; pos-- ) {
		if( array[pos] == item ) {
			if( pos > 0 ) {
				nextpos = array[pos-1];
			}
			break;
		}
	}

	/*Log(L"item: %i | nextpos: %i\n",item,nextpos );
	Log(L"Array (left): ");
	for( pos = 0; pos < numitems; pos++ ) {
		if( array[pos] == nextpos ) {
			Log(L"|[%i]|",array[pos] );
		} else if( array[pos] == item ) {
			Log(L"|_%i_|",array[pos] );
		} else {
			Log(L"| %i |",array[pos] );
		}
	}
	Log(L"\n");//*/

	delete [] array;

	return(nextpos);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int Header_GetClosestItemWithlParamGreaterThan( HWND hwnd, int item, LPARAM lParam ) {
	int next = item;
	while( next != -1 ) {
		next = Header_GetItemToLeft( hwnd, next );
		if( next != -1 ) {
			HDITEM hditem;
				hditem.mask = HDI_LPARAM;
			Header_GetItem( hwnd, next, &hditem );
			if( hditem.lParam > lParam ) {
				return( next );
			}
		}
	}
	next = item;
	while( next != -1 ) {
		next = Header_GetItemToRight( hwnd, next );
		if( next != -1 ) {
			HDITEM hditem;
				hditem.mask = HDI_LPARAM;
			Header_GetItem( hwnd, next, &hditem );
			if( hditem.lParam > lParam ) {
				return( next );
			}
		}
	}
	return(next);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This removes any items in 'hwnd' that have an LPARAM
   of 'lparam'. */
void Header_RemoveItemFromHeader( HWND hwnd, int lParam ) {
	// Get how many items are in the desired header:
	int numitems = Header_GetItemCount(hwnd);
	for( int h = 0; h < numitems; h++ ) {
		HDITEM header_item;
		ZeroMemory( &header_item, sizeof(header_item) );
			header_item.mask = HDI_LPARAM;
		Header_GetItem( hwnd, h, &header_item );
		
		// If we found it, remove it.
		if( header_item.lParam == lParam ) {
			Header_DeleteItem( hwnd, h );
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This returns the zero-based index IN THAT HWND of the
   item that has an LPARAM of 'lParam'. */
int Header_GetIndexOfItemWithLParamOf( HWND hwnd, int lParam ) {
	// Get how many items are in the desired header:
	int numitems = Header_GetItemCount(hwnd);
	for( int h = 0; h < numitems; h++ ) {
		HDITEM header_item;
		ZeroMemory( &header_item, sizeof(header_item) );
			header_item.mask = HDI_LPARAM;
		Header_GetItem( hwnd, h, &header_item );
		
		// If we found it, remove it.
		if( header_item.lParam == lParam ) {
			return( h );
		}
	}
	return(-1);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
