// KANJIDIC_Sort.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef KANJIDIC_SORT_H
#define KANJIDIC_SORT_H
///////////////////////////////////////////////////////////
#include <Algorithm>

#include "EDICT_Sort.h"
///////////////////////////////////////////////////////////
/* sort_kanji_by[] keeps information on how to actually do the
   sort.  See the SORT_BY structure. */
SORT_BYK sort_kanji_by[2] = { {0,0},{0,0} };
/* sort_on is used by the sort functions to pick what
   attribute to sort on.
   Those functions will use sort_by[sort_on] to do it. */
int sort_kanji_on = PRIMARY;

/* This determines if two entries are equal. */
bool operator==( const KANJIDIC_Entry &a, const KANJIDIC_Entry &b ) {
	switch( sort_kanji_by[sort_kanji_on].window_id ) {
		case WID_KANJI_LIST1: {
			switch( sort_kanji_by[sort_kanji_on].index ) {
				case 0: { // KANJI
					return( a.kanji == b.kanji );
				}
				case 1: { // ON
					if( a.on.size() >= 1 && b.on.size() >= 1 ) {
						return( a.on[0] == b.on[0] );
					} else {
						return( a.on.size() == b.on.size() );
					}
				}
				case 2: { // KUN
					if( a.kun.size() >= 1 && b.kun.size() >= 1 ) {
						return( a.kun[0] == b.kun[0] );
					} else {
						return( a.kun.size() == b.kun.size() );
					}
				}
				case 3: { // STROKE COUNT
					return( a.strokecount == b.strokecount );
					break;
				}
			}
			break;
		}
		case WID_KANJI_LIST2: {
			switch( sort_kanji_by[sort_kanji_on].index ) {
				case 0: { // KANJI (Not used)
					break;
				}
				case 1: { // MEANING
					if( a.meanings.size() >= 1 && b.meanings.size() >= 1 ) {
						return( a.meanings[0] == b.meanings[0] );
					} else {
						Log(L"KANJIDIC_Entry: ==: ERROR: a.meanings.size or b.meanings.size < 1");
						return( a.meanings.size() == b.meanings.size() );
					}
					break;
				}
			}
			break;
		}
	}//*/
	return( a.kanji == b.kanji );
}

/* This uses sort_kanji_by[sort_kanji_on] to determine if one entry
   is 'greater' than another. */
bool operator>( const KANJIDIC_Entry &a, const KANJIDIC_Entry &b ) {
	switch( sort_kanji_by[sort_kanji_on].window_id ) {
		case WID_KANJI_LIST1: {
			switch( sort_kanji_by[sort_kanji_on].index ) {
				case 0: { // KANJI
					if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
						return( a.kanji > b.kanji );
					} else {
						return( a.kanji < b.kanji );
					}
				}
				case 1: { // ON:
					if( a.on.size() >= 1 && b.on.size() >= 1 ) {
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.on[0] > b.on[0] );
						} else {
							return( a.on[0] < b.on[0] );
						}
					} else {
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.on.size() > b.on.size() );
						} else {
							return( a.on.size() < b.on.size() );
						}
					}
				}
				case 2: { // KUN
					if( a.kun.size() >= 1 && b.kun.size() >= 1 ) {
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.kun[0] > b.kun[0] );
						} else {
							return( a.kun[0] < b.kun[0] );
						}
					} else {
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.kun.size() > b.kun.size() );
						} else {
							return( a.kun.size() < b.kun.size() );
						}
					}
				}
				case 3: { // STROKE COUNT:
					if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
						return( a.strokecount > b.strokecount );
					} else {
						return( a.strokecount < b.strokecount );
					}
				}
			}
			break;
		}
		case WID_KANJI_LIST2: {
			switch( sort_kanji_by[sort_kanji_on].index ) {
				case 0: { // KANJI (not used)
					break;
				}
				case 1: { // MEANING
					if( a.meanings.size() >= 1 && b.meanings.size() >= 1 ) {
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.meanings[0] > b.meanings[0] );
						} else {
							return( a.meanings[0] < b.meanings[0] );
						}
					} else {
						Log(L"KANJIDIC_Entry: >: ERROR: a.meanings.size or b.meanings.size < 1");
						if( sort_kanji_by[sort_kanji_on].sortstyle == ASCENDING ) {
							return( a.meanings.size() > b.meanings.size() );
						} else {
							return( a.meanings.size() < b.meanings.size() );
						}
					}
					break;
				}
			}
			break;
		}
	}//*/
	return( a.kanji > b.kanji );
}

/* If it's not greater than an entry and it's not equal
   to it, then it must be less than it. */
bool operator<( const KANJIDIC_Entry &a, const KANJIDIC_Entry &b ) {
	return( !(a > b) && !(a == b) );
}

/* Sort the results by the primary and secondary sorts.
   If there is no secondary, it skips it.
	If there is no primary, it skips it.
//*/
void SortKanjiResults() {
#ifdef UBERDEBUG
	Log(L"(Not sorting KANJIDIC results)\n");
	return;
#endif
	if( kanjidic_results.size() > 0 ) {
		// If there is a secondary way to sort, do it:
		if( sort_kanji_by[SECONDARY].window_id != 0 ) {
			sort_kanji_on = SECONDARY;
				sort( kanjidic_results.begin(), kanjidic_results.end() );
		}
		// Sort by the primary:
		if( sort_kanji_by[PRIMARY].window_id != 0 ) {
			sort_kanji_on = PRIMARY;
				sort( kanjidic_results.begin(), kanjidic_results.end() );
			RedrawWindow( hwnd_kanjiwindow, NULL, NULL, RDW_INVALIDATE );
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
