// Main_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H
///////////////////////////////////////////////////////////
#include "Dialog_Options.h"
#include "Dialog_AddDictionary.h"
///////////////////////////////////////////////////////////
enum {
	MI_RESIZE_NONE = 0,
	MI_RESIZE_NS,
	MI_RESIZE_S_EW,
	MI_RESIZE_S_W_NS
};
///////////////////////////////////////////////////////////
#include "Divider.h"

Divider div_main;
///////////////////////////////////////////////////////////
#include "Update.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Main_Window( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {\
	// These are used for resizing the divider bars:
	static int mouseinstatus = 0;
	static POINT ptstart;
	static POINT ptlast;

	// These are the percentages of the status bar's part's sizes:
	static double part_percent[2] = {  0.90, 0.10  };

	// The next window in the clipboard chain:
	static HWND hwnd_cbnext = NULL;
   
	// Okay, switch the message:
   switch( message ) {
//////////////////////////////////////////
///                                    ///
///   W I N D O W    C R E A T I O N   ///
///                                    ///
//////////////////////////////////////////
		case WM_CREATE: {
			Log(1,L"Main Window: WM_CREATE:\n");

			RECT rc;
				GetClientRect( hWnd, &rc );

			// Make the status bar at the bottom:
			hwnd_main_status = CreateWindowEx( 
		      0L, // no extended styles
		      STATUSCLASSNAME, // status bar
	         TEXT(""), // text 
				WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS, // styles
				rc.bottom-10,0, // (x,y)
				rc.right,10, // (w,h)
			   hWnd, // parent window
		      (HMENU)100, // window ID
	         g_hinst, // instance
				NULL // window data
			);
			// Make sure that the window was created:
			if( !hwnd_main_status ) {
				Log(L"ERROR: Status window was not made.\n");
				MessageBox(NULL,L"Status window was not made.",L"Error",MB_OK );
				exit(-1);
			}
			// Get the rectangle for the status bar at the bottom:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );

			// Split the status bar up:
			int parts[2] = {  double(rcStatus.right)*part_percent[0], -1  };
			SendMessage( hwnd_main_status, SB_SETPARTS, 2, (LPARAM)parts );


			// Resize the client area to account for the status window:
			rc.bottom -= (rcStatus.bottom - rcStatus.top)+1;


			// Okay, now, create the rest of the windows that we'll be using:
			hwnd_searchwindow = CreateSearch_Window( hWnd, 0,0, 800,100 );
			hwnd_kanjiwindow = CreateKanji_Window( hWnd, 0,0, 100,100 );
			hwnd_resultwindow = CreateResult_Window( hWnd, 0,0, 100,100 );
			hwnd_radicallookup = CreateRadicalLookup_Window( hWnd, 0,0, 100,100 );


			// Set the position of the resize-bars:
			int vert = 145, vert_bottom_horiz = 220;
			if( registry.GetBOOL(".\\Window\\RememberSize") ) {
				vert = registry.GetINT(".\\Window\\Div_vert");
				vert_bottom_horiz = registry.GetINT(".\\Window\\Div_vert_bottom_horiz");

				if( vert < 0 || vert > rc.bottom-rc.top ||
					 vert_bottom_horiz < 0 || vert_bottom_horiz > rc.right-rc.left ) {
					MessageBox(NULL,L"The Divider settings were incorrect.\n\nThey have been restored to the defaults.",L"Warning:",MB_OK );
					Log(L"WARNING: The Divider settings were incorrect--They have been restored to the defaults.");
					vert = 145; vert_bottom_horiz = 220;
				}
			}

			// Set the bounds for the dividers:
			div_main.SetRECT( rc.left,rc.top, rc.right-rc.left, rc.bottom-rc.top );
			// Divide it in half vertically:
			div_main.Divide( DIV_VERTICAL, vert );
				// Set the top half to the search window:
				div_main.GetHalf(DIV_TOP)->SetWindow( hwnd_searchwindow );
				// Divide the bottom half horizontally:
				div_main.GetHalf(DIV_BOTTOM)->Divide( DIV_HORIZONTAL, vert_bottom_horiz );
					// Set the left half to the kanji window:
					div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->SetWindow( hwnd_kanjiwindow );
					// Set the right half to the results window:
					div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_RIGHT)->SetWindow( hwnd_resultwindow );


			// Get into the clipboard-viewer chain.
			hwnd_cbnext = SetClipboardViewer( hWnd );

			Log(1,L"END: Main Window: WM_CREATE\n");
			return(0);
		}
////////////////////////////////////////////////
///                                          ///
///   C L I P B O A R D    M E S S A G E S   ///
///                                          ///
////////////////////////////////////////////////
		case WM_DRAWCLIPBOARD: {
		/* The contents of the clipboard have changed. */
			static bool firsttime = true;
			if( search_lock ) {  return(0);  }
			if( !firsttime && !thread_searching && registry.GetBOOL(".\\Misc\\GrabClipboard") ) {
				SendMessage( hwnd_searchwindow, WM_COMMAND, MAKEWPARAM(ID_ENCODE_PASTE,0), 0 );

				// Auto-search?
				if( registry.GetBOOL(".\\Misc\\GrabClipboard-AutoSearch") ) {
					bool fix = false;
					if( search_conversion_romaji == 1 ) {
						search_conversion_romaji = 0;
						fix = true;
					}
					SendMessage( hwnd_searchwindow, WM_COMMAND, MAKEWPARAM(ID_SEARCH_GO,0), 0 );
					if( fix ) {
						search_conversion_romaji = 1;
					}
				}
			}
			firsttime = false;

			// Send it to the next one:
			SendMessage( hwnd_cbnext, WM_DRAWCLIPBOARD, wParam, lParam );
			return(0);
		}
		case WM_CHANGECBCHAIN: {
			// If the next window is closing, repair the chain. 
         if( (HWND)wParam == hwnd_cbnext ) {
				hwnd_cbnext = (HWND)lParam;
			}
         // Otherwise, pass the message to the next link. 
         else if( hwnd_cbnext != NULL ) {
				SendMessage( hwnd_cbnext, WM_CHANGECBCHAIN, wParam, lParam );
			}
			return(0);
		}// case WM_CHANGECBCHAIN
//////////////////////////////////////
///                                ///
///   S I Z E    M E S S A G E S   ///
///                                ///
//////////////////////////////////////
		case WM_SIZE: {
		/* WM_SIZE is used to:
			1) Resize the status window.
			2) Resize the windows controlled by the Divider.
		*/
			// These values apply to the CLIENT AREA:
			int neww = LOWORD(lParam);
			int newh = HIWORD(lParam);

			// Fix the status window automatically:
			SendMessage( hwnd_main_status, WM_SIZE, wParam,lParam );

			int parts[2] = {  double(neww)*part_percent[0], -1  };
			SendMessage( hwnd_main_status, SB_SETPARTS, 2, (LPARAM)parts );

			// Get the modified sizes:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );
			newh -= (rcStatus.bottom - rcStatus.top)+1;

			// Resize the dividers.
			// BUT!
			// If the window is minimized, do NOT resize them.
			WINDOWPLACEMENT wp;
				wp.length = sizeof(WINDOWPLACEMENT);
			GetWindowPlacement( hWnd, &wp );
			if( wp.showCmd&SW_SHOWMAXIMIZED == SW_SHOWMAXIMIZED || wp.showCmd&SW_SHOWNORMAL == SW_SHOWNORMAL ) {
				// Resize the dividers:
				RECT rcMain = div_main.GetRECT();
				div_main.SetRECT( rcMain.left,rcMain.top, neww,newh );
			}
			return(0);
		}
		case WM_SIZING: {
			// Get the new dimentions:
			RECT *rcsizing = (LPRECT)lParam;
				int neww = rcsizing->right;
				int newh = rcsizing->bottom;

			// Get the modified sizes:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );
			RECT rc;
				GetClientRect( hWnd, &rc );
			neww = rc.right;
			newh = rc.bottom;
				newh -= (rcStatus.bottom - rcStatus.top)+1;

			// Resize the divider windows:
			RECT rcMain = div_main.GetRECT();
			div_main.SetRECT( rcMain.left,rcMain.top, neww,newh );
			return(0);
		}
////////////////////////////////////////
///                                  ///
///   M O U S E    M E S S A G E S   ///
///                                  ///
////////////////////////////////////////
		case WM_MOUSEMOVE: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// Set the cursor if we're over a resize bar:
			int result = div_main.SetWindowCursor( mx,my );
			if( result != 0 ) {
				// If we're NOT resizing, then set it to the arrow.
				if( mouseinstatus == 0 ) {
					SetCursor( cursor_arrow );
				}
			}

			// If we're not dragging, then just forget it:
			if( mouseinstatus == 0 ) {  return(0);  }

			// Where the mouse is now:
			POINT pt;
				pt.x = mx; pt.y = my;
			// Get the change in position:
			int dx = pt.x-ptlast.x, dy = pt.y-ptlast.y;

			if( mouseinstatus == MI_RESIZE_NS ) {
				int result = div_main.MoveBar( dy );
				if( result != 0 ) {
					Log(L"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			} else if( mouseinstatus == MI_RESIZE_S_EW ) {
				int result = div_main.GetHalf(DIV_BOTTOM)->MoveBar( dx );
				if( result != 0 ) {
					Log(L"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			} else if( mouseinstatus == MI_RESIZE_S_W_NS ) {
				int result = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->MoveBar( dy );
				if( result != 0 ) {
					Log(L"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			}
			ptlast = pt;
			return(0);
		}
		case WM_MOUSELEAVE: {
			// Reset the mouse status when it enters a child window:
			mouseinstatus = 0;
			SetCursor( LoadCursor(NULL,IDC_ARROW) );
			return(0);
		}
		case WM_LBUTTONDOWN: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);
			POINT pt;
				pt.x = mx; pt.y = my;
			// If there's not resizing going on:
			if( mouseinstatus == 0 ) {
				RECT rcBar;
				// If it's in the vertical bar:
				rcBar = div_main.GetDividerBarRECT();
					rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
				if( PtInRect( &rcBar, pt ) ) {
					mouseinstatus = MI_RESIZE_NS;
						//fprintf(g_flog,"   (In the NS bar...)\n");
				} else {
				// If it's in the horizontal bar:
					rcBar = div_main.GetHalf(DIV_BOTTOM)->GetDividerBarRECT();
						rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
					if( PtInRect( &rcBar, pt ) ) {
						mouseinstatus = MI_RESIZE_S_EW;
							//fprintf(g_flog,"   (In the S_EW bar...)\n");
					} else if( radicallookup_active == 1 ) {
					// If it's in the vertical bar in the horizontal bar:
						rcBar = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->GetDividerBarRECT();
							rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
						if( PtInRect( &rcBar, pt ) ) {
							mouseinstatus = MI_RESIZE_S_W_NS;
								//fprintf(g_flog,"   (In the S_W_NS bar...)\n");
						}
					}
				}
				
				// If he clicked something to resize:
				if( mouseinstatus != 0 ) {
					// Set the start point and last point (for delta):
					ptstart.x = mx; ptstart.y = my;

					ptlast.x = mx; ptlast.y = my;

					SetCapture( hWnd );

					TRACKMOUSEEVENT e = { sizeof(TRACKMOUSEEVENT), TME_LEAVE, hWnd, 0 };
					_TrackMouseEvent( &e );
				}
			}
			return(0);
		}
		case WM_LBUTTONUP: {
			int x = LOWORD(lParam), y = HIWORD(lParam);
			// If we were dragging, it's done now, so clean up:
			if( mouseinstatus != 0 ) {
				mouseinstatus = 0;
				SetCapture(NULL);
				SetCursor( LoadCursor(NULL,IDC_ARROW) );
			}
			return(0);
		}
////////////////////////////////////////////
///                                      ///
///   C O M M A N D    M E S S A G E S   ///
///                                      ///
////////////////////////////////////////////
		case WM_COMMAND: {
			/* wParam
					The high-order word specifies the notification code if the message is from a control.
					If the message is from an accelerator, this value is 1.
					If the message is from a menu, this value is zero. 
					
					The low-order word specifies the identifier of the menu item, control, or accelerator. 

				lParam
					Handle to the control sending the message if the message is from a control. Otherwise, this parameter is NULL. 
			//*/
			if( hmenu_dictionaries != NULL ) {
				switch( LOWORD(wParam) ) {
					case 0:
					case DM_DONE: {
						break;
					} 
					default: {
						if( LOWORD(wParam) >= DM_DSTART ) {
							int dict = LOWORD(wParam)-DM_DSTART;
							edict_dictionaries[dict]->on ^= 1;
							wParam = 0;
							lParam = 0;
						}
						//MessageBox(hWnd,L"default:",L"!",MB_OK );
					}
				}
				return(0);
			}
			switch( LOWORD(wParam) ) {			
				case ID_FILE_EXIT: {
					SendMessage( hWnd, WM_CLOSE, 0,0 );
					break;
				}
				case ID_ADD_DICT: {
				// Add a dictionary:

					if( thread_searching || search_lock ) {
						MessageBox(hWnd,L"Please finish any searches or dictionary-loads before loading a dictionary.\n",L"Note",MB_OK );
						break;
					}

					Log(1,L"ID_ADD_DICT: About to load the load-dictionary dialog...\n");
					CreateDialog_AddDictionary( hWnd );
					Log(1,L"ID_ADD_DICT: load-dictionary dialog ended.\n");
					break;
				}
				case ID_RADICALLOOKUP: {
				// Turn on/off the radical lookup engine:

					// If the radical information hasn't been loaded, yet, stop.
					if( !radical_info ) {  return(0);  }

					radicallookup_active ^= 1;

					HMENU hmenu = GetMenu( hWnd );
					if( radicallookup_active == 1 ) {
					// If Radical Lookup was just turned on:
						CheckMenuItem( hmenu, ID_RADICALLOOKUP, MF_BYCOMMAND|MF_CHECKED );

						// Get the divider of the kanji window:
						// REMEMBER: the 'right' and 'bottom' values of the Divider
						//    rectangles are 'width' and 'height'.
						Divider *div_S_W = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT);
						// Divide the kanji window (bottom 1/3 is the radical window).
						RECT rcDiv = div_S_W->GetRECT();
						div_S_W->Divide( DIV_VERTICAL, (rcDiv.bottom)*2/3 );
						div_S_W->GetHalf(DIV_TOP)->SetWindow( hwnd_kanjiwindow );
						div_S_W->GetHalf(DIV_BOTTOM)->SetWindow( hwnd_radicallookup );
					} else {
					// If Radical Lookup was just turned off:
						CheckMenuItem( hmenu, ID_RADICALLOOKUP, MF_BYCOMMAND|MF_UNCHECKED );

						// Get the divider of the kanji window:
						Divider *div_S_E = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT);
						div_S_E->Divide( DIV_NONE, 0 );
						div_S_E->SetWindow( hwnd_kanjiwindow );
						ShowWindow( hwnd_radicallookup, SW_HIDE );
					}
					return(0);
				}
				case ID_DICTIONARIES: {
					RECT rcWindow;
						GetWindowRect( hWnd, &rcWindow );
					POINT pt;
						GetCursorPos( &pt );
					hmenu_dictionaries = CreateDictionaryMenu( hWnd );
					int result = TrackPopupMenuEx( hmenu_dictionaries, TPM_LEFTALIGN|TPM_LEFTBUTTON, pt.x,pt.y, hWnd, NULL );
						DestroyMenu( hmenu_dictionaries );
						hmenu_dictionaries = NULL;
					return(0);
				}
				case ID_OPTIONS_SETTINGS: {
					CreateDialog_Options( hWnd );
					break;
				}
				case ID_OPTIONS_ALWAYSONTOP: {
					// ontop is true if it has JUST been decided that it will always be on top.
					bool ontop = !registry.GetBOOL(".\\Misc\\AlwaysOnTop");
					registry.SetBOOL(".\\Misc\\AlwaysOnTop",ontop );

					HMENU hmenu = GetMenu( hWnd );
					if( ontop ) {
						CheckMenuItem( hmenu, ID_OPTIONS_ALWAYSONTOP, MF_BYCOMMAND|MF_CHECKED );
					} else {
						CheckMenuItem( hmenu, ID_OPTIONS_ALWAYSONTOP, MF_BYCOMMAND );
					}
					break;
				}
				case ID_OPTIONS_CHECKFORUPDATES: {
					// Do NOT do this if a thread is going:
					if( thread_searching ) {
						MessageBox(hWnd,L"Please wait for any searches or dictionary-loadings to complete\nbefore attempting to update.",L"Note",MB_OK );
						break;
					}
					Update();
					break;
				}
				case ID_HELP_ABOUT: {
					if( window_about_open == false ) {
						// Get the area that a full-screen window can use:
						RECT rcDesk;
							SystemParametersInfo( SPI_GETWORKAREA, 0, &rcDesk, 0 );
						int w = 400, h = 400;
						int x = (rcDesk.right-rcDesk.left)/2-w/2, y = (rcDesk.bottom-rcDesk.top)/2-h/2;

						HWND hwnd_about = CreateWindow_About( hWnd, x,y, w,h );
						ShowWindow( hwnd_about, SW_SHOW );
					}
					break;
				}
			}//*/
			return(0);
		}
		case WM_MENURBUTTONUP: {
		/*	wParam
				Specifies the position of the item when the mouse was released. 
			lParam
				Handle to the menu containing the item.//*/
			if( lParam == (LPARAM)hmenu_dictionaries ) {
				int id = GetMenuItemID( hmenu_dictionaries, wParam );
				if( id >= DM_DSTART && id <= DM_DSTOP ) {
					int state = GetMenuState( hmenu_dictionaries, wParam, MF_BYPOSITION );
					if( state&MF_CHECKED ) {
						CheckMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION );
						edict_dictionaries[id-DM_DSTART]->on = false;
					} else {
						CheckMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_CHECKED );
						edict_dictionaries[id-DM_DSTART]->on = true;
					}
					EnableMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_GRAYED );
					EnableMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_ENABLED );


					// Update the registry:
					int dict = id-DM_DSTART;
					registry.CD(".\\Dictionaries\\EDICT");
						char path[35]; sprintf(path,"%i",dict );
						registry.CD( path );
							registry.SetBOOL( "On", edict_dictionaries[dict]->on );
						registry.CD("..");
					registry.CD("..");
				} else if( id == DM_ALL ) {
					for( int d = 0; d < edict_dictionaries.size(); d++ ) {
						CheckMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_CHECKED );
						edict_dictionaries[d]->on = true;
						
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_GRAYED );
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_ENABLED );

						// Update the registry:
						int dict = d;
						registry.CD(".\\Dictionaries\\EDICT");
							char path[35]; sprintf(path,"%i",dict );
							registry.CD( path );
								registry.SetBOOL( "On", edict_dictionaries[dict]->on );
							registry.CD("..");
						registry.CD("..");
					}
				} else if( id == DM_NONE ) {
					for( int d = 0; d < edict_dictionaries.size(); d++ ) {
						CheckMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND );
						edict_dictionaries[d]->on = false;

						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_GRAYED );
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_ENABLED );

						// Update the registry:
						int dict = d;
						registry.CD(".\\Dictionaries\\EDICT");
							char path[35]; sprintf(path,"%i",dict );
							registry.CD( path );
								registry.SetBOOL( "On", edict_dictionaries[dict]->on );
							registry.CD("..");
						registry.CD("..");
					}
				}
			}
			return(0);
		}
//////////////////////////////////////////////////
///                                            ///
///   A C T I V A T I O N    M E S S A G E S   ///
///                                            ///
//////////////////////////////////////////////////
		case WM_ACTIVATEAPP: {
		/* wParam
				Specifies whether the window is being activated or deactivated. This parameter
				is TRUE if the window is being activated; it is FALSE if the window is being deactivated.
			lParam
				Specifies a thread identifier (a DWORD). If the wParam parameter is TRUE, lParam is
				the identifier of the thread that owns the window being deactivated. If wParam is FALSE,
				lParam is the identifier of the thread that owns the window being activated.
		*/
			if( registry.GetBOOL(".\\Misc\\AlwaysOnTop") ) {
				SetWindowPos( hWnd, HWND_TOPMOST, 0,0, 0,0, SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOSIZE );
				return(0);
			}
			SetWindowPos( hWnd, HWND_NOTOPMOST, 0,0, 0,0, SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOSIZE );
			break;
		}// WM_ACTIVATEAPP
//////////////////////////////////////
///                                ///
///   K I L L    M E S S A G E S   ///
///                                ///
//////////////////////////////////////
		case WM_CLOSE:
			SetWindowText( hWnd, L"Closing..." );
    	case WM_QUIT:
    	case WM_DESTROY: {  // message: window being destroyed
			// Make sure that the clipboard chain is fixed:
			ChangeClipboardChain( hWnd, hwnd_cbnext );
         PostQuitMessage(0);
         return(0); // ~"yatta!"
      }
   }
   return( DefWindowProc(hWnd, message, wParam, lParam) );
} // WndProc
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateMain_Window( HWND hwndParent, int x,int y, int w,int h, int show = SW_SHOW ) {
   WNDCLASS wndClass;
	   wndClass.style          = CS_HREDRAW | CS_VREDRAW; //CS_SAVEBITS;
		wndClass.lpfnWndProc    = WndProc_Main_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = LoadIcon( g_hinst,MAKEINTRESOURCE(IDI_PROGRAM) ); //LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = //(HBRUSH)GetStockObject(WHITE_BRUSH);
										  //(HBRUSH)GetStockObject(BLACK_BRUSH);
										  //(HBRUSH)GetStockObject(LTGRAY_BRUSH);
										  GetSysColorBrush(COLOR_MENU);
										  //GetSysColorBrush(COLOR_WINDOW);
	   wndClass.lpszMenuName   = MAKEINTRESOURCE(IDR_MENU_MAIN); //NULL;
		wndClass.lpszClassName  = TEXT("JLookUp");
   
	if( RegisterClass(&wndClass) == 0 ) {
		MessageBox(NULL,L"Error registering window class.",L"Error",MB_OK );
		exit(-1);
	}

	Uint16 strwindow[1024];
		swprintf(strwindow,L"JLookUp %s",version_info );
   
   HWND hWnd = CreateWindowEx(
		0,
      TEXT("JLookUp"), // window class name
		strwindow, // Window title
      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, // window style
      x,y,
		w,h,
      hwndParent, // parent window handle
      NULL, // window menu handle
      g_hinst, // program instance handle
      NULL // creation parameters
	);

	if( !hWnd ) {
		MessageBox(NULL,L"Error opening the main window.",L"Error",MB_OK );
		exit(-1);
	}
	  
   ShowWindow( hWnd, show );
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////