// Radical.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef RADICAL_H
#define RADICAL_H
///////////////////////////////////////////////////////////
enum {
	RAD_NONE = 0,
	RAD_SELECTED
};
///////////////////////////////////////////////////////////
struct RadicalInfo {
	Uint16 radical;
	Uint16 *relatedkanji;
	int relatedkanji_size;
	unsigned char strokes;
	unsigned char image_index;
	Uint16 *meaning;
	
	int status;
};

RadicalInfo *radical_info = NULL;
int radical_info_size = 0;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This returns the number of '$'s in the first line in the file.
   Note: It resets the file-position pointer. */
long CountRadsInFile( FILE *f ) {
	if( !f ) {  return(0);  }

	fseek(f, 0L, 0 );
	long rads = 0;

	int buffer_rad = registry.GetINT(".\\Buffer\\RADKFILE");

	char *line = new char[buffer_rad];
	while( !feof(f) ) {
		fgets( line, buffer_rad-1, f );
			if( feof(f) ) {  break;  }
		if( line[strlen(line)-1] != '\n' ) {
			Log(L"CountRadsInFile: ERROR: Increase the raddat buffer size.\n");
		}
		// Ignore comments:
		if( line[0] == '#' ) {  continue;  }
		if( line[0] == '$' ) {
			rads++;
		}
	}
	delete [] line;
	fseek(f, 0L, 0 );
	return( rads );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int LoadRadicalInfo( wstring fname ) {
	FILE *frad = _wfopen( fname.c_str(), L"rb" );
	if( !frad ) {
		Log(L"ERROR: Could not find Radical Info file:\n   %s\n",fname.c_str() );
		return(-1);
	}

	int rads = CountRadsInFile( frad );
	radical_info = new RadicalInfo[rads];
	radical_info_size = rads;

	int current_rad = -1;
	wstring current_kanji = L"";

	int buffer_rad = registry.GetINT(".\\Buffer\\RADKFILE");

	char *line = new char[buffer_rad];
	while( !feof(frad) ) {
		fgets( line, buffer_rad-1, frad );
		if( feof(frad) ) {
			// AFTER all the kanji, we can add it to the radical info:
			if( current_rad != -1 ) {
				radical_info[current_rad].relatedkanji_size = current_kanji.size()+1;
				radical_info[current_rad].relatedkanji = new Uint16[radical_info[current_rad].relatedkanji_size];
				wcscpy( radical_info[current_rad].relatedkanji, current_kanji.c_str() );

				//Log(L"Radical: %c\nStrokes: %i\nImage Index: %i\nEnglish: %s\nKanji: %s\n\n",radical_info[current_rad].radical,radical_info[current_rad].strokes,radical_info[current_rad].image_index,radical_info[current_rad].meaning, radical_info[current_rad].relatedkanji );

				current_kanji = L"";
			}

			// File's done; bye-bye:
			break;
		}
		// Ignore comments:
		if( line[0] == '#' ) {  continue;  }
		// Get rid of the endline:
		if( line[strlen(line)-1] == '\n' ) {  line[strlen(line)-1] = '\0';  }
		if( line[strlen(line)-1] == '\r' ) {  line[strlen(line)-1] = '\0';  }

		// If the line is the start of a radical info entry:
		if( line[0] == '$' ) {
			// The kanji for the PREVIOUS radical:
			if( current_rad != -1 ) {
				radical_info[current_rad].relatedkanji_size = current_kanji.size()+1;
				radical_info[current_rad].relatedkanji = new Uint16[radical_info[current_rad].relatedkanji_size];
				wcscpy( radical_info[current_rad].relatedkanji, current_kanji.c_str() );

				//Log(L"Radical: %c\nStrokes: %i\nImage Index: %i\nEnglish: %s\nKanji: %s\n\n",radical_info[current_rad].radical,radical_info[current_rad].strokes,radical_info[current_rad].image_index,radical_info[current_rad].meaning, radical_info[current_rad].relatedkanji );

				current_kanji = L"";
			}

			current_rad++;
			wstring uline = L"";
			int status = ConvertEncoding( line,strlen(line), "SHIFT_JIS","UTF-16LE", uline );
			if( status == -1 ) {
				Log(L"LoadRadicalInfo(): ERROR: ConvertEncoding() returned -1.\n");
				continue;
			}
			//Log(L"Radical data: (%s)\n",uline.c_str() );

			// Parse the string:
				// Note: [0] is '$' and [1] is ' '.
			// The Radical:
			radical_info[current_rad].radical = uline[2];
				// Note: [3] is ' '.
			uline = uline.substr( 4,-1 );
			// The Stroke Count:
			int pos = uline.find( L' ');
			radical_info[current_rad].strokes = _wtoi( uline.substr(0,pos).c_str() );
			uline = uline.substr( pos+1,-1 );
				// Note: [5] is ' '.
			// The Image Index:
			pos = uline.find( L' ' );
			radical_info[current_rad].image_index = _wtoi( uline.substr(0,pos).c_str() );
			uline = uline.substr( pos+1,-1 );
				// Note: [7] is ' '.
			// The English meaning:
			radical_info[current_rad].meaning = new Uint16[uline.size()+1];
			wcscpy( radical_info[current_rad].meaning, uline.c_str() );
		} else {
			wstring uline;
			int status = ConvertEncoding( line,strlen(line), "SHIFT_JIS","UTF-16LE", uline );
			if( status == -1 ) {
				Log(L"LoadRadicalInfo: ERROR: ConvertEncoding() returned -1.\n");
				continue;
			}
			//Log(L"Read line: (%s)\n",uline.c_str() );

			current_kanji += uline;
		}
	}
	delete [] line;


	fclose( frad );
	return(0);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
