// Registry.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef REGISTRY_H
#define REGISTRY_H
///////////////////////////////////////////////////////////
/* Disable the following warning:
	warning C4800: 'void *' : forcing value to bool 'true' or 'false' (performance warning). */
//#pragma warning(disable: 4800)
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
enum {
	TYPE_STRING, TYPE_INT, TYPE_BOOL
};
struct OptionData {
	// Every one has one of these:
	char *name;
	int int_type;
	char *char_type;
	// It only uses ONE of these:
	char *char_value;
	int int_value;
	bool bool_value;
};

struct OptionKey {
	OptionKey *parent;
	char *path;
	vector<OptionKey*> keys;
	vector<OptionData*> data;
	vector<string> comments;
};
///////////////////////////////////////////////////////////
const int REGISTRY_BUFFER = 5000;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int StringPosInVector( char* str, vector<OptionKey*> &v ) {
	for( int i = 0; i < v.size(); i++ ) {
		if( strcmp(v[i]->path,str) == 0 ) {
			return(i);
		}
	}
	return( -1 );
}
int StringPosInVector( char* str, vector<OptionData*> &v ) {
	for( int i = 0; i < v.size(); i++ ) {
		if( strcmp(v[i]->name,str) == 0 ) {
			return(i);
		}
	}
	return( -1 );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//		This is used for printing to the registry's outfile.
// If this is not defined, then the file isn't made.
//#define DEBUG_REGISTRY
///////////////////////////////////////////////////////////
class Registry {
public:
	Registry() {
		line_parts_size = 0;
		reg_root = new OptionKey;
			reg_root->parent = NULL;
			reg_root->path = NULL;
		reg_key_current = reg_root;
#ifdef DEBUG_REGISTRY
		freg = fopen("out.registry.txt","wt");
#endif
	}
	~Registry() {
	}

	bool LoadFromFile( char *path ) {
		FILE *f = fopen( path, "rt" );
		if( f ) {
			Load( f );
			fclose( f );
			return(true);
		}
		return(false);
	}

	void Load( FILE *file ) {
		if( !file ) {  return;  }

		while( !feof(file) ) {
			char line[REGISTRY_BUFFER];
			fgets( line, REGISTRY_BUFFER-1, file );
			if( feof(file) ) {  break;  }

			// If it's a comment, skip it:
			if( line[0] == '#' ) {
				reg_key_current->comments.push_back( line );
				continue;
			}
			// Otherwise, get rid of the end-line stuff:
			if( line[strlen(line)-1] == '\n' ) {  line[strlen(line)-1] = '\0';  }
			if( line[strlen(line)-1] == '\r' ) {  line[strlen(line)-1] = '\0';  }

			//fprintf(g_flog,"LINE:\n<<%s>>\nPARTS:\n",line );

			// Separate the line into its pieces and get rid of any whitespace between them:
			SeparateLine( line );

			//for( int i = 0; i < line_parts_size; i++ ) {  fprintf(g_flog,"<<%s>>\n",line_parts[i]);  }

			if( line_parts_size == 1 ) {
				if( strcmp(line_parts[0],"KEY_STOP") == 0 ) {
					return;
				}
			} else if( line_parts_size == 2 ) {
				if( strcmp(line_parts[0],"KEY_START") == 0 ) {
				//	This is the start of a key (a folder).
				//	1) Make the key.
				//	2) Switch to the key.
				//	3) Recurse this.
				//	4) Go back a level and keep going.

					OptionKey *key = new OptionKey;
						key->parent = reg_key_current;
						key->path = new char[strlen(line_parts[1])+1];
							strcpy(key->path,line_parts[1] );
					reg_key_current->keys.push_back( key );

					CD( key->path );
						Load( file );
					CD( ".." );
				}
			} else if( line_parts_size == 3 ) {
				OptionData *optiondata = new OptionData;
					// Set the type:
					optiondata->char_type = new char[strlen(line_parts[0])+1];
						strcpy(optiondata->char_type,line_parts[0] );
					// Set the name:
					optiondata->name = new char[strlen(line_parts[1])+1];
						strcpy(optiondata->name,line_parts[1] );

					// (Defaults):
					optiondata->char_value = NULL;
					optiondata->int_value = 0;
					optiondata->bool_value = false;

					if( strcmp(optiondata->char_type,"STRING") == 0 ) {
						optiondata->int_type = TYPE_STRING;
						optiondata->char_value = new char[strlen(line_parts[2])+1];
							strcpy(optiondata->char_value,line_parts[2] );
					} else if( strcmp(optiondata->char_type,"INT") == 0 ) {
						optiondata->int_type = TYPE_INT;
						optiondata->int_value = atoi( line_parts[2] );
					} else if( strcmp(optiondata->char_type,"BOOL") == 0 ) {
						optiondata->int_type = TYPE_BOOL;
						optiondata->bool_value = (strcmp(line_parts[2],"TRUE")==0)?true:false;
					}
				reg_key_current->data.push_back( optiondata );
			}
		}
	}

	void CD( char *path ) {
		if( strcmp(path,".") == 0 ) {
		// If it's 'go back to the top'...
			while( reg_key_current->parent != NULL ) {
				reg_key_current = reg_key_current->parent;
			}
		} else if( strcmp(path,"..") == 0 ) {
		// If it's 'go up a level'...
			if( reg_key_current->parent != NULL ) {
				reg_key_current = reg_key_current->parent;
			}
		} else {
			char *next = strstr( path,"\\" );
			if( next == NULL ) {
				for( int i = 0; i < reg_key_current->keys.size(); i++ ) {
					if( strcmp(reg_key_current->keys[i]->path,path) == 0 ) {
						reg_key_current = reg_key_current->keys[i];
						return;
					}
				}

				// The desired key does not exist.
				Log(L"Registry: CD(): ERROR: path '%S' does not exist.\n",path );

				Uint16 msg[1000];
				swprintf(msg,L"\
There is no path by the name '%S' in JLookUp's registry.\n\
\n\
You can add the path by adding the following to Options.jlreg:\n\
     KEY_START :: %S\n\
     KEY_STOP\n\
under the key '%S'.\n\
\n\
The safest option would be to back up your Options.jlreg file and then delete it, so that JLookUp will make a new one.\n\
\n\
JLookUp will now quit to be safe.\
",path,path,GetPathFromHere().c_str() );
				MessageBox(NULL,msg,L"Error",MB_OK );
				exit(-1);

			} else {
				char *path2 = new char[strlen(path)+1];
					strcpy(path2,path);
				char *next2 = path2 + (next-path);
				next2[0] = '\0';
				CD( path2 );
				CD( next2+1 );
				delete [] path2;
			}
		}
	}





	string GetPathFromHere() {
		string path = "";
		OptionKey *key = reg_key_current;
		//while( key ) {
		while( key->parent != NULL ) {
			path = key->path + string("\\") + path;
			key = key->parent;
		}
		return( path );
	}
	string GetPathFrom( OptionKey *key ) {
		string path = "";
		//while( key ) {
		while( key->parent != NULL ) {
			path = key->path + string("\\") + path;
			key = key->parent;
		}
		return( path );
	}


	bool ItemExists( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					return(true);
				}
			}
			return(false);
		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				bool exists = ItemExists( next2+1 );
				delete [] path2;
			CD( ".." );
			return(exists);
		}
		return(false);
	}


	char* GetSTRING( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					if( reg_key_current->data[i]->int_type != TYPE_STRING ) {
						Log(L"ERROR: GetSTRING(): %S\\%S :: type is not string.\n",GetPathFromHere().c_str(),path );
						return(NULL);
					}
					return( reg_key_current->data[i]->char_value );
				}
			}

			// The data does not exist.
			Log(L"ERROR: Registry: GetSTRING(): '%S' not found.\n",path );

			Uint16 msg[1000];
			swprintf(msg,L"\
There is no data by the name '%S' in JLookUp's registry.\n\
\n\
You can add the data by adding the following to Options.jlreg:\n\
     STRING :: %S :: some text\n\
under '%S'.\n\
\n\
The safest option would be to back up your Options.jlreg file and then delete it, so that JLookUp will make a new one.\n\
\n\
JLookUp will now quit to be safe.\
",path,path,GetPathFromHere().c_str() );
			MessageBox(NULL,msg,L"Error",MB_OK );
			Log(L"ERROR:\n%s\n",msg );
			exit(-1);

		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				char *data = GetSTRING( next2+1 );
				delete [] path2;
			CD( ".." );
			return(data);
		}
		return(NULL);
	}
	int GetINT( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					if( reg_key_current->data[i]->int_type != TYPE_INT ) {
#ifdef DEBUG_REGISTRY
						fprintf(freg,"GetINT: type is not int.\n");
#endif
						return(0);
					}
					return( reg_key_current->data[i]->int_value );
				}
			}

			// The data does not exist.
			Log(L"Registry: GetINT(): '%S' not found.\n",path );

			Uint16 msg[1000];
			swprintf(msg,L"\
There is no data by the name '%S' in JLookUp's registry.\n\
\n\
You can add the data by adding the following to Options.jlreg:\n\
     INT :: %S :: number\n\
under '%S'.\n\
\n\
The safest option would be to back up your Options.jlreg file and then delete it, so that JLookUp will make a new one.\n\
\n\
JLookUp will now quit to be safe.\
",path,path,GetPathFromHere().c_str() );
			MessageBox(NULL,msg,L"Error",MB_OK );
			exit(-1);

		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				int data = GetINT( next2+1 );
				delete [] path2;
			CD( ".." );
			return(data);
		}
		return(0);
	}
	bool GetBOOL( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					if( reg_key_current->data[i]->int_type != TYPE_BOOL ) {
#ifdef DEBUG_REGISTRY
						fprintf(freg,"GetBOOL: type is not bool.\n");
#endif
						return(NULL);
					}
					return( reg_key_current->data[i]->bool_value );
				}
			}

			// The data does not exist.
			Log(L"Registry: GetBOOL(): '%S' not found.\n",path );

			Uint16 msg[1000];
			swprintf(msg,L"\
There is no data by the name '%S' in JLookUp's registry.\n\
\n\
You can add the data by adding the following to Options.jlreg:\n\
     BOOL :: %S :: TRUE\n\
or\n\
     BOOL :: %S :: FALSE\n\
under '%S'.\n\
\n\
The safest option would be to back up your Options.jlreg file and then delete it, so that JLookUp will make a new one.\n\
\n\
JLookUp will now quit to be safe.\
",path,path,GetPathFromHere().c_str() );
			MessageBox(NULL,msg,L"Error",MB_OK );
			exit(-1);

		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				bool data = GetBOOL( next2+1 );
				delete [] path2;
			CD( ".." );
			return(data);
		}
		return(false);
	}

	void SetSTRING( char *path, const char *value ) {  SetSTRING( path, (char*)value );  }
	void SetSTRING( char *path, char *value ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			// Find the data value to change:
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					// This is the one that we need to change:
					OptionData *optiondata = reg_key_current->data[i];

					if( optiondata->int_type != TYPE_STRING ) {
#ifdef DEBUG_REGISTRY
						fprintf(freg,"SetSTRING: type is not string.\n");
#endif
						Log(L"Registry: SetSTRING(): type of %S\\%S is not string.\n",GetPathFromHere().c_str(),path );
						return;
					}

					// Clear the value:
					delete [] optiondata->char_value;

					// Now, set the value:
					optiondata->char_value = new char[strlen(value)+1];
						strcpy(optiondata->char_value,value );
					return;
				}
			}
			// If we need to make a new one:
				OptionData *optiondata = new OptionData;
					// Set the type:
					optiondata->char_type = new char[strlen("STRING")+1];
						strcpy(optiondata->char_type,"STRING" );
					// Set the name:
					optiondata->name = new char[strlen(path)+1];
						strcpy(optiondata->name,path );

					// (Defaults):
					optiondata->int_value = 0;
					optiondata->bool_value = false;

					optiondata->int_type = TYPE_STRING;
					optiondata->char_value = new char[strlen(value)+1];
						strcpy(optiondata->char_value,value );
				reg_key_current->data.push_back( optiondata );
			return;
		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				SetSTRING( next2+1, value );
				delete [] path2;
			CD( ".." );
		}
	}
	void SetINT( char *path, int value ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			// Find the data value to change:
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					// This is the one that we need to change:
					OptionData *optiondata = reg_key_current->data[i];

					if( optiondata->int_type != TYPE_INT ) {
#ifdef DEBUG_REGISTRY
						fprintf(freg,"SetINT: type is not int.\n");
#endif
						Log(L"Registry: SetINT(): type of %S\\%S is not int.\n",GetPathFromHere().c_str(),path );
						return;
					}

					// Now, set the value:
					optiondata->int_value = value;
					return;
				}
			}
			// If we need to make a new one:
				OptionData *optiondata = new OptionData;
					// Set the type:
					optiondata->char_type = new char[strlen("INT")+1];
						strcpy(optiondata->char_type,"INT" );
					// Set the name:
					optiondata->name = new char[strlen(path)+1];
						strcpy(optiondata->name,path );

					// (Defaults):
					optiondata->char_value = NULL;
					optiondata->bool_value = false;

					optiondata->int_type = TYPE_INT;
					optiondata->int_value = value;
				reg_key_current->data.push_back( optiondata );
			return;
		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				SetINT( next2+1, value );
				delete [] path2;
			CD( ".." );
		}
	}
	void SetBOOL( char *path, bool value ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			// Find the data value to change:
			for( int i = 0; i < reg_key_current->data.size(); i++ ) {
				if( strcmp(path,reg_key_current->data[i]->name) == 0 ) {
					// This is the one that we need to change:
					OptionData *optiondata = reg_key_current->data[i];

					if( optiondata->int_type != TYPE_BOOL ) { 
#ifdef DEBUG_REGISTRY
						fprintf(freg,"SetBOOL: type is not bool.\n");
#endif
						Log(L"Registry: SetBOOL(): type of %S\\%S is not bool.\n",GetPathFromHere().c_str(),path );
						return;
					}

					// Now, set the value:
					optiondata->bool_value = value;
					return;
				}
			}
			// If we need to make a new one:
				OptionData *optiondata = new OptionData;
					// Set the type:
					optiondata->char_type = new char[strlen("BOOL")+1];
						strcpy(optiondata->char_type,"BOOL" );
					// Set the name:
					optiondata->name = new char[strlen(path)+1];
						strcpy(optiondata->name,path );

					// (Defaults):
					optiondata->char_value = NULL;
					optiondata->int_value = 0;

					optiondata->int_type = TYPE_BOOL;
					optiondata->bool_value = value;
				reg_key_current->data.push_back( optiondata );
			return;
		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				SetBOOL( next2+1, value );
				delete [] path2;
			CD( ".." );
		}
	}

	void NewKey( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			// See if the key already exists:
			for( int i = 0; i < reg_key_current->keys.size(); i++ ) {
				if( strcmp(path,reg_key_current->keys[i]->path) == 0 ) {
#ifdef DEBUG_REGISTRY
					fprintf(freg,"NewKey: key already exists.\n");
#endif
					return;
				}
			}
			// Make the new key:
			OptionKey *key = new OptionKey;
				key->parent = reg_key_current;
				key->path = new char[strlen(path)+1];
					strcpy(key->path,path );
				reg_key_current->keys.push_back( key );
		} else {
#ifdef DEBUG_REGISTRY
			fprintf(freg,"NewKey: A key cannot have '\\'s in the name.\n");
#endif
		}
	}





	void DeleteKey( char *path ) {
		char *next = strstr( path,"\\" );
		if( next == NULL ) {
			for( int i = 0; i < reg_key_current->keys.size(); i++ ) {
				if( strcmp(path,reg_key_current->keys[i]->path) == 0 ) {
					// Delete the path:
					OptionKey *key = reg_key_current->keys[i];
					for( int k = 0; k < key->keys.size(); k++ ) {
						DeleteKey( key->path );
					}
					for( int d = 0; d < key->data.size(); d++ ) {
						OptionData *data = key->data[d];
							delete data->char_type;
							delete data->name;
							if( data->int_type == TYPE_STRING ) {
								delete data->char_value;
							}
						delete data;
					}
					reg_key_current->keys.erase( reg_key_current->keys.begin() + i );
					return;
				}
			}
		} else {
			char *path2 = new char[strlen(path)+1];
				strcpy(path2,path);
			char *next2 = path2 + (next-path);
			next2[0] = '\0';
			CD( path2 );
				DeleteKey( next2+1 );
				delete [] path2;
			CD( ".." );
		}
	}





	void Print( int level=0 ) {

		char *tab = new char[level*3+1];
			for( int t = 0; t < level*3; t++ ) {  tab[t] = ' ';  }
			tab[level*3] = '\0';

			Log(L"%S[%S]\n",tab,reg_key_current->path );

		for( int i = 0; i < reg_key_current->keys.size(); i++ ) {
			CD( reg_key_current->keys[i]->path );
				Print( level+1 );
			CD( ".." );
		}

		delete [] tab;
		tab = new char[(level+1)*3+1];
			for( t = 0; t < (level+1)*3; t++ ) {  tab[t] = ' ';  }
			tab[(level+1)*3] = '\0';

		for( i = 0; i < reg_key_current->data.size(); i++ ) {
			Log(L"%S%S = ",tab,reg_key_current->data[i]->name );
			switch( reg_key_current->data[i]->int_type ) {
				case TYPE_STRING: {
					Log(L"%S\n", reg_key_current->data[i]->char_value );
					break;
				}
				case TYPE_INT: {
					Log(L"%i\n", (int)reg_key_current->data[i]->int_value );
					break;
				}
				case TYPE_BOOL: {
					Log(L"%S\n", (reg_key_current->data[i]->bool_value)?"TRUE":"FALSE" );
					break;
				}
			}
		}

		delete [] tab;
	}


	// Save the registry to a file.
	bool SaveToFile( char *path ) {
		CD(".");
		FILE *file = fopen(path,"wt");
		if( file ) {
			Save( file );
			fclose(file);
			return(true);
		} else {
			return(false);
		}
	}

	// Write the registry to a FILE stream.
	void Save( FILE *file, int level=-1 ) {
		if( reg_key_current->parent ) {
			for( int t = 0; t < level; t++ ) {  fprintf(file,"\t");  }
			fprintf(file,"KEY_START :: %s\n", reg_key_current->path );
		}
			for( int i = 0; i < reg_key_current->comments.size(); i++ ) {
				fprintf(file,"%s",reg_key_current->comments[i].c_str() );
			}
			for( i = 0; i < reg_key_current->keys.size(); i++ ) {
				CD( reg_key_current->keys[i]->path );
				Save( file, level+1 );
				CD( ".." );
			}
			for( i = 0; i < reg_key_current->data.size(); i++ ) {
				for( int t = 0; t < level+1; t++ ) {  fprintf(file,"\t");  }
				fprintf(file,"%s :: %s :: ",reg_key_current->data[i]->char_type,reg_key_current->data[i]->name );
				switch( reg_key_current->data[i]->int_type ) {
					case TYPE_STRING: {
						fprintf(file,"%s\n", reg_key_current->data[i]->char_value );
						break;
					}
					case TYPE_INT: {
						fprintf(file,"%i\n", reg_key_current->data[i]->int_value );
						break;
					}
					case TYPE_BOOL: {
						fprintf(file,"%s\n", (reg_key_current->data[i]->bool_value)?"TRUE":"FALSE" );
						break;
					}
				}
			}
		if( reg_key_current->parent ) {
			for( int t = 0; t < level; t++ ) {  fprintf(file,"\t");  }
			fprintf(file,"KEY_STOP\n");
		}
	}


	// This merges two registries:
	void MergeFrom( Registry &reg ) {
		OptionKey *new_root = reg.reg_root;
		OptionKey *new_current = new_root;

		OptionKey *this_current = reg_root;

		MergyMergy( this_current, reg, new_current );
	}

	// This is the recursive call for MergeFrom().
	void MergyMergy( OptionKey *this_current, Registry &reg, OptionKey *new_current ) {
		for( int i = 0; i < new_current->keys.size(); i++ ) {
			// Get the position of the path:
			int pos = StringPosInVector( new_current->keys[i]->path, this_current->keys );

			OptionKey *this_go = NULL;

			if( pos == -1 ) {
				OptionKey *key = new OptionKey;
					key->parent = this_current;
					key->path = new char[ strlen(new_current->keys[i]->path)+1 ];
					strcpy( key->path, new_current->keys[i]->path );
				this_current->keys.push_back( key );
				this_go = key;
				Log(L"*** Made new key: %S%S\n",GetPathFrom(this_current).c_str(), key->path );
			} else {
				this_go = this_current->keys[pos];
			}
			MergyMergy( this_go, reg, new_current->keys[i] );
		}
		for( i = 0; i < new_current->data.size(); i++ ) {
			int pos = StringPosInVector( new_current->data[i]->name, this_current->data );

			if( pos == -1 ) {
				
				OptionData *data = new OptionData;
				// Every one has one of these:
				data->name = new char[ strlen(new_current->data[i]->name)+1 ];
					strcpy( data->name, new_current->data[i]->name );
				data->int_type = new_current->data[i]->int_type;
				data->char_type = new char[ strlen(new_current->data[i]->char_type)+1 ];
					strcpy( data->char_type, new_current->data[i]->char_type );
				// It only uses ONE of these:
				data->char_value = NULL;
					if( new_current->data[i]->char_value ) {
						data->char_value = new char[ strlen(new_current->data[i]->char_value)+1 ];
							strcpy( data->char_value, new_current->data[i]->char_value );
					}
				data->int_value = new_current->data[i]->int_value;
				data->bool_value = new_current->data[i]->bool_value;

				this_current->data.push_back( data );
				Log(L"*** Made new dat: %S%S\n",GetPathFrom(this_current).c_str(), data->name );

			}
		}
		//for( i = 0; i < new_current->comments.size(); i++ ) {
		//}
	}


private:
	OptionKey *reg_root;
	OptionKey *reg_key_current;

#ifdef DEBUG_REGISTRY
	FILE *freg;
#endif

	char *line_parts[3];
	int line_parts_size;

	void SeparateLine( char *line ) {
		// Delete any old parts there:
		for( int i = 0; i < line_parts_size; i++ ) {
			delete [] line_parts[i];
		}
		line_parts_size = 0;


		// Trim the front of the line:
		while( true ) {
			if( line[0] == ' ' || line[0] == '\t' ) {
				line++;
			} else {
				break;
			}
		}

		// Split time:
		char *searchtext = line;
		while( true ) {
			char *break_start = strstr( searchtext, "::" );
			if( break_start != NULL ) {
				break_start[0] = '\0';
				break_start++;
			/* This means that there is something like this, now:
				text1 :: text2
				'text1 ' is searchtext, and ': text2' is break_start'.
			*/
				// (The front should already be trimmed, so trim the back):
				while( true ) {
					if( searchtext[strlen(searchtext)-1] == ' ' || searchtext[strlen(searchtext)-1] == '\t' ) {
						searchtext[strlen(searchtext)-1] = '\0';
					} else {
						break;
					}
				}

				// Trim the front of the second half:
				break_start++;
				while( true ) {
					if( break_start[0] == ' ' || break_start[0] == '\t' ) {
						break_start++;
					} else {
						break;
					}
				}

				// Make the new string:
				line_parts[line_parts_size] = new char[strlen(searchtext)+1];
				sprintf(line_parts[line_parts_size],"%s",searchtext );

				line_parts_size++;

				searchtext = break_start;
			} else {
				// (The front should already be trimmed, so trim the back):
				while( true ) {
					if( searchtext[strlen(searchtext)-1] == ' ' || searchtext[strlen(searchtext)-1] == '\t' ) {
						searchtext[strlen(searchtext)-1] = '\0';
					} else {
						break;
					}
				}

				// Make the new string:
				line_parts[line_parts_size] = new char[strlen(searchtext)+1];
				sprintf(line_parts[line_parts_size],"%s",searchtext );

				line_parts_size++;
				break;
			}
		}
	}
};
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
