// Result_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef RESULT_WINDOW_H
#define RESULT_WINDOW_H
///////////////////////////////////////////////////////////
// These are for the drawing operations:
int bx = 3, by = 3;
int result_lineh1 = 0;
int result_lineh2 = 0;
int entry_height = 0;

// This marks where to start drawing entries:
int start_entry = 0;

// This is for maintaining the size of the client area:
RECT rcClient_result;

// These are the fonts for various entry parts:
HFONT hfont_result_word;
HFONT hfont_result_kana;
HFONT hfont_result_meaning;
HFONT hfont_result_info;

// These are the colors for various entry parts:
Color color_result_word( 0,0,96 );
Color color_result_kana( 0,0,0 );
Color color_result_info( 255,0,96 );
Color color_result_meaning( 0,0,0 );

// These are the colors for the alternating entry backgrounds:
Color color_result_bg1( 255,255,255 );
Color color_result_bg2( 196,196,196 );

// This is the color of the border around selected objects:
Color color_result_border_outside( 0,0,0 );
// This is for the border within an object:
Color color_result_border_inside( 128,128,128 );

// This is the background color of the whole window:
Color color_result_windowbg( 255,255,255 );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// For the Result Window's headers:
#include "Result_List.h"
// For sorting a vector<EDICT_Entry>:
#include "EDICT_Sort.h"

// For the Result Header dialog box:
#include "Dialog_ResultHeader.h"

// For copying entry info:
#include "Edict_Copy.h"

// For inverting colors:
#include "Invert.h"

// For the tooltip-like boxes:
#include "Result_Hover.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function returns the index of the entry (in
   'edict_results' that currently contains the point (mx,my). */
int ResultWindow_GetEntryFromPoint( int mx, int my ) {
	// Get vertical scroll bar position:
	SCROLLINFO si;
		si.cbSize = sizeof(si);
		si.fMask  = SIF_POS;
	GetScrollInfo( hwnd_resultwindow, SB_VERT, &si );
	
	// This is the top entry on the screen:
	// (GLOBAL)
	start_entry = si.nPos;

	// This is the number of entries that fit on the screen:
	int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
	// This is the very last entry on the screen:
	int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

	// 'return_entry' is the entry that is at point (mx,my) on the screen.
	int return_entry = -1;
	// Go through the entries on the screen:
	for( int e = start_entry; e <= end_entry; e++ ) {
		// 'i' is the current entry converted to a SCREEN ENTRY.
		// An 'i' of 0 means that it is the TOP entry on the screen.
		int i = e-start_entry;

		// If he clicked an item (x-values are irrelevant for now), select it:
		if( my >= rcClient_result.top + i*entry_height &&
			 my <= rcClient_result.top + (i+1)*entry_height-1 ) {
			return_entry = e;
			break;
		}
	}

	return( return_entry );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
enum {
	SELECT_MODE_SINGLE = 0,
	SELECT_MODE_ADD = 1,
	SELECT_MODE_FILL = 2
};
void ResultWindow_SelectEntry( int entry, int mode ) {
	int result_last_old = result_last;

	// The last thing that was clicked is 'entry'.
	result_last = entry;


	// If we're just doing a normal click, it's the base.
	// Also, de-select everything else.
	if( mode==SELECT_MODE_SINGLE ) {
		result_base = entry;
		edict_results[result_base].selected = 1;
		for( int i = 0; i < edict_results.size(); i++ ) {
			if( i != result_base ) {
				edict_results[i].selected = false;
			}
		}
	}
	if( mode&SELECT_MODE_ADD ) {
	// If CTRL is down, the base that we're using is 'entry'.
		result_base = entry;
		edict_results[result_base].selected ^= 1;
	}
	if( mode&SELECT_MODE_FILL ) {
		// Select what's between them.
		if( result_base != -1 && result_last != -1 ) {
			// Unselect the old range:
			for( int i = min(result_base,result_last_old); i <= max(result_base,result_last_old); i++ ) {
				edict_results[i].selected = false;
			}
			// Select the new range:
			for( i = min(result_base,result_last); i <= max(result_base,result_last); i++ ) {
				edict_results[i].selected = true;
			}
		}

	}

	RedrawWindow( hwnd_resultwindow, NULL, NULL, RDW_INVALIDATE );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Result_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	static bool tracking = false;

	static POINT hover_pos;
	static int hover_entry = -1;
	static bool hover_on = false;
	static HWND hwnd_hover = NULL;

	static int last_mx = 0, last_my = 0;

	static HMENU hmenu_result_header = NULL;

	switch( message ) {
//////////////////////////////////////////
///                                    ///
///   W I N D O W    C R E A T I O N   ///
///                                    ///
//////////////////////////////////////////
		case WM_CREATE: {
			Log(1,L"Result Window: WM_CREATE:\n");

			// Create the headers:
			/*hwnd_resultlist1 = CreateResult_List1( hWnd );
			RECT rcList1;
				GetClientRect( hwnd_resultlist1, &rcList1 );
			hwnd_resultlist2 = CreateResult_List2( hWnd, 0,rcList1.bottom );//*/

			CreateResultHeaders( hWnd );

			RECT rcList1;
				GetClientRect( hwnd_resultlist[0], &rcList1 );

			SetWindowPos( hwnd_resultlist[0], 0, 0,0, 0,0, SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW );
			SetWindowPos( hwnd_resultlist[1], 0, 0,rcList1.bottom, 0,0, SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW );
			
			LoadResultHeaderInfo();
			
			SetResultHeaders( hWnd );

			// Set the margins for the bitmaps (in the headers):
			Header_SetBitmapMargin( hwnd_resultlist[0], 0 );
			Header_SetBitmapMargin( hwnd_resultlist[1], 0 );

			Log(1,L"END: Result Window: WM_CREATE\n");
			return(0);
		}
		case WM_SIZE: {
		/* The Result Window has been resized */
			// Retrieve the dimensions of the new client area:
			int yClient = HIWORD(lParam);
			int xClient = LOWORD(lParam);

#ifndef UBERDEBUG
			// Get the new lineh1 and lineh2 values:
			Graphics graphics( GetDC(hWnd) );

				int max_fontheight[2] = { 0, 0 };
				for( int i = 0; i < HDR_RESULT_MAX; i++ ) {
					if( result_header_items[i].hwnd == -1 ) {  continue;  }

					int thisheight = 0;
					switch( i ) {
						case HDR_RESULT_WORD:
						case HDR_RESULT_KANA: {  thisheight = Font( GetDC(hWnd),hfont_result_word ).GetHeight( &graphics );  break;  }
						case HDR_RESULT_INFO: {  thisheight = Font( GetDC(hWnd),hfont_result_word ).GetHeight( &graphics );  break;  }
						case HDR_RESULT_WORD_L:
						case HDR_RESULT_KANA_L:
						case HDR_RESULT_MEANING: {  thisheight = Font( GetDC(hWnd),hfont_result_word ).GetHeight( &graphics );  break;  }
					}

					max_fontheight[result_header_items[i].hwnd] = max( max_fontheight[result_header_items[i].hwnd], thisheight );
				}

				result_lineh1 = by + max_fontheight[0] + by;
				result_lineh2 = by + max_fontheight[1] + by;
			entry_height = result_lineh1 + result_lineh2;
#else
			entry_height = 10;
#endif

			// Fix the windows:
			RECT rcWindow;
				GetClientRect( hWnd, &rcWindow );

			// If the window is smaller than a certain size, leave it that way and just cover it up:
			if( rcWindow.right < 100 ) {  break;  }

			// Otherwise, scale the headers:
			RECT rcList1;
				GetClientRect( hwnd_resultlist[0], &rcList1 );
			//Header_SetSizeScale( hwnd_resultlist1, rcWindow.right,rcList1.bottom );
			Header_ScaleItems( hwnd_resultlist[0], rcWindow.right,rcList1.bottom );
			
			RECT rcList2;
				GetClientRect( hwnd_resultlist[1], &rcList2 );
			//Header_SetSizeScale( hwnd_resultlist2, rcWindow.right,rcList2.bottom );
			Header_ScaleItems( hwnd_resultlist[1], rcWindow.right,rcList2.bottom );


			// Set the client information:
			rcClient_result.left = 0;
			rcClient_result.top = 0 + rcList1.bottom + rcList1.bottom;

			rcClient_result.right = xClient;
			rcClient_result.bottom = yClient;

			//if( entry_height == 0 ) {  MessageBox(hWnd,L"WTF",L"!KJ#L:K",MB_OK );  }
 
			// Set the vertical scrolling range and page size:
			SCROLLINFO si;
				si.cbSize = sizeof(si); 
				si.fMask  = SIF_RANGE | SIF_PAGE;// | SIF_DISABLENOSCROLL; 
				si.nMin   = 0; 
				si.nMax   = edict_results.size() - 1;
				si.nPage  = (rcClient_result.bottom-rcClient_result.top) / (entry_height); 
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE);

			// Recalculate the start based on the new size:
				si.fMask  = SIF_ALL;
         GetScrollInfo( hWnd, SB_VERT, &si );
				start_entry = si.nPos;

			// This makes resizing smooth:
			SendMessage( hWnd, WM_PAINT, 0,0 );
			return( 0 );
		}
///////////////////////////////////
///                             ///
///   N O T I F I C A T I O N   ///
///                             ///
///////////////////////////////////
		case WM_NOTIFY: {
			if( wParam == WID_RESULT_LIST1 || wParam == WID_RESULT_LIST2 ) {
				NMHEADER *header = (NMHEADER*)lParam;
				//fprintf(g_flog,"WM_NOTIFY: WID_RESULT_LIST1: %i (%i)\n",header->hdr.code,HDN_TRACK);
				switch( header->hdr.code ) {
					/////////////////////////////////////
					///                               ///
					///      HEADER BAR MESSAGES      ///
					///                               ///
					/////////////////////////////////////
					case HDN_BEGINTRACKW:
					case HDN_BEGINTRACKA: {
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );

						// If it's on the far right, do NOT let it resize.
						if( Header_GetItemToRight( header->hdr.hwndFrom, header->iItem ) == -1 ) {
							return(TRUE);
						}

						// Note: Just returning TRUE dis-allows the user to drag the dividers.
						return(FALSE);
						break;
					}//*/
					case HDN_TRACKW:
					case HDN_TRACKA: {
						// The width of the header:
						int width = Header_GetWidth( header->hdr.hwndFrom );

						// Here's the item in its original state:
						HDITEM item_original;
							item_original.mask = HDI_LPARAM|HDI_WIDTH;
						Header_GetItem( header->hdr.hwndFrom, header->iItem, &item_original );
		
						// This is the item (in its changed stage):
						HDITEM *item = header->pitem;
							
						// Fix it up.
						// Basically, get the new percent-size of it,
						// and resize it so that it is that percent-size (not so smooth, but valid).
						int percent_old = result_header_items[item_original.lParam].percent;
							// Get the new percent:
							result_header_items[item_original.lParam].percent = (double(item->cxy)/double(width)*100.0);
							// Here's the new width (based on the new percent):
							item->cxy = (int)(double(result_header_items[item_original.lParam].percent)/100.0*double(width));
						int percent_new = result_header_items[item_original.lParam].percent;

						// Now, fix up the item to its right:
						int right = Header_GetItemToRight( header->hdr.hwndFrom, header->iItem );
						if( right == -1 ) {
							MessageBox(hWnd,L"Right is -1",L"Error",MB_OK );
							return(TRUE);
						}

						// This is the one to the right:
						HDITEM item2;
							item2.mask = HDI_LPARAM|HDI_WIDTH;
						Header_GetItem( header->hdr.hwndFrom, right, &item2 );
							//item2.cxy = item2.cxy - (item->cxy-item_original.cxy);
							//result_header_items[item2.lParam].percent = (double)ceil(double(item2.cxy)/double(width)*100.0);
							
							result_header_items[item2.lParam].percent -= (percent_new-percent_old);
							//item2.cxy = (int)(double(result_header_items[item2.lParam].percent)/100.0*double(width));

						// Only resize if the items' sizes are greater than 10:
						if( item2.cxy > 10 && item->cxy > 10 ) {
							// If it actually changed:
							if( item_original.cxy != item->cxy ) {
								//Header_SetItem( header->hdr.hwndFrom, right, &item2 );
								//Header_SetItem( header->hdr.hwndFrom, header->iItem, item );
							
								SetResultHeaders( hWnd );

								RedrawWindow( header->hdr.hwndFrom, NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
							}
							return(FALSE); // keep dragging...
						} else {
							// Okay, we're done.
							RedrawWindow( header->hdr.hwndFrom, NULL, NULL, RDW_INVALIDATE );
							RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
							return(TRUE); // stop dragging.
						}//*/
						break;
					}//*/
					case HDN_ENDTRACKW:
					case HDN_ENDTRACKA: {
						RECT rc; GetClientRect(hWnd,&rc);
						SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom) );

						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/
					// This one definitely is good; don't change it.
					case HDN_ENDDRAG: {
						// Here's the item in its original state:
						HDITEM item_original;
							item_original.mask = HDI_LPARAM|HDI_WIDTH;
						Header_GetItem( header->hdr.hwndFrom, header->iItem, &item_original );

						// Set the new positon:
						result_header_items[item_original.lParam].position = header->pitem->iOrder;
						int position_old = Header_GetPositionFromIndex( header->hdr.hwndFrom, header->iItem );
						int position_new = result_header_items[item_original.lParam].position;
						Log(1,L"Result Window: HDN_ENDDRAG: Old Position: %i\n", position_old );
						Log(1,L"Result Window: HDN_ENDDRAG: New Position: %i\n", position_new );

						Log(1,L"Zero-based index of the item in question: %i\n",header->iItem );
						Log(1,L"   And, its LPARAM: %i\n",item_original.lParam );

						// Update all of the other positions:
						if( position_old < position_new ) {
						// If it moved RIGHT:
							int last_right = header->iItem;
							for( int i = position_old+1; i <= position_new; i++ ) {
								int right = Header_GetItemToRight( header->hdr.hwndFrom, last_right );
								Log(1,L"Item to right (index): %i\n",right );
								if( right == -1 ) {
									Log(L"ERROR: right is -1!\n");
									break;
								}
						
								HDITEM item_right;
								ZeroMemory( &item_right, sizeof(item_right) );
									item_right.mask = HDI_LPARAM;
								Header_GetItem( header->hdr.hwndFrom, right, &item_right );
								result_header_items[item_right.lParam].position--;

								last_right = right;
							}
						} else if( position_old > position_new ) {
						// If it moved LEFT
							int last_left = header->iItem;
							for( int i = position_old-1; i >= position_new; i-- ) {
								int left = Header_GetItemToLeft( header->hdr.hwndFrom, last_left );
								Log(1,L"Item to left (index): %i\n",left );
								if( left == -1 ) {
									Log(L"ERROR: left is -1!\n");
									break;
								}
						
								HDITEM item_left;
								ZeroMemory( &item_left, sizeof(item_left) );
									item_left.mask = HDI_LPARAM;
								Header_GetItem( header->hdr.hwndFrom, left, &item_left );
								result_header_items[item_left.lParam].position++;

								last_left = left;
							}
						}

						SetResultHeaders( hWnd );

						// Repaint the screen when a heading is moved.
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/
					case NM_RCLICK: {
						SendMessage( hWnd,WM_KEYDOWN,VK_SPACE,0 );
						return(TRUE);
						break;
					}//*/
					/*case HDN_ITEMCHANGED: {
						//RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/

					case HDN_ITEMCLICK: {
					/* When an item is clicked, sort by that item.
					   The old PRIMARY becomes the new SECONDARY. */

						// This is the index of 'result_header_items[]' that he clicked on.
						int type = Header_GetLParamOfIndex( header->hdr.hwndFrom, header->iItem );

						Log(1,L"Result Window: HDN_ITEMCLICK:\n");
						Log(1,L"   type: %i\n",type );

						// Do not allow sorting by 'Info':
						if( type == HDR_RESULT_INFO ) {
							break;
						}

						// If he clicked the same item twice in a row,
						// we change the arrow to be a DESCENDING arrow.
						if( sort_by[PRIMARY].index == type ) {
							Log(1,L"   Changing primary sortstyle.\n");

							// Toggle it so that it sorts the other way:
							sort_by[PRIMARY].sortstyle ^= 1;

							// Give it an icon: (primary).
							// Get the information:
							HDITEM item;
							ZeroMemory( &item, sizeof(item) );
								item.mask = HDI_BITMAP | HDI_FORMAT;
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
							// Modify the icon:
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							// Update the information:
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );
						} else {
							// The click was on an item that is NOT the primary.

							// Take away the sort arrow on the old secondary one.
							// (He will be replaced by the old primary).
							if( sort_by[SECONDARY].index != -1 ) {
								Log(1,L"   Removing the old secondary.\n");
								// Get the information from the 'index' of the secondary:
								HWND hwnd_old = hwnd_resultlist[ result_header_items[sort_by[SECONDARY].index].hwnd ];
								int index_old = Header_GetIndexOfItemWithLParamOf( hwnd_old, sort_by[SECONDARY].index );
								// Now, take away its arrow:
								// Get the information:
								HDITEM item;
								ZeroMemory( &item, sizeof(item) );
									item.mask = HDI_BITMAP | HDI_FORMAT;
								Header_GetItem( hwnd_old, index_old, &item );
								// Modify the icon:
									item.mask = HDI_BITMAP | HDI_FORMAT;
									item.hbm = NULL;
									item.fmt ^= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
								// Update the information:
								Header_SetItem( hwnd_old, index_old, &item );
							}

							Log(1,L"   Old Secondary: %i by %i\n",sort_by[SECONDARY].index, sort_by[SECONDARY].sortstyle );
							Log(1,L"   Old Primary  : %i by %i\n",sort_by[PRIMARY].index, sort_by[PRIMARY].sortstyle );

							// The new secondary sort is the old primary sort:
							sort_by[SECONDARY].index = sort_by[PRIMARY].index;
							sort_by[SECONDARY].sortstyle = sort_by[PRIMARY].sortstyle;
							Log(1,L"   Current Secondary: %i by %i\n",sort_by[SECONDARY].index, sort_by[SECONDARY].sortstyle );

							// Set the new info:
							sort_by[PRIMARY].index = type;
							sort_by[PRIMARY].sortstyle = ASCENDING;
							Log(1,L"   Current Primary  : %i by %i\n",sort_by[PRIMARY].index, sort_by[PRIMARY].sortstyle );


							// MODIFY THE PRIMARY ICON:
							// Give it an icon:
							HDITEM item;
							ZeroMemory( &item, sizeof(item) );
								item.mask = HDI_BITMAP | HDI_FORMAT;
							// Get the information:
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
							// Modify the icon:
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							// Update the information:
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );

							// MODIFY THE SECONDARY ICON:
							// Give the secondary a different icon:
							if( sort_by[SECONDARY].index != -1 ) {
								HWND hwndsecondary = hwnd_resultlist[ result_header_items[sort_by[SECONDARY].index].hwnd ];
								int indexsecondary = Header_GetIndexOfItemWithLParamOf( hwndsecondary, sort_by[SECONDARY].index );
								//HDITEM item;
								ZeroMemory( &item, sizeof(item) );
									item.mask = HDI_BITMAP;
								// Get the information:
								Header_GetItem( hwndsecondary, indexsecondary, &item );
								// Modify the icon:
									item.mask = HDI_BITMAP;
									item.hbm = (sort_by[SECONDARY].sortstyle==ASCENDING)?bitmap_sort_asc2:bitmap_sort_dsc2;
								// Update the information:
								Header_SetItem( hwndsecondary, indexsecondary, &item );
							}
						}
						// If there are any results, sort them:
						SortResults(); // (this checks to see if there are any).
						break;
					}
					case HDN_DIVIDERDBLCLICK: {
					// Resize all of the items in the header control.

						int numitems = Header_GetItemCount( header->hdr.hwndFrom );
						if( numitems == 0 ) {  break;  }

						int windowid = (wParam==WID_RESULT_LIST1)?0:1;

						int percentage = 100/numitems;
						int percentage_so_far = 0;

						for( int i = 0; i < HDR_RESULT_MAX; i++ ) {
							Header_Item_Information *info = &result_header_items[i];
							if( info->hwnd == windowid ) {
								info->percent = percentage;
								percentage_so_far += info->percent;
								if( Header_GetItemToRight( header->hdr.hwndFrom, Header_GetIndexOfItemWithLParamOf(header->hdr.hwndFrom,i) ) == -1 ) {
									info->percent += (100-percentage_so_far);
								}
							}
						}

						SetResultHeaders( hWnd );

						// Repaint the screen.
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}
				}
			}// if wparam == WID_RESULT_LIST1 or ~2
			return(0);
		}
////////////////////////////////
///                          ///
///   S C R O L L    B A R   ///
///                          ///
////////////////////////////////
		case WM_VSCROLL: {
         // Get all the vertial scroll bar information
			SCROLLINFO si; 
				si.cbSize = sizeof(si);
				si.fMask  = SIF_ALL;
         GetScrollInfo (hWnd, SB_VERT, &si);
         // Save the position for comparison later on
         start_entry = si.nPos;
         switch( LOWORD(wParam) ) {
				// user clicked the HOME keyboard key
				case SB_TOP: {  si.nPos = si.nMin;  break;  }  
				// user clicked the END keyboard key
				case SB_BOTTOM: {  si.nPos = si.nMax;  break;  }
				// user clicked the top arrow
				case SB_LINEUP: {  si.nPos -= 1;  break;  }
				// user clicked the bottom arrow
				case SB_LINEDOWN: {  si.nPos += 1;  break;  }  
				// user clicked the shaft above the scroll box
				case SB_PAGEUP: {  si.nPos -= si.nPage;  break;  }  
				// user clicked the shaft below the scroll box
				case SB_PAGEDOWN: {  si.nPos += si.nPage;  break;  }  
				// user dragged the scroll box
				case SB_THUMBTRACK: {  si.nPos = si.nTrackPos;  break;  }
         }
         // Set the position and then retrieve it.  Due to adjustments
         //   by Windows it may not be the same as the value set.
				si.fMask = SIF_POS;
         SetScrollInfo( hWnd, SB_VERT, &si, TRUE );
         GetScrollInfo( hWnd, SB_VERT, &si );
         // If the position has changed, scroll window and update it
         if( si.nPos != start_entry ) {
				RECT rc;
					GetClientRect( hWnd,&rc );
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
				// RDW_ERASE makes it redraw the BG.
				// RDW_INVALIDATE makes it WM_PAINT.
         }
			return(0);
		}
/////////////////////
///               ///
///   P A I N T   ///
///               ///
/////////////////////
		case WM_PAINT: {
#ifdef UBERDEBUG
			return(0);
#endif
			// If the headers aren't there, then we have nothing to do:
			if( hwnd_resultlist[0] == NULL || hwnd_resultlist[1] == NULL ) {  break;  }

			if( search_lock ) {  return(0);  }
			// If there are no results, then just clear it:
			if( edict_results.size() < 1 || thread_searching ) {
				HDC hdc;
				PAINTSTRUCT ps;
				hdc = BeginPaint(hWnd, &ps);
				   Graphics graphics(hdc);

					SolidBrush brush_result_bg( color_result_windowbg );

					RectF rcFill;
						rcFill.X = rcClient_result.left;
						rcFill.Y = rcClient_result.top;
						rcFill.Width = rcClient_result.right;
						rcFill.Height = rcClient_result.bottom;
					graphics.FillRectangle( &brush_result_bg, rcFill );

				EndPaint(hWnd, &ps);
				return(0); // An application should return 0 if it processes WM_PAINT.
			}
			// If, for some reason, the entry height is zero, we're done.
			if( entry_height == 0 ) {
				Log(L"ERROR: Result Window: WM_PAINT: 'entry_height' is Zero.\n");
				return(0);
			}
		
			// Now, we know that there are entries; it's time to draw them:
			HDC hdc;
			PAINTSTRUCT ps;
	      hdc = BeginPaint(hWnd, &ps);
				Graphics graphics(hdc);

				// Set up the fonts for the text:
				Font font_word( hdc,hfont_result_word );
				Font font_kana( hdc,hfont_result_kana );
				Font font_meaning( hdc,hfont_result_meaning );
				Font font_info( hdc,hfont_result_info );

				// This is for the background of the window:
				SolidBrush brush_windowbg( color_result_windowbg );

				StringFormat format(StringFormatFlagsNoWrap);
					format.SetTrimming( StringTrimmingNone );

				struct Draw_Result_Info {
					bool visible;
					RECT rc;
					int line_mod;
					int height;
				};

				Draw_Result_Info draw_info[HDR_RESULT_MAX];
				for( int i = 0; i < HDR_RESULT_MAX; i++ ) {
					Header_Item_Information *info = &result_header_items[i];
					draw_info[i].visible = (info->hwnd!=(-1));
					if( draw_info[i].visible ) {
						draw_info[i].line_mod = 0;
						int index = Header_GetIndexOfItemWithLParamOf( hwnd_resultlist[info->hwnd], i );
						Header_GetItemRect( hwnd_resultlist[info->hwnd], index, &draw_info[i].rc );

						if( info->hwnd == 0 ) {
							draw_info[i].height = result_lineh1;
						} else if( info->hwnd == 1 ) {
							draw_info[i].height = result_lineh2;

							draw_info[i].line_mod = result_lineh1;
						}
					}
				}


				// Get vertical scroll bar position
				SCROLLINFO si;
				   si.cbSize = sizeof(si);
		         si.fMask  = SIF_POS;
		      GetScrollInfo( hWnd, SB_VERT, &si );
			   start_entry = si.nPos;

				int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
				int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

				// Draw all of the entries that fit on the screen:
				for( int e = start_entry; e <= end_entry; e++ ) {
					// 'i' is the entry, relative to the screen.
					// Thus, i=0 is the TOP entry on the screen.
					int i = e-start_entry;

					// 'entry' is what' we're going to be working with:
					EDICT_Entry *entry = &edict_results[e];

					// Set up the colors for the entry:
					SolidBrush brush_bg( color_result_bg1 );
					if( e%2 ) {	brush_bg.SetColor( color_result_bg2 );	}
					SolidBrush brush_word( color_result_word );
					SolidBrush brush_kana( color_result_kana );
					SolidBrush brush_info( color_result_info );
					SolidBrush brush_meaning( color_result_meaning );
					Pen pen_border( color_result_border_outside );
					Pen pen_border_inside( color_result_border_inside );
					// If the entry is selected:
					if( entry->selected ) {
						InvertColor( brush_bg );
						InvertColor( brush_word );
						InvertColor( brush_kana );
						InvertColor( brush_info );
						InvertColor( brush_meaning );
						InvertColor( pen_border );
						InvertColor( pen_border_inside );
					}



					// Fill in the rectangle that'll be the background of the entry:
					RectF rc;
						rc.X = 0;
						rc.Y = rcClient_result.top + i*entry_height;
						rc.Width = rcClient_result.right-1;
						rc.Height = entry_height-1;
					graphics.FillRectangle( &brush_bg, rc );



					for( int x = 0; x < HDR_RESULT_MAX; x++ ) {
						// If this item is disabled, don't draw it:
						if( !draw_info[x].visible ) {  continue;  }

						// Draw the box around the item:
							rc.X = draw_info[x].rc.left;
							rc.Y = rcClient_result.top + i*entry_height + draw_info[x].line_mod;
							rc.Width = draw_info[x].rc.right - draw_info[x].rc.left - 1;
							rc.Height = draw_info[x].height;
						graphics.DrawRectangle( &pen_border_inside, rc );
						// This is where we can draw the text:
							rc.X += bx;	rc.Width -= bx;
							rc.Y += by;	rc.Height -= by;
						switch( x ) {
							case HDR_RESULT_WORD: {
								graphics.DrawString( entry->word.c_str(), -1, &font_word, rc, &format, &brush_word );
								break;
							}
							case HDR_RESULT_KANA: {
								if( registry.GetBOOL(".\\Misc\\ROMAjiMode") ) {
									graphics.DrawString( KanaStringToRomaji(entry->kana).c_str(), -1, &font_kana, rc, &format, &brush_kana );
								} else {
									graphics.DrawString( entry->kana.c_str(), -1, &font_kana, rc, &format, &brush_kana );
								}
								break;
							}
							case HDR_RESULT_INFO: {
								wstring type;
								for( int t = 0; t < entry->info.size(); t++ ) {
									type += entry->info[t];
									if( t < entry->info.size()-1 ) {
										type += L" | ";
									}
								}
								graphics.DrawString( type.c_str(), -1, &font_info, rc, &format, &brush_info );
								break;
							}
							case HDR_RESULT_MEANING: {
								wstring meaning = Edict_GetMeaning( *entry );
								graphics.DrawString( meaning.c_str(), -1, &font_meaning, rc, &format, &brush_meaning );
								break;
							}
							case HDR_RESULT_WORD_L: {
								wchar_t lengthw[10]; _itow(entry->word.size(),lengthw,10);
								graphics.DrawString( wstring(lengthw).c_str(), -1, &font_meaning, rc, &format, &brush_meaning );
								break;
							}
							case HDR_RESULT_KANA_L: {
								wchar_t lengthk[10]; _itow(entry->kana.size(),lengthk,10);
								graphics.DrawString( wstring(lengthk).c_str(), -1, &font_meaning, rc, &format, &brush_meaning );
								break;
							}
						}//switch(x)
					}//for(x)
					



					// Draw a black bounding rectangle around everything:
						rc.X = 0;
						rc.Y = rcClient_result.top + i * entry_height;
						rc.Width = rcClient_result.right-1;
						rc.Height = entry_height-1;
					graphics.DrawRectangle( &pen_border, rc );
				}// (the for loop from start_entry to end_entry).


				// Fill in the remaining space:
				RectF rcFill;
					rcFill.X = rcClient_result.left;
					rcFill.Y = rcClient_result.top + (end_entry-start_entry+1)*entry_height;
					rcFill.Width = rcClient_result.right;
					rcFill.Height = rcClient_result.top + rcClient_result.bottom - (int)rcFill.Y;
				if( rcFill.Height > 0 ) {
					graphics.FillRectangle( &brush_windowbg, rcFill );
				}

		   EndPaint(hWnd, &ps);
			return(0); // An application should return 0 if it processes WM_PAINT.
		}//*/
		case WM_ERASEBKGND: {
			// An application should return 0 if it did NOT erase the backgound.
			// Here, we're faking that we did, so...
			return(TRUE);
		}
//////////////////////////////////////////////
///                                        ///
///   M E N U    N O T I F I C A T I O N   ///
///                                        ///
//////////////////////////////////////////////
		case WM_MENURBUTTONUP: {
			/*	wParam
				Specifies the position of the item when the mouse was released. 
			lParam
				Handle to the menu containing the item.//*/
			if( lParam == (LPARAM)hmenu_result_header ) {
				// The click was for the Header menu:
				int id = GetMenuItemID( hmenu_result_header, wParam );
				if( id >= 1 && id <= 6 ) {
					enum { HEADER_ADD, HEADER_REMOVE };
					int action;
					int state = GetMenuState( hmenu_result_header, wParam, MF_BYPOSITION );


					int item = id-1;
					// This is the header window to use:
					HWND header = (item<4)?hwnd_resultlist[0]:hwnd_resultlist[1];

					if( state&MF_CHECKED ) {
					////////////////////////////////////////////
					///   WE ARE GOING TO UNCHECK THE ITEM   ///
					////////////////////////////////////////////
						// If it's the only one, then stop:
						bool onlyone = true;
						int max = (item<4)?4:2;
						int item_x = item;  if( item_x >= 4 ) {  item_x -= 4;  }
						for( int i = 0; i < max; i++ ) {
							HDITEM hditem;
								hditem.mask = HDI_LPARAM;
							Header_GetItem( header, i, &hditem );
							if( hditem.lParam != (LPARAM)(double)0 && i != item_x ) {
								onlyone = false;
								break;
							}
						}
						if( onlyone ) {  return(0);  }

						// Otherwise, remove the check mark:
						CheckMenuItem( hmenu_result_header, wParam, MF_BYPOSITION );
						action = HEADER_REMOVE;
					} else {
					//////////////////////////////////////////
					///   WE ARE GOING TO CHECK THE ITEM   ///
					//////////////////////////////////////////
						// Add the check mark:
						CheckMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_CHECKED );
						action = HEADER_ADD;
					}
					// Force it to redraw the menu:
					EnableMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_GRAYED );
					EnableMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_ENABLED );

					// This is the array index of the item:
					if( item >= 4 ) {  item -= 4;  }
					// Here's the guy we're primarily changing:
					HDITEM hditem;
						hditem.mask = HDI_LPARAM;
					Header_GetItem( header, item, &hditem );
					// Here's the other guy who needs to pick up the slack:
					int next = -1;
					HDITEM hditem2;
						hditem2.mask = HDI_LPARAM;
					// Let it go?
					bool allow = true;
					// Okay, gogogo:
					if( action == HEADER_ADD ) {
					///////////////////////////////////
					///   ATTEMPT TO ADD THE ITEM   ///
					///////////////////////////////////
						hditem.lParam = (LPARAM)(double)10;

						// Shrink another one:
						next = Header_GetClosestItemWithlParamGreaterThan( header, item, (LPARAM)(double)10 );
						if( next == -1 ) {
							allow = false;
						} else {
							Header_GetItem( header, next, &hditem2 );
								hditem2.lParam -= (LPARAM)(double)hditem.lParam;
						}
					} else {
					//////////////////////////////////////
					///   ATTEMPT TO DELETE THE ITEM   ///
					//////////////////////////////////////

						// Add to another one:
						next = Header_GetClosestItemWithlParamGreaterThan( header, item, (LPARAM)(double)0 );
						if( next == -1 ) {
							allow = false;
						} else {
							Header_GetItem( header, next, &hditem2 );
								hditem2.lParam += (LPARAM)(double)hditem.lParam;
						}

						hditem.lParam = (LPARAM)(double)0;
					}
					// If it's possible, add the item and change another:
					if( allow ) {
						Header_SetItem( header, item, &hditem );
						Header_SetItem( header, next, &hditem2 );

						RECT rc; GetClientRect( hWnd, &rc );
						SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					}
				} else if( id == 7 ) {
					// Normalize all of the header bars:
					for( int i = 0; i < 4; i++ ) {
						HDITEM hditem;
							hditem.mask = HDI_LPARAM;
							if( i == 0 ) {      hditem.lParam = 25;  }
							else if( i== 1 ) {  hditem.lParam = 61;  }
							else if( i== 2 ) {  hditem.lParam = 7;  }
							else if( i== 3 ) {  hditem.lParam = 7;  }
						Header_SetItem( hwnd_resultlist[0], i, &hditem );
					}
					for( i = 0; i < 2; i++ ) {
						HDITEM hditem;
							hditem.mask = HDI_LPARAM;
							if( i == 0 ) {      hditem.lParam = 25;  }
							else if( i== 1 ) {  hditem.lParam = 75;  }
						Header_SetItem( hwnd_resultlist[1], i, &hditem );
					}
					
					RECT rc; GetClientRect( hWnd, &rc );
					SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
					RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
				}
			}//*/
			return(0);
		}
//////////////////////////////////////
///                                ///
///   M O U S E    B U T T O N S   ///
///                                ///
//////////////////////////////////////
		case WM_LBUTTONDOWN: {
		/* If he clicked on an item, select that item.
		   If he clicked on blank space, deselect any selected item. */
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// Regardless of anything, this window now has keyboard focus:
			SetFocus( hWnd );

			// Select the entry that he clicked on:
			// If there are no results, do nothing:
			if( edict_results.size() == 0 ) {  break;  }

			int mode = 0;
			if( GetKeyState(VK_CONTROL)&0x8000 ) {
				mode |= SELECT_MODE_ADD;
			}
			if( GetKeyState(VK_SHIFT)&0x8000 ) {
				mode |= SELECT_MODE_FILL;
			}
			ResultWindow_SelectEntry( ResultWindow_GetEntryFromPoint( mx,my ), mode );
			return(0); // An application should return 0 if it processes WM_LBUTTONDOWN.
		}
		case WM_LBUTTONDBLCLK: {
			// wParam :: what virtual keys are down.
			// lParam :: LONG( x, y ).
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);
			
			// Select it if it's not already selected:
			int index = ResultWindow_GetEntryFromPoint( mx,my );
			if( !edict_results[index].selected ) {
				SendMessage( hWnd, WM_LBUTTONDOWN, wParam,lParam );
			}

			// The double-click action is really just the keyboard's ENTER action:
			SendMessage( hWnd,WM_KEYDOWN,VK_RETURN,0 );
			return(0); // An application should return 0 if it processes WM_LBUTTONDBLCLK.
		}
		case WM_RBUTTONDOWN: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// If there's already a search going on, stop.
			if( thread_searching ) {  return(0);  }
			if( search_lock ) {  return(0);  }

			// Select the item that it's over:
			if( edict_results.size() < 1 ) {  return(0);  }

			// Select it if it's not already selected:
			int index = ResultWindow_GetEntryFromPoint( mx,my );
			if( !edict_results[index].selected ) {
				SendMessage( hWnd, WM_LBUTTONDOWN, wParam,lParam );
			}

			RECT rcWindow;
				GetWindowRect(hWnd,&rcWindow);

			// Make the menu:
			HMENU hMenu = CreatePopupMenu();

			HMENU hmenu_search = CreatePopupMenu();
				AppendMenu( hmenu_search, MF_STRING, 1, L"The kanji in this word" );
				AppendMenu( hmenu_search, MF_STRING, 2, L"Words starting with this word" );
				AppendMenu( hmenu_search, MF_STRING, 3, L"Words ending with this word" );
				AppendMenu( hmenu_search, MF_STRING, 4, L"Words with this word anywhere" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_search, L"Search..." );
				AppendMenu( hMenu, MF_SEPARATOR, NULL,NULL );
			HMENU hmenu_copy = CreatePopupMenu();
				AppendMenu( hmenu_copy, MF_STRING, 5, L"This word" );
				AppendMenu( hmenu_copy, MF_STRING, 6, L"This word's kana" );
				AppendMenu( hmenu_copy, MF_STRING, 7, L"This word's meanings" );
				AppendMenu( hmenu_copy, MF_STRING, 8, L"All info on this word" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_copy, L"Copy..." );

			int result = TrackPopupMenuEx( hMenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );
			if( result == 1 ) {
			// Show the kanji in this word:
				kanjidic_results.clear();
				KanjiWindow_SearchForKanjiIn( edict_results[result_last].word );
				KanjiWindow_SetUpForNewResults();
			} else if( result >= 2 && result <= 4 ) {
			// Search for the word in...:
				wstring str;
				if( result == 2 ) {
					str = edict_results[result_last].word;
					str += WC_STAR;
				} else if( result == 3 ) {
					str = WC_STAR;
					str += edict_results[result_last].word;
				} else if( result == 4 ) {
					str = WC_STAR;
					str += edict_results[result_last].word;
					str += WC_STAR;
				}
				// Set the text of the search edit control to the kanji:
				SETTEXTEX st;
					st.flags = ST_KEEPUNDO;
					st.codepage = 1200; // 1200 == unicode.
				SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)str.c_str() );

				// Select the text:
				SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );

				// Do a search with that text:
				SendMessage( hwnd_searchwindow,WM_COMMAND, ID_SEARCH_GO,0 );
			} else if( result == 5 ) {
			// Copy the word to the clipboard:
				CopySelectedEdictInfo( EINFO_WORD );
			} else if( result == 6 ) {
			// Copy the word's kana to the clipboard:
				CopySelectedEdictInfo( EINFO_KANA );
			} else if( result == 7 ) {
			// Copy the word's meanings to the clipboard:
				CopySelectedEdictInfo( EINFO_MEANING );
			} else if( result == 8 ) {
			// Copy all info on the word to the clipboard:
				CopySelectedEdictInfo( EINFO_ALL );
			}
				DestroyMenu( hmenu_search );
				DestroyMenu( hmenu_copy );
			DestroyMenu( hMenu );

			return(0); // An application should return 0 if it processes WM_RBUTTONDOWN.
		}
////////////////////////////////////////
///                                  ///
///   M O U S E    M O V E M E N T   ///
///                                  ///
////////////////////////////////////////
		case WM_MOUSEMOVE: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);
			last_mx = mx; last_my = my;

			// Set the cursor to the standard arrow:
			SetCursor( cursor_arrow );

			// If we have captured the mouse (and it moves out of bounds),
			// then give it back so we can get the WM_MOUSELEAVE message.
			if( hover_on ) {
				if( mx < rcClient_result.left || my < rcClient_result.top ||
					 mx >= rcClient_result.right || my >= rcClient_result.bottom ) {
					ReleaseCapture();
				}
			}

			// If the mouse moves over the window, capture it:
			HWND hwnd_focus = GetFocus();
			if( GetForegroundWindow()==hwnd_mainwindow && hwnd_focus != hWnd && !registry.GetBOOL(".\\Search\\AlwaysSearchFocus") ) {
				HWND hwnd_old = SetFocus( hWnd );
				if( hwnd_old == NULL ) {
					Log(L"ResultWindow: WM_MOUSEMOVE:\n   SetFocus(): ERROR: %u\n",GetLastError() );
				}
			}

			// When the mouse hovers over an entry, show a pop-up view of it.

			// If there are no results, then we're not doing any pop-ups.
			if( edict_results.size() == 0 ) {  return(0);  }
			if( thread_searching ) {  return(0);  }
			if( search_lock ) {  return(0);  }

			// This is the entry that it's over *now*.
			int hover_entry_test = ResultWindow_GetEntryFromPoint(mx,my);
			
			// If there's already a pop-up window, and the one that it's
			// over is *different* from that one, then destroy the old one.
			if( hover_on ) {
				if( hover_entry_test != hover_entry ) {
					DestroyWindow( hwnd_hover );
					hover_on = false;
					ReleaseCapture();
				}
			}

			// Now, see if the mouse hovers over an entry.
			if( hover_entry_test != -1 ) {
				// Cancel any others:
				{
					TRACKMOUSEEVENT event;
						event.cbSize = sizeof(TRACKMOUSEEVENT);
						event.dwFlags = TME_CANCEL|TME_HOVER;
						event.dwHoverTime = 700; // hover-time (milliseconds).
						event.hwndTrack = hWnd;
					_TrackMouseEvent( &event );
				}
				TRACKMOUSEEVENT event;
					event.cbSize = sizeof(TRACKMOUSEEVENT);
					event.dwFlags = TME_HOVER;
					event.dwHoverTime = 700; // hover-time (milliseconds).
					event.hwndTrack = hWnd;
				_TrackMouseEvent( &event );
				// Set the hover variables:
				hover_entry = hover_entry_test;
				hover_pos.x = mx;
				hover_pos.y = my;
			}
			return(0); // An application should return 0 if it processes WM_MOUSEMOVE.
		}
		case WM_MOUSEHOVER: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// NEGATIVE CASES:
			// * If there's already a hover window up, don't draw it again:
			if( hover_on ) {  return(0);  }
			// If the mouse moved since we sent the hover message, NO:
			if( abs( hover_pos.x - mx ) > 10 || abs( hover_pos.y - my ) > 10 ) {
				return(0);
			}
			// If it trailed off into another entry, NO:
			if( ResultWindow_GetEntryFromPoint(mx,my) != hover_entry ) {
				return(0);
			}

			/////////////////////////////////
			///   MAKE THE HOVER WINDOW   ///
			/////////////////////////////////
			// Otherwise, we have a hover window to create:
			hwnd_hover = CreateResultHoverWindow( hover_entry );

			// Capture the mouse so that this window keeps getting mouse input.
			SetCapture( hWnd );

			// Set its position:
			// First, figure out where it is (this uses the parent window for positioning):
			RECT rcWin;
				GetWindowRect( hWnd, &rcWin );
			int newx = rcWin.left+mx, newy = rcWin.top+my;
			// Then, figure out how big it is:
				GetWindowRect( hwnd_hover, &rcWin );
			int neww = rcWin.right-rcWin.left, newh = rcWin.bottom-rcWin.top;

			// Get the area that a full-screen window can use:
			RECT rcDesk;
			SystemParametersInfo( SPI_GETWORKAREA, 0, &rcDesk, 0 );

			// If the pop-up goes too far off, put it back on:
			if( newx+neww >= rcDesk.right ) {  newx = rcDesk.right-neww;  }
			if( newx < 0 ) {  newx = 0;  }

			if( newy+newh >= rcDesk.bottom ) {  newy = rcDesk.bottom-newh;  }
			if( newy < 0 ) {  newy = 0;  }

			// Now, set the position and show it:
			SetWindowPos( hwnd_hover, NULL,
				newx, newy,
				0,0,
				SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW
			);

			// Yes, we have a hover window up.
			hover_on = true;

			// We want to know if the mouse leaves the window, so that we
			// can destroy the hover window then.
			TRACKMOUSEEVENT event;
				event.cbSize = sizeof(TRACKMOUSEEVENT);
				event.dwFlags = TME_LEAVE;
				event.hwndTrack = hWnd;
			_TrackMouseEvent( &event );
			return(0);
		}
		case WM_MOUSELEAVE: {
			if( hover_on ) {
				hover_on = false;
				DestroyWindow( hwnd_hover );
				ReleaseCapture();
			}
			return(0);
		}
//////////////////////////////////
///                            ///
///   M O U S E    W H E E L   ///
///                            ///
//////////////////////////////////
		case WM_MOUSEWHEEL: {
		// We use the mousewheel to scroll:
			// Wheel info:
			WORD fwKeys = LOWORD(wParam);
			short zDelta = (short)HIWORD(wParam);
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// Use the following code to obtain the horizontal and vertical position:
			short xPos = LOWORD(lParam); 
			short yPos = HIWORD(lParam);

			if( zDelta < 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEDOWN,0), 0L);
			} else if( zDelta > 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEUP,0), 0L);
			}
			SendMessage(hWnd, WM_MOUSEMOVE, 0, MAKELPARAM(mx,my) );
			return(0); // An application should return 0 if it processes WM_MOUSEWHEEL.
		}
///////////////////////////
///                     ///
///   K E Y B O A R D   ///
///                     ///
///////////////////////////
		/* NOTE:
				The keyboard selection things still need a bit of work.
		*/
		case WM_KEYDOWN: {
			int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
			int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

			int mode = 0;
			if( GetKeyState(VK_CONTROL)&0x8000 ) { mode |= SELECT_MODE_ADD;	}
			if( GetKeyState(VK_SHIFT)&0x8000 ) {	mode |= SELECT_MODE_FILL;	}

			WORD wScrollNotify = -1;
			switch (wParam) {
				// Handle the keys for scrolling:
				case VK_UP: {
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( result_last > 0 ) {
						if( result_last == start_entry ) {	wScrollNotify = SB_LINEUP;	}
						ResultWindow_SelectEntry( result_last-1, mode );
					} else {
						wScrollNotify = SB_LINEUP;
					}
					break;
				}
				case VK_DOWN: {
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( result_last < edict_results.size()-1 ) {
						if( result_last == end_entry-1 ) {	wScrollNotify = SB_LINEDOWN;	}
						ResultWindow_SelectEntry( result_last+1, mode );
					} else {
						wScrollNotify = SB_LINEDOWN;
					}
					break;
				}
				case VK_PRIOR: {//	wScrollNotify = SB_PAGEUP;		break;	}
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( result_last >= 0 ) {
						int index = result_last;
						if( index > 0+entries_that_fit_on_screen-1 ) {
							index -= entries_that_fit_on_screen;
						} else {
							index = 0;
						}
						ResultWindow_SelectEntry( index, mode );
						wScrollNotify = SB_PAGEUP;
					} else {
						wScrollNotify = SB_PAGEUP;
					}
					break;
				}
				case VK_NEXT: {//	wScrollNotify = SB_PAGEDOWN;	break;	}
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( result_last >= 0 ) {
						int index = result_last;
						if( index < (int)edict_results.size()-entries_that_fit_on_screen ) {
							index += entries_that_fit_on_screen;
						} else {
							index = edict_results.size()-1;
						}
						ResultWindow_SelectEntry( index, mode );
						wScrollNotify = SB_PAGEDOWN;
					} else {
						wScrollNotify = SB_PAGEDOWN;
					}
					break;
				}
				case VK_HOME: {//ScrollNotify = SB_TOP;			break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( result_last >= 0 ) {
						if( result_last != 0 ) {
							ResultWindow_SelectEntry( 0, mode );
						}
						wScrollNotify = SB_TOP;
					} else {
						wScrollNotify = SB_TOP;
					}
					break;
				}
				case VK_END: {//wScrollNotify = SB_BOTTOM;		break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( result_last >= 0 ) {
						if( result_last != edict_results.size()-1 ) {
							ResultWindow_SelectEntry( edict_results.size()-1, mode );
						}
						wScrollNotify = SB_BOTTOM;
					} else {
						wScrollNotify = SB_BOTTOM;
					}
					break;
				}
				// Switch windows:
				case VK_TAB: {
					SetFocus( hwnd_searchedit );
					break;
				}
				// Select the top one:
				case VK_RETURN: {
				/* If an entry is selected already, treat it as a double-click.
				   Otherwise, select the top entry on the screen. */

					// If there's already a search going on, stop.
					if( thread_searching ) {  return(0);  }
					if( search_lock ) {  return(0);  }

					if( edict_results.size() > 0 ) {
						// If there's no selected entry, select the top one.
						if( result_last == -1 ) {
							ResultWindow_SelectEntry( start_entry, mode );
						} else {
						// Do the double-click action on it.
						// * Clear the kanjidic results.
						// * Add only the kanji that apply to what was double-clicked:
							// Clear the kanjidic results:
							kanjidic_results.clear();

							// Add the kanji from the selected entries to it:
							// Go through every entry:
							int num_selected = 0;
							int num_selected_index = -1;
							for( int e = 0; e < edict_results.size(); e++ ) {
								if( !edict_results[e].selected ) {  continue;  }
								num_selected++; num_selected_index = e;
								// Go through every character in that entry:
								for( int w = 0; w < edict_results[e].word.size(); w++ ) {
									Uint16 character = edict_results[e].word[w];
									// If it's not hiragana or katakana, then it could be kanji:
									if( !( IsHiragana(character) || IsKatakana(character) ) ) {
										KANJIDIC_Index_Entry *index = Kanjidic_GetIndexOf( character );
										if( index != NULL ) {
											// Now we want to make sure that we haven't already added it:
											bool okaytoadd = true;
											for( int l = 0; l < kanjidic_results.size(); l++ ) {
												if( kanjidic_results[l].kanji == index->kanji ) {
													okaytoadd = false;
													break;
												}
											}
											// If it's new, then add it:
											if( okaytoadd ) {
												KANJIDIC_Entry entry;
												// New:2004.12.21
												ZeroMemory( &entry, sizeof(entry) );
													LoadKANJIDICInfoInto( entry, index );
												kanjidic_results.push_back( entry );
											}
										}
									} // if it's probably kanji
								}// for w = 0 --> word_size
							}// for e = 0 --> entries_size

							// Resize & redraw the kanji window:
							KanjiWindow_SetUpForNewResults();



							if( num_selected == 1 ) {
								// Set the text of the search edit control to the word:
								SETTEXTEX st;
									st.flags = ST_KEEPUNDO;
									st.codepage = 1200; // 1200 == unicode.
								SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)edict_results[num_selected_index].word.c_str() );

								// Select the text:
								SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
							}
						}
					}
					break;
				}// case VK_RETURN
				case VK_a: {
					if( GetKeyState(VK_CONTROL)&0x8000 ) {
					// CTRL+A
					// *Select all entries*
						// Select every entry:
						for( int i = 0; i < edict_results.size(); i++ ) {
							edict_results[i].selected = true;
						}
						// Update the window:
						RedrawWindow( hwnd_resultwindow, NULL, NULL, RDW_INVALIDATE );
					}
					break;
				}
				case VK_c: {
					if( GetKeyState(VK_CONTROL)&0x8000 ) {
					// CTRL+C
					// *Copy the selected entries*
						CopySelectedEdictInfo( EINFO_ALL );
					}
					break;
				}// case VK_c
				case VK_DELETE: {
					// Remove the selected entries:
					bool deleted = false;
					for( int e = 0; e < edict_results.size(); e++ ) {
						if( edict_results[e].selected ) {
							edict_results.erase( edict_results.begin()+e );
								e--;
							deleted = true;
						}
					}
					if( deleted ) {
						// Be sure to adjust the positions:
						if( result_base >= edict_results.size() ) {
							result_base = edict_results.size()-1;
						}
						if( result_last >= edict_results.size() ) {
							result_last = edict_results.size()-1;
						}
						// Adjust the scroll bar:
						RECT rc; GetClientRect( hWnd, &rc );
							SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					}
					break;
				}
				case VK_SPACE: {
					Log(L"RESULT HEADER DIALOG BOX\n");
					CreateDialog_ResultHeader( hWnd );
					break;
				}
			}

 
			if( wScrollNotify != -1 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L);
				SendMessage(hWnd, WM_MOUSEMOVE, 0, MAKELPARAM(last_mx,last_my) );
			}
			return(0);
		}// case WM_KEYDOWN
		case WM_CHAR: {
			switch( wParam ) {
				case L'\t': {
					SetFocus( hwnd_searchedit );
					break;
				}
			}
			return(0);
		}// case WM_CHAR
	}
/////////////////////////
///                   ///
///   D E F A U L T   ///
///                   ///
/////////////////////////
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateResult_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS wndClass; 
	   //wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.style          = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		wndClass.lpfnWndProc    = WndProc_Result_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("ResultWindow");
   
   RegisterClass(&wndClass);
   
   HWND hWnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
      TEXT("ResultWindow"),   // window class name
      TEXT("(ResultWindow)"),  // window caption
      WS_CHILD| WS_VISIBLE | WS_VSCROLL, // window style
      x,y, // (x,y)
      w,h, // (w,h)
      hwndParent,                     // parent window handle
      NULL,                     // window menu handle
      g_hinst,                // program instance handle
      NULL                    // creation parameters
	);
	if( !hWnd ) {
		Log(L"ERROR: Couldn't create Result Window\n");
		MessageBox(NULL,L"Couldn't create Result Window",L"ERROR",MB_OK );
		exit(-1);
	}
	  
   //ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////