// Search.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_H
#define SEARCH_H
///////////////////////////////////////////////////////////
#include <StringExtras\StringExtras.h>
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Search return codes:
enum {
	SEARCH_STATUS_OKAY = 0,
	SEARCH_STATUS_TRUNCATED = 1,
};
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This searches through all active dictionary index
	files for the kana (or the word, depending on the search
	method).
		When if finds a match, it loads all of the information
	from the EDICT into the edict_results vector.
*/
int SearchFor( wstring text ) {
	Log(1,L"SearchFor()\n");
	Log(1,L"   text1: %s\n",text.c_str() );

	int search_status = 0;

	// If it's kana-insensitive, convert it to katakana first,
	// and then do the search:
	if( search_conversion_kana == SEARCH_CONVERSION_KANA_INSENSITIVE ) {
		text = ToKatakana( text );
		Log(1,L"KANA-INSENSITIVE SEARCH\n");
	}
	Log(1,L"   text2: %s\n",text.c_str() );

	Log(1,L"   Number of dictionaries: %i\n",edict_dictionaries.size() );
	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		Log(1,L"      Processing DICT %i / %i (%S)\n",d+1,edict_dictionaries.size(),dictionary->displayname.c_str() );

		if( !dictionary->on ) {  continue;  }

		Log(1,L"      The dictionary is 'on'.\n");

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		Log(1,L"      Status bar text set.\n");

		// Go through all the entries:
		// (If the dictionary wasn't loaded, it won't pass the for loop's
		// initial condition, so we don't have to worry about anything).
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			// If we already have all of the results that we need:
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				Log(1,L"MAX RESULTS REACHED\n");
				break;
			}

			// Here's what we'll be playing with:
			wstring entry_kana = dictionary->index_entries[i].kana;

			// See if we're going to be searching in the word or kana part:
			// How this could happen:
			// 1) It was forced by a button.
			// 2) If the searchtext isn't kana.
			if( search_forcewordsearch == 1 || !IsKana_WildcardsOkay(text) ) {
				entry_kana = dictionary->index_entries[i].word;
				// NOTE:
				// As of now, "entry_kana" is the WORD.
			}

			// If we're doing a kana-insensitve search,
			// convert the entry's kana to katakana.
			if( search_conversion_kana == SEARCH_CONVERSION_KANA_INSENSITIVE ) {
				entry_kana = ToKatakana( entry_kana );
			}

			// Finally, do the strings match?
			bool addit = StringsMatch( entry_kana, text );

			if( addit ) {
				Log(1,L"   RESULT %i (entry: %i)\n",edict_results.size()+1, i+1 );
				EDICT_Entry entry;
				ZeroMemory( &entry, sizeof(entry) );
					entry.selected = false;
				LoadInfoInto( dictionary, entry, dictionary->index_entries[i] );
					SetInfo( entry );
				edict_results.push_back( entry );
			}
		}// for each entry in the dictionary.
		Log(1,L"      Finished searching in dictionary.\n");
	}// for loop for dictionaries.

	Log(1,L"Search done: status: %i.\n", search_status);
	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This is the English SearchFor() variation.  It manually
	loads every entry of every dictionary and checks the
	meaning string to see if there's a match.
*/
int SearchFor_English( wstring wtext ) {
	Log(1,L"SearchFor_English()\n");

	// ('wtext' is in Unicode format).
	string text = ToString(wtext);
	text = ToLower( text );

	// These are the english words that we're going to look for:
	vector<string> english_words;


	// Fullsearch searches for entries whose definitions are THAT word.
	bool fullsearch = false;
	string fullword;
	if( text[0] == '\"' && text[text.size()-1] == '\"' ) {
		fullword = text.substr(1,text.size()-2 );
		fullsearch = true;
	}


	if( !fullsearch ) {
	// If we're not using a full search, then break the search query down
	// into individual words.
		string word;
		bool quoting = false;
		for( int i = 0; i < text.size(); i++ ) {
			// If it's a letter, add it to the current word:
			if( text[i] >= 'A' && text[i] <= 'Z' || text[i] >= 'a' && text[i] <= 'z' ) {
				word += text[i];
				if( i == text.size()-1 && word.size() > 0 ) {
					english_words.push_back(word);
				}
			} else if( text[i] == '\"' ) {
				if( quoting ) {
					if( word.size() > 0 ) {
						english_words.push_back(word);
						word = "";
					}
					quoting = false;
				}
				else {  quoting = true;  }
			} else {
				if( quoting ) {
					word += text[i];
					if( i == text.size()-1 && word.size() > 0 ) {
						english_words.push_back(word);
					}
				}
				if( !quoting && word.size() > 0 ) {
					english_words.push_back(word);
					word = "";
				}
			}
		}

		if( debug_level > 0 ) {
			Log(1,L"   English search text: %S\n",text.c_str() );
			for( i = 0; i < english_words.size(); i++ ) {
				Log(1,L"      %S\n",english_words[i].c_str() );
			}
		}
	} // if( !fullsearch )...




	//////////////////////////////
	///   COMMENCE SEARCHING   ///
	//////////////////////////////


	int search_status = 0;

	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		if( !dictionary->on ) {  continue;  }

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		FILE *fedict = fopen( dictionary->filename.c_str(),"rb");
		if( !fedict ) {
			Log(L"ERROR: Could not open dictionary '%S'; skipping it.\n", dictionary->filename.c_str() );
			continue;
		}
		Log(1,L"   Searching in dictionary '%S'...\n",dictionary->filename.c_str() );

		// Go through all the entries:
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				break;
			}

			// Go through the english words vector, and if the entry has the word, add it.
			// (Remember to apply search rules...)

			bool okaytoadd = true;

			// Get the full line:
			char *line = GetEdictLine( fedict, dictionary->index_entries[i] );
			if( !line ) {  Log(L"      No line!\n");  continue;  }
			// Get the meaning from the line:
			char *original_meaning = strchr(line,'/');
			if( !original_meaning ) {  Log(L"      No meaning!\n");  continue;  }
			// Convert it to lower case:
			strlwr( original_meaning );


			if( fullsearch ) {
			///////////////////////
			///   FULL SEARCH   ///
			///////////////////////
				//Log(L"String: %S\n",original_meaning );

				vector<string> meanings;

				// Get the meanings:
				string strline = original_meaning;
				int pos = strline.find( '/',0 );
				strline = strline.substr( pos );
				while( strline.size() > 0 ) {
					if( strline[0] == L'/' ) {
						pos = strline.find( '/',1 );
						if( pos != -1 ) {
							string strmeaning = strline.substr( 1,pos-1 );
								meanings.push_back( strmeaning );
							strline = strline.substr( pos,-1 );
						} else {
							break;
						}
					} else {  break;  }
				}

				// Cut off any superficial text for each meaning:
				// (exp) (n) happy (as in feeling)
				//    is changed to:
				// happy
				for( int i = 0; i < meanings.size(); i++ ) {
					bool parentheses = false;
					int start = 0;
					while( parentheses || !( meanings[i][0+start]>='a'&&meanings[i][0+start]<='z' ) ) {
						if( start == meanings[i].size()-1 ) {  break;  }
						if( meanings[i][0+start] == '(' ) {
							parentheses = true;
						} else if( meanings[i][0+start] == ')' ) {
							parentheses = false;
						}
						start++;
					}

					parentheses = false;
					int end = 0;
					while( parentheses || !( meanings[i][meanings[i].size()-1-end]>='a'&&meanings[i][meanings[i].size()-1-end]<='z' ) ) {
						if( end == meanings[i].size()-1 ) {  break;  }
						if( meanings[i][meanings[i].size()-1-end] == ')' ) {
							parentheses = true;
						} else if( meanings[i][meanings[i].size()-1-end] == '(' ) {
							parentheses = false;
						}
						end++;
					}

					if( meanings[i].size() > end+start ) { 
						meanings[i] = meanings[i].substr(start,meanings[i].size()-end);
					}
					//Log(L"   Mod: %S\n",meanings[i].c_str() );
				}


				okaytoadd = false;
				for( i = 0; i < meanings.size(); i++ ) {
					if( strcmp( meanings[i].c_str(), fullword.c_str() ) == 0 ) {
						okaytoadd = true;
						break;
					}
				}


			///////////////////////////
			///   END FULL SEARCH   ///
			///////////////////////////
			} else {
			/////////////////////////
			///   GOOGLE SEARCH   ///
			/////////////////////////
				okaytoadd = true;

				// Go through the English search words:
				// Remember, we need to have ALL of the words be in the entry,
				// or else we don't want to allow it.
				for( int w = 0; w < english_words.size(); w++ ) {
					// If we've already failed, then stop.
					if( !okaytoadd ) {  break;  }

					// Restore the meaning:
					char *meaning = original_meaning;

					char *found = NULL;
					while( true ) {
						// This is where the match is:
						found = strstr( meaning,english_words[w].c_str() );
						if( !found ) {
							// The word that we're searching for is not in the entry at all.
							okaytoadd = false;
							break;
						} else {
							// If we're searching for the whole word, the text before and after
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_WHOLE ) {
								bool beginningok = true;
								bool endok = true;
								if( found-meaning > 0 ) {
									char c = meaning[(found-meaning)-1];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										beginningok = false;
									}
								}
								if( strlen(found) > english_words[w].size() ) {
									char c = found[english_words[w].size()];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										endok = false;
									}
								}

								// If both are okay, get out of the loop and move on to the next word:
								if( beginningok && endok ) {
									// Get out of the loop and move on to the next word.
									break;
								}
							}

							// If we're searching for the beginning of the word, then the text before
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_START ) {
								if( found-meaning > 0 ) {
									char c = meaning[(found-meaning)-1];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										;
									} else {
										// OKAY!!
										// Get out of the loop and move on to the next word.
										break;
									}
								}
							}

							// If we're searching for the end of the word, then the text after
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_END ) {
								if( strlen(found) > english_words[w].size() ) {
									char c = found[english_words[w].size()];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										;
									} else {
										// OKAY!!
										// Get out of the loop and move on to the next word.
										break;
									}
								}
							}

							// If we're searching for the word anywhere, just get it over with:
							if( search_style_location == SEARCH_STYLE_MIDDLE ) {
								// Get out of the loop and move on to the next word.
								break;
							}
						}

						// Okay, now we need to fix up the entry's meaning string.
						// If it still has some text in it, we want to focus on the next
						// set of characters.
						// In other words, we drop the first character and search all over
						// again.
						if( strlen(found)>1 ) {
							meaning = found+1;
						} else {
							okaytoadd = false;
							break;
						}
					} // while(true);

					if( !found ) {
						okaytoadd = false;
					}
				} // for english words...
			/////////////////////////////
			///   END GOOGLE SEARCH   ///
			/////////////////////////////
			} // if( !fullsearch )...

			// Clean up.
			delete [] line;

			// If we had a match, add it:
			if( okaytoadd ) {
				Log(1,L"      Okay to add...\n");
				EDICT_Entry entry_empty;
				ZeroMemory( &entry_empty, sizeof(entry_empty) );
					entry_empty.selected = false;
				LoadInfoInto( fedict, dictionary, entry_empty, dictionary->index_entries[i] );
					SetInfo( entry_empty );
				edict_results.push_back( entry_empty );
				Log(1,L"      Entry added\n");
			}

		}// for each entry in the dictionary.

		// Close the dictionary and move on:
		fclose( fedict );
		Log(1,L"   Finished this dictionary; Closed 'fedict'.\n");
	}// for loop for dictionaries.

	Log(1,L"END: Search_English()\n");
	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
