// Search_Dict.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_DICT_H
#define SEARCH_DICT_H
///////////////////////////////////////////////////////////
// For the sort functions:
#include "EDICT_Sort.h"
#include "KANJIDIC_Sort.h"

// For searching:
#include "Search.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    Add the kanji in 'wstr' to the kanji-result-list. */
void KanjiWindow_SearchForKanjiIn( wstring wstr ) {
	for( int j = 0; j < wstr.size(); j++ ) {
		Uint16 character = wstr[j];
		// If it's not hiragana or katakana, then it could be kanji:
		if( !( IsHiragana(character) || IsKatakana(character) ) ) {
			KANJIDIC_Index_Entry *index = NULL;
			for( int k = 0; k < kanjidic_index_entries_size; k++ ) {
				if( kanjidic_index_entries[k].kanji == character ) {
					index = &kanjidic_index_entries[k];
					break;
				}
			}
			if( index != NULL ) {
				// Now we want to make sure that we haven't already added it:
				bool okaytoadd = true;
				for( int l = 0; l < kanjidic_results.size(); l++ ) {
					if( kanjidic_results[l].kanji == index->kanji ) {
						okaytoadd = false;
						break;
					}
				}
				// If it's new, then add it:
				if( okaytoadd ) {
					KANJIDIC_Entry entry;
						LoadKANJIDICInfoInto( entry, index );
					kanjidic_results.push_back( entry );
				}
			}//*/
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    Format 'searchtext' by putting the wildcard WC_STAR
	where it belongs (before/after/not-at-all) for the type
	of search specified. */
void AddWildcards( wstring &searchtext ) {
	if( search_style_location == SEARCH_STYLE_MIDDLE ) {
		if( searchtext[searchtext.size()-1] != WC_STAR ) {
			searchtext = searchtext + (Uint16)WC_STAR;
		}
		if( searchtext[0] != WC_STAR ) {
			searchtext = (Uint16)WC_STAR + searchtext;
		}
	} else if( search_style_location == SEARCH_STYLE_START ) {
		if( searchtext[searchtext.size()-1] != WC_STAR ) {
			searchtext = searchtext + (Uint16)WC_STAR;
		}
	} else if( search_style_location == SEARCH_STYLE_END ) {
		if( searchtext[0] != WC_STAR ) {
			searchtext = (Uint16)WC_STAR + searchtext;
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    Process 'searchtext', and then either call
   SearchFor() or SearchFor_English() with it. */
int Edict_SearchFor( wstring &searchtext ) {
	Log(1,L"Edict_SearchFor()\n");
	// If there's no text, then just quit:
	if( searchtext.size() == 0 ) {  return(-1);  }

	// NEW:2004.12.15
	// Are we forcing a type of search?
	int force_lang = registry.GetINT(".\\Search\\Language");
	if( searchtext.size() > 3 ) {
		if( searchtext.find(L"EN:") == (wstring::size_type)0 || searchtext.find(L"en:") == (wstring::size_type)0 ) {
		// Force English?
			searchtext = searchtext.substr( 3,-1 );
				force_lang = ID_LANG_EN;
		} else if( searchtext.find(L"JP:") == (wstring::size_type)0 || searchtext.find(L"jp:") == (wstring::size_type)0 ) {
		// Force Japanese?
			searchtext = searchtext.substr( 3,-1 );
				force_lang = ID_LANG_JP;
		}
	}

	// If we're allowing ROMAji conversion:
	if( !(force_lang==ID_LANG_EN) && search_conversion_romaji == SEARCH_CONVERSION_ROMAJI_TOKANA ) {
		// RomajiToKana converts the romaji to kana:
		searchtext = RomajiStringToKana( searchtext );
		Log(1,L"   Text converted to kana: %s.\n",searchtext.c_str() );
	}

	// Cap the search, if necessary:
	// (get the maximum number of results allowed):
	search_results_max = GetMaxResults();
	Log(1,L"   Maximum number of results: %i.\n",search_results_max );

#ifdef UBERDEBUG
	// There's no toolbar, so we need to make sure it searches:
	search_results_max = 200;
#endif


	// See how we're going to do the search.
	if( force_lang == ID_LANG_X ) {
		// We have to guess what kind of search it's going to be.

		// By now, if we were going to convert ROMAji to kana, it would have
		// already been done.  So, we just need to see if this is a ROMAji
		// search (English), or not (Japanese).

		// Now, we can check:
		if( IsRomaji(searchtext) ) {
			force_lang = ID_LANG_EN;
		} else {
			force_lang = ID_LANG_JP;
		}
	}


	// Actually do the search:
	int search_status = 0;
	if( force_lang == ID_LANG_EN ) {
		// FORCED ENGLISH SEARCH:
		search_status = SearchFor_English( searchtext );
	} else if( force_lang == ID_LANG_JP ) {
		// FORCED JAPANESE SEARCH:
		// NEW: 2004.11.21
		// Convert any ROMAji to Fullwidth ROMAji.
		for( int k = 0; k < searchtext.size(); k++ ) {
			if( IsRomaji(searchtext[k]) ) {  searchtext[k] = RomajiToFullwidth( searchtext[k] );  }
		}

		// If we're doing a fuzzy search (that conjugates, et cetera):
		if( registry.GetBOOL(".\\Search\\Toolbars\\Fuzzy") ) {
			vector<wstring> strings = FuzzySearch( searchtext );
			for( int s = 0; s < strings.size(); s++ ) {
				// Add wildcards to the text depending on what search options are on:
				AddWildcards( strings[s] );
				//Log(1,L"Searching for %s (from FuzzySearch)...\n",strings[s].c_str() );
				search_status = SearchFor( strings[s] );
			}//*/
		} else {
			// Otherwise, we're doing a normal search:

			// Add wildcards to the text depending on what search options are on:
			AddWildcards( searchtext );
			search_status = SearchFor( searchtext );
		}
	}
	Log(1,L"   Search complete.\n");

	// Sort the results:
	SortResults();
	Log(1,L"Results sorted.\n");

	return( search_status );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function should be called after a search is done.
   It:
	* Sorts the kanji results.
	* Clears the selected kanji.
	* Resets the scroll bar.
	* Tells it to redraw the window. */
void KanjiWindow_SetUpForNewResults() {
	// Sort the results:
	SortKanjiResults();

	// Reset the selection:
	kanji_selected = -1;

	// Set the bar's position:
	SCROLLINFO si;
		si.cbSize = sizeof(si); 
		si.fMask  = SIF_POS;
		si.nPos   = 0;
	SetScrollInfo( hwnd_kanjiwindow, SB_VERT, &si, TRUE);//*/

	// Redraw the kanji window:
	// Make sure that the scrollbar is correct:
	RECT rc;
		GetClientRect( hwnd_kanjiwindow,&rc );
	SendMessage( hwnd_kanjiwindow,WM_SIZE, 0,MAKELONG(rc.right,rc.bottom) );
	// Redraw it:
	RedrawWindow( hwnd_kanjiwindow, &rc, NULL, RDW_INVALIDATE );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function should be called after a search is done.
   It:
	* Sorts the edict results.
	* Clears the selected entries.
	* Resets the scroll bar.
	* Tells it to redraw the window. */
void ResultWindow_SetUpForNewResults() {
	// Sort the results:
	SortResults();

	// Redraw the result window:
	result_base = -1;
	result_last = -1; // none is selected, now.

	// Set the bar's position:
	SCROLLINFO si;
		si.cbSize = sizeof(si); 
		si.fMask  = SIF_POS;
		si.nPos   = 0;
	SetScrollInfo( hwnd_resultwindow, SB_VERT, &si, TRUE);//*/

	// Make sure that the scrollbar is correct:
	RECT rc;
		GetClientRect( hwnd_resultwindow,&rc );
	SendMessage( hwnd_resultwindow,WM_SIZE, 0,MAKELONG(rc.right,rc.bottom) );
	// Redraw it:
	RedrawWindow( hwnd_resultwindow, &rc, NULL, RDW_INVALIDATE );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////