// Search_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_WINDOW_H
#define SEARCH_WINDOW_H
///////////////////////////////////////////////////////////
// For the Rich Edit controls:
#include <Richedit.h>

// For the Search Window's Rebar:
#include "Search_Rebar.h"

// For converting ROMAji to kana:
#include "Conversion.h"

// For converting KANJIDIC's kanji to unicode:
#include "JISToUNI2.h"

// For copying text based on the combobox:
#include "CopyPaste.h"


#include <WinAble.h>
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring str_ime = L"";
int ime_start = -1;
bool ime_lock = false;
void StartIME() {
	str_ime = L"";
	SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&ime_start,NULL );
}
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Search_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	switch( message ) {
//////////////////////////////////////////
///                                    ///
///   W I N D O W    C R E A T I O N   ///
///                                    ///
//////////////////////////////////////////
		case WM_CREATE: {
			Log(1,L"Search Window: WM_CREATE:\n");

#ifndef UBERDEBUG
			hwnd_searchrebar = CreateSearch_Rebar( hWnd );

			RECT rcToolbar;
				GetWindowRect( hwnd_searchrebar, &rcToolbar );
#else
// NORMAL:
			RECT rcToolbar = { 0,0, 0,0 };
#endif

			RECT rc;
				GetClientRect( hWnd, &rc );
			rc.top += rcToolbar.bottom - rcToolbar.top;

			// Load the rich-edit library.
			//    We want to use Rich Edit 3.0 (which is in the
			// file "riched20.dll".
#ifdef VERSION_NT
			// In the NT version, load it only if we have to.
			if( !GetModuleHandle(L"RICHED20.DLL") ) {
				HMODULE hmod = LoadLibrary(L"RICHED20.DLL");
#else
			// In the non-NT version, load the one in the directory.
			{
				Log(L"(We want to load 'RICHED20.DLL').\n");
				// Load the one in the directory to be safe:
				LPTSTR cmdline = GetCommandLine();
				Log(L"   cmdline: %s\n",cmdline );
				wstring path = cmdline;
				int pos = path.find(L'\"');
				if( pos >= 0 ) {
					path = path.substr( pos+1 );

					pos = path.find_last_of(L'\\');
					if( pos >= 0 ) {
						path = path.substr( 0, pos );
					}
				}
				Log(L"   path: %s\n",path.c_str() );
				wstring str = path + L"\\RICHED20.DLL";
				HMODULE hmod = LoadLibrary(str.c_str());
#endif
				if( !hmod ) {
					Log(L"ERROR: Library 'RICHED20.DLL' not loaded.\n");
#ifndef VERSION_NT
					Log(L"   Attempting to use system's version...");
					if( !GetModuleHandle(L"RICHED20.DLL") ) {
						hmod = LoadLibrary(L"RICHED20.DLL");
						if( !hmod ) {
							Log(L"FAILED\n");
							MessageBox(hWnd,L"LoadLibrary(RICHED20.DLL) failed.",L"Error",MB_OK );
						} else {
							Log(L"OKAY\n");
						}
					}
#endif
				} else {
					Log(L"Library 'RICHED20.DLL' loaded sucessfully.\n");
				}
			}

			// Create the search edit-box:
			hwnd_searchedit = CreateWindowEx(
				WS_EX_CLIENTEDGE,
				RICHEDIT_CLASS, NULL,
            WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | 
            ES_LEFT | ES_MULTILINE,//|ES_NOHIDESEL,// | ES_MULTILINE|ES_WANTRETURN,// | ES_AUTOVSCROLL, 
            0,0, 0,0,  // set size in WM_SIZE message 
            hWnd,        // parent window 
            (HMENU)WID_SEARCH_EDIT, // edit control ID 
            (HINSTANCE)g_hinst, 
            NULL
			);
			// Make sure that it was created:
			if( !hwnd_searchedit ) {
				Log(L"ERROR: hwnd_searchedit not created!\n");
				MessageBox(NULL,L"hwnd_searchedit not created!",L"ERROR",MB_OK );
				exit(-1);
			}

			// Set its position:
			SetWindowPos( hwnd_searchedit, 0, rc.left,rc.top, rc.right-rc.left,rc.bottom-rc.top, SWP_NOZORDER|SWP_SHOWWINDOW );
			// Set the margins:
			SendMessage( hwnd_searchedit, EM_SETMARGINS, EC_LEFTMARGIN,2 );

			// Rich-edit options:
			// To receive EN_MSGFILTER notifications for events: 
			SendMessage( hwnd_searchedit, EM_SETEVENTMASK, 0, ENM_KEYEVENTS|ENM_MOUSEEVENTS|ENM_SELCHANGE );
			// Set it so that it's just normal, plain text:
			SendMessage( hwnd_searchedit, EM_SETTEXTMODE, TM_PLAINTEXT, 0 );

			// Basically, set the font and stuff:
			CHARFORMAT2 charformat;
				charformat.cbSize = sizeof(CHARFORMAT2);
				charformat.dwEffects = 0;
				charformat.bPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
				charformat.dwMask = CFM_FACE | CFM_SIZE | CFM_COLOR | CFM_CHARSET | CFM_WEIGHT;
				// CFM_FACE:
				wcscpy(charformat.szFaceName, ToWString( registry.GetSTRING(".\\Search\\Font\\Font") ).c_str() );
				// CFM_SIZE:
				charformat.yHeight = registry.GetINT(".\\Search\\Font\\Size") * 20; // NOTE: 20 is a constant which converts 1/1440 of an inch to 1/72 of an inch (pt size).
				// CFM_COLOR:
				charformat.crTextColor = registry.GetINT(".\\Search\\Font\\Color");
				// CFM_CHARSET:
				charformat.bCharSet = DEFAULT_CHARSET;//SHIFTJIS_CHARSET;//DEFAULT_CHARSET;
				// CFM_WEIGHT:
				charformat.wWeight = FW_REGULAR;
			SendMessage( hwnd_searchedit, EM_SETCHARFORMAT, (WPARAM)SCF_ALL, (LPARAM)&charformat );

			// Set the background color of the window:
			SendMessage( hwnd_searchedit, EM_SETBKGNDCOLOR, FALSE, registry.GetINT(".\\Search\\WindowBG") );


			// Put in some text:
			Uint16 starttext[2048];
			if( registry.GetBOOL(".\\Search\\ShowInitialText") ) {
				swprintf(starttext,L"\
   Press CAPSLOCK to toggle the IME.\n\
   English does English searches; Japanese does Japanese searches.\n\
\n\
      Japanese searches allow wildcards: _ (any number of characters), ? (just one character).\n\
\n\
      English searches behave like Google searches.  Start the search with \"s to search for EXACTLY that text\n\
\n\
Check for updates & important news often: Options-->Check for Updates.\
");
			} else {
				swprintf(starttext,L"Search for...");
			}
			SendMessage( hwnd_searchedit, WM_SETTEXT, 0, (LPARAM)starttext );

			Log(1,L"END: Search Window: WM_CREATE:\n");
			return(0);
		}
		case WM_SIZE: {
			int neww = LOWORD(lParam), newh = HIWORD(lParam);

#ifndef UBERDEBUG
			// Fix the toolbar window automatically:
			if( neww > 800 ) {
				SendMessage( hwnd_searchrebar, WM_SIZE, wParam,lParam );
			}
			// (I don't want it to mess up or anything)

			// Fix the edit window:
			RECT rcToolbar;
				GetWindowRect( hwnd_searchrebar, &rcToolbar );
#else
			RECT rcToolbar = {0,0,0,0};
#endif
			RECT rcWindow;
				GetWindowRect( hWnd, &rcWindow );
			SetWindowPos( hwnd_searchedit, NULL,
				0,rcToolbar.bottom-rcToolbar.top,
				neww, newh-(rcToolbar.bottom-rcToolbar.top-1),
				SWP_NOZORDER|SWP_SHOWWINDOW
			);
			return(0);
		}
///////////////////////////////////
///                             ///
///   N O T I F I C A T I O N   ///
///                             ///
///////////////////////////////////
		case WM_NOTIFY: {
			// If the notification is from the edit-control:
			if( wParam == WID_SEARCH_EDIT ) {
				NMHDR *hdr = (NMHDR*)lParam;
				switch( hdr->code ) {
					case EN_MSGFILTER: {
						MSGFILTER *mf = (MSGFILTER*)lParam;
						switch( mf->msg ) {
							case WM_MOUSEMOVE: {
							/* If the mouse moves over the edit window (in the search window),
							   then select all of the text automatically. */
								HWND hwnd_focus = GetFocus();
								if( GetForegroundWindow()==hwnd_mainwindow && hwnd_focus != hwnd_searchedit ) {
									HWND hwnd_old = SetFocus( hwnd_searchedit );
									if( hwnd_old == NULL ) {
										Log(L"SetFocus(): ERROR: %u\n",GetLastError() );
									}
									SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
								}
								break;
							}
							case WM_RBUTTONDOWN: {
							/* If the right mouse button is clicked, bring up a menu:
									Undo [action]
									Copy
									Cut
									Paste
								(It uses SendMessage() to do all of these). */
								SHORT mx = LOWORD(mf->lParam), my = HIWORD(mf->lParam);

								RECT rcWindow;
									GetWindowRect(hdr->hwndFrom,&rcWindow);

								/////////////////////////
								///   Make the menu   ///
								/////////////////////////
								HMENU hmenu = CreatePopupMenu();
								// UNDO:
									wstring strundo = L"&Undo";
									int undoflags = 0;
									int undoaction = SendMessage( hdr->hwndFrom, EM_GETUNDONAME, 0,0 );
									switch( undoaction ) {
										case UID_TYPING: {  strundo += L" typing"; break;  }
										case UID_DELETE: {  strundo += L" delete"; break;  }
										case UID_DRAGDROP: {  strundo += L" drag and drop"; break;  }
										case UID_CUT: {  strundo += L" cut"; break;  }
										case UID_PASTE: {  strundo += L" paste"; break;  }
									} 
									if( SendMessage( hdr->hwndFrom,EM_CANUNDO, 0,0 ) == 0 ) {
										undoflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|undoflags, 1, strundo.c_str() );
										AppendMenu( hmenu, MF_SEPARATOR, NULL,NULL );
								// COPY, CUT:
									int copyflags = 0;
									if( SendMessage( hdr->hwndFrom,EM_SELECTIONTYPE, 0,0 ) == SEL_EMPTY ) {
										copyflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|copyflags, 2, L"&Copy" );
									AppendMenu( hmenu, MF_STRING|copyflags, 3, L"C&ut" );
								// PASTE:
									int pasteflags = 0;
									if( SendMessage( hdr->hwndFrom,EM_CANPASTE, 0, 0 ) == 0 ) {
										pasteflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|pasteflags, 4, L"&Paste" );								

								///////////////////////////
								///   Handle the Menu   ///
								///////////////////////////
								int result = TrackPopupMenuEx( hmenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );
								if( result == 1 ) {
								// UNDO:
									SendMessage( hdr->hwndFrom, WM_UNDO, 0,0 );
								} else if( result == 2 ) {
								// COPY:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
								} else if( result == 3 ) {
								// CUT:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
									SendMessage( hdr->hwndFrom, EM_REPLACESEL, TRUE, (LPARAM)L"" );
								} else if( result == 4 ) {
								// PASTE:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_PASTE,0), 0 );
								}

								return(0);
							}
							case WM_KEYDOWN: {
								if( mf->wParam == VK_RETURN ) {
								/* Do a SEARCH when he presses ENTER.
									However, if SHIFT is down, then insert a line break. */
									// If shift is down, just stop:
									if( GetKeyState(VK_SHIFT)&0x8000 ) {  ime_start++;  return(0);  }
									// Otherwise, do the search:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_SEARCH_GO,0), 0 );
									
									if( ime_start != -1 ) {  StartIME();  }
									
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								} else if( mf->wParam == VK_v && GetKeyState(VK_CONTROL)&0x8000 ) {
								// If it's CTRL+V (paste):
								// Convert the text to Unicode and paste it.
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_PASTE,0), 0 );
									
									if( ime_start != -1 ) {  StartIME();  }

									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								} else if( mf->wParam == VK_c && GetKeyState(VK_CONTROL)&0x8000 ) {
								// If it's CTRL+C (copy):
								// Copy the text to whatever format is in the box:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
									
									if( ime_start != -1 ) {  StartIME();  }

									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								} else if( mf->wParam == VK_CAPITAL ) {
								//////////////////////////
								///   TOGGLE THE IME   ///
								//////////////////////////
								// If it's CAPSLOCK:
								// Start/Stop the IME:
									static bool booly = false;
									if( !booly ) {
										if( ime_start == -1 ) {
											StartIME();
											SetStatusBarText( 1,L"IME -on-" );
										} else {
											ime_start = -1;
											SetStatusBarText( 1,L"IME -off-" );
										}
										booly = true;

										// Capslock is down, so release it:
										keybd_event(VK_CAPITAL, 0xBA, KEYEVENTF_KEYUP, 0);
										// And then press it again:
										keybd_event(VK_CAPITAL, 0xBA, 0, 0);
									} else {
										booly = false;
									}
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								}
								break;
							}
							case WM_CHAR: {
								if( mf->wParam == L'\t' ) {
								// Switch windows when 'tab' is pressed:
									SetFocus( hwnd_kanjiwindow );			
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								}
								if( ime_start != -1 && mf->wParam == L'\b' ) {
								// Backspace:
									if( str_ime.size() > 0 ) {
										str_ime = str_ime.substr(0,str_ime.size()-1 );
									} else {
										// Do nothing.
										// The selection-change message will fix it.
									}
								}
								if( ime_start != -1 && IsRomaji(mf->wParam) ) {
									// Add the character to the IME:
									str_ime += mf->wParam;

									// Make sure that 'n's are done properly.
									// (Take the double n and replace it with n').
									if( str_ime.size() >= 2 && str_ime.find(L"nn") == str_ime.size()-2 ) {  str_ime[str_ime.size()-1] = L'\'';  }
									if( str_ime.size() >= 2 && str_ime.find(L"NN") == str_ime.size()-2 ) {  str_ime[str_ime.size()-1] = L'\'';  }

									wstring str2 = RomajiStringToKana( str_ime, false );

									/*Log(L"::IME::\n");
									Log(L"   str_ime: %s\n",str_ime.c_str() );
									Log(L"   str2   : %s\n",str2.c_str() );
									Log(L"   ime_start: %i\n",ime_start );//*/

									ime_lock = true;

									// If there's a selection, we need to:
									// 1) Clear it
									// 2) Make IME_START the START of the ex-selection.
									int sel_start, sel_stop;
									SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );

									if( sel_start != sel_stop ) {
										// Clear what's selected.
										SETTEXTEX st;
											st.flags = ST_KEEPUNDO|ST_SELECTION;
											st.codepage = 1200; // 1200 == unicode.
										SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)L"" );

										// Change ime_start:
										ime_start = sel_start;
									}


									// Select from ime_start until str_ime.size().
									SendMessage( hwnd_searchedit, EM_SETSEL, ime_start, ime_start+str_ime.size()-1 );


									// Replace that text with str2.
									SETTEXTEX st;
										st.flags = ST_KEEPUNDO|ST_SELECTION;
										st.codepage = 1200; // 1200 == unicode.
									SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)str2.c_str() );

									// Separate the kana from the romaji in str2.
									wstring kana = L"";
									for( int i = 0; i < str2.size(); i++ ) {
										if( !IsRomaji(str2[i]) ) {
											kana += str2[i];
										} else {
											break;
										}
									}
									str_ime = str2.substr( kana.size(), -1 );

									/*Log(L"   str_ime: %s\n",str_ime.c_str() );//*/

									// ime_start = ime_start + # of kana.
									ime_start += kana.size();

									/*Log(L"   str_ime: %s\n",str_ime.c_str() );
									Log(L"   str2   : %s\n",str2.c_str() );
									Log(L"   kana   : %s (%i)\n",kana.c_str(), kana.size() );
									Log(L"   ime_start: %i\n",ime_start );//*/


									ime_lock = false;

									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								}
								return(0);
								break;
							}
						}// switch(mf->msg)
						break;
					} // case EN_MSGFILTER
					case EN_SELCHANGE: {
						if( ime_lock ) {  break;  }
						if( ime_start == -1 ) {  break;  }

						SELCHANGE *sc = (SELCHANGE*)lParam;
		
						/*Log(L"New: %i, %i\n",sc->chrg.cpMin,sc->chrg.cpMax );//*/

						// If the change in selection is NOT the NATURAL change that
						// would result from typing, then RESET the IME.
						if( sc->chrg.cpMax != ime_start + str_ime.size() ) {
							/*Log(L"******\n");//*/
							StartIME();
							if( sc->chrg.cpMin == 0 ) {
								ime_start = 0;
							} else {
								ime_start = sc->chrg.cpMin;//+1;
							}
						}
						break;
					}// case EN_SELCHANGE
				} // switch hdr->code

			} else if( wParam == WID_SEARCH_REBAR ) {
				NMHDR *hdr = (NMHDR*)lParam;
				switch( hdr->code ) {
					case RBN_AUTOSIZE: {
						//MessageBox(NULL,L"DING",L"!",MB_OK );
						NMRBAUTOSIZE *nas = (NMRBAUTOSIZE*)lParam;
						nas->fChanged = FALSE;
						break;
					}
				}
			} else {
				HandleTooltips(lParam);
			}
			return(0);
		}
////////////////////////////////////////////
///                                      ///
///   C O M M A N D    M E S S A G E S   ///
///                                      ///
////////////////////////////////////////////
		case WM_COMMAND: {
			/* wParam
					The high-order word specifies the notification code if the message is from a control.
					If the message is from an accelerator, this value is 1.
					If the message is from a menu, this value is zero. 
					
					The low-order word specifies the identifier of the menu item, control, or accelerator. 

				lParam
					Handle to the control sending the message if the message is from a control. Otherwise, this parameter is NULL. 
			//*/

			// For the Edit Control:
			// lParam is who sent it (hande).
			// LOWORD(wParam) is who sent it (id).
			// HIWORD(wParam) is the notification message.

			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				case ID_SEARCH_START: {
					search_style_location = SEARCH_STYLE_START;
					break;
				}
				case ID_SEARCH_MIDDLE: {
					search_style_location = SEARCH_STYLE_MIDDLE;
					break;
				}
				case ID_SEARCH_END: {
					search_style_location = SEARCH_STYLE_END;
					break;
				}
				case ID_SEARCH_WHOLE: {
					search_style_location = SEARCH_STYLE_WHOLE;
					break;
				}

				case ID_MATCHSUBSTRINGS: {
					search_matchsubstrings ^= 1;
					break;
				}
				case ID_FORCEWORDSEARCH: {
					search_forcewordsearch ^= 1;
					break;
				}
				case ID_FUZZY: {
					bool fuzzy = registry.GetBOOL(".\\Search\\Toolbars\\Fuzzy");
					registry.SetBOOL(".\\Search\\Toolbars\\Fuzzy", !fuzzy );
					break;
				}

				case ID_SEARCH_CONVERSION_ROMAJI: {
					search_conversion_romaji ^= 1;
					break;
				}
				case ID_SEARCH_CONVERSION_KANA: {
					search_conversion_kana ^= 1;
					break;
				}

				case ID_SEARCH_CONVERSION_SHOW: {
					search_conversion_show ^= 1;
					break;
				}

				// The Language buttons:
				case ID_LANG_JP:
				case ID_LANG_EN:
				case ID_LANG_X: {
					// If the button was just clicked, then he's our boy:
					if( code == BN_CLICKED ) {
						registry.SetINT(".\\Search\\Language", identifier );
					}
					break;
				}

				case WID_SEARCH_COMBOBOX: {
					switch( HIWORD(wParam) ) {
						case CBN_SELENDOK: {
							// If he clicks on <<Something>>, make the selection the one under that.
							int encoding = SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 );
							int index = -1;
							for( int i = 0; i < encoding_array_size; i++ ) {
								if( encoding_array[i].combobox_index == encoding ) {
									index = i;
									break;
								}
							}
							if( i == -1 ) {
								Log(L"ERROR: The combobox's index was not found.\n");
							} else {
								if( encoding_array[i].iconv_string == NULL ) {
									SendMessage( hwnd_searchrebar_combobox_encode,CB_SETCURSEL, (WPARAM)(encoding+1), 0 );
								}
							}
							break;
						}
					}
					break;
				}

				case ID_ENCODE_PASTE: {
				// Convert the text (from the format in the combobox) to Unicode (for the program).
				// Note: this copies the text on the Clipboard and pastes it in place of the selection.
				/* IMPORTANT:
					The clipboard is interesting in some respects.  For example if I copy the word:
					   T[syatu]
					most programs will copy the text in Unicode format for Windows.  Now,
					because it's unicode, the program will put it in CF_UNICODETEXT format.  BUT,
					also, to be nice to non-unicode programs, it will ALSO put it in CF_TEXT format,
					looking like:
					   T???
					This way, CF_TEXT CANNOT be used reliably 
				*/
					// Get the desired encoding (index):
					int encoding = SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 );
					int index = -1;
					for( int i = 0; i < encoding_array_size; i++ ) {
						if( encoding_array[i].combobox_index == encoding ) {  index = i;  break;  }
					}

					wstring newtext;

					// As long as the index is valid, convert it:
					if( index >= 0 ) {						
						// index 1 is 'Normal Unicode' (no processing).
						if( index == 1 && IsClipboardFormatAvailable(CF_UNICODETEXT) ) {
							// Just make sure that it's plain-text:
							// Get the text that he wants to paste:
							OpenClipboard( NULL );
								HGLOBAL hglobal = GetClipboardData( CF_UNICODETEXT );
								Uint16* paste_text = (Uint16*)GlobalLock(hglobal);
									//wcscat( paste_text, L"\0" );
							CloseClipboard();
	
							newtext = paste_text;
						} else if( IsClipboardFormatAvailable(CF_TEXT) ) {
							// Get the text that he wants to paste:
							OpenClipboard( NULL );
								HGLOBAL hglobal = GetClipboardData( CF_TEXT );
								char* paste_text = (char*)GlobalLock( hglobal );
									//strcat(paste_text,"\0");
							CloseClipboard();
	
							// Convert the text to Unicode so we can display it:
							ConvertEncoding( paste_text,strlen(paste_text), encoding_array[index].iconv_string, "UTF-16LE", newtext );
						} else {
							//MessageBox(hWnd,TEXT("CF_TEXT format is not availabe."),TEXT("Warning"),MB_OK );	
						}
					} else {
						Log(L"ID_ENCODE_PASTE: Error: index out of bounds (index: %i).\n",index );
					}
					// Replace the selection with the new text (now in unicode format):
					//SendMessage( hwnd_searchedit,EM_REPLACESEL, (WPARAM)TRUE, (LPARAM)newtext.c_str() );//*/
					SETTEXTEX st;
						st.flags = ST_KEEPUNDO|ST_SELECTION;
						st.codepage = 1200; // 1200 == unicode.
					SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)newtext.c_str() );
					return(0);
				}
				case ID_ENCODE_COPY: {
				/* Copy the text to the format as specified by the combobox:
				      If the option is 'Normal Unicode', then don't do any conversion and
					just copy it in the CF_UNICODETEXT format.
				*/
					// Get the text from the edit control:
					int sel_start,sel_stop;
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					// If something's selected, copy it:
					if( sel_stop-sel_start > 0 ) {
						Uint16 *tcharsearchtext = new Uint16[sel_stop-sel_start+1];
						SendMessage(hwnd_searchedit,EM_GETSELTEXT,0,(LPARAM)tcharsearchtext );
						wstring searchtext = tcharsearchtext;
							delete [] tcharsearchtext;
					
						// If there's nothing to copy, just stop.
						if( searchtext.size() <= 0 ) {	return(0);	}

						// Use a nice function to do the copying:
						CopyTextUsingComboboxEncoding( searchtext );
					}
					return(0);
				}
				case ID_SEARCH_GO: {
				// Search for the text in the search window:
	
					// search_lock is the new way to lock searches.
					// thread_searching was good, but it let a search START when another
					// one was still GOING ON.  Now, each search locks itself as soon as it
					// starts, and it releases it when it's done.
					if( search_lock ) {
						Log(1,L"WARNING :: SEARCH_LOCK :: (aborting new search)\n");
						break;
					} else {
						search_lock = true;
					}


					// As of now, this search is locked.


					// If a dictionary is loading in a separate thread, cancel the search.
					// We don't want to be searching for something when the dictionaries
					// are still being written to.
					//    OR
					// If a search is already underway, stop.
					if( thread_searching ) {
						//MessageBox(hWnd,L"Please finish the current search first or\nlet the dictionary finish loading.\n",L"Note",MB_OK );
						Log(1,L"WARNING :: THREAD_SEARCHING :: (aborting new search)\n");
						search_lock = false;
						break;
					}

					Log(1,L"************************\n");
					Log(1,L"***   ID_SEARCH_GO   ***\n");
					Log(1,L"************************\n");

					// Set the status bar text:
					SetStatusBarText(L"Searching...");
					Log(1,L"   Status bar text set.\n");

					// Get the text from the edit control:
					int sel_start,sel_stop;
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					// If something isn't selected, select everything:
					if( sel_stop-sel_start <= 0 ) {
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					}
					Log(1,L"   Selection: %i--%i.\n",sel_start,sel_stop);

					Uint16 *tcharsearchtext = NULL;
					if( sel_stop-sel_start > 0 ) {
						tcharsearchtext = new Uint16[sel_stop-sel_start+1];
							wcsnset( tcharsearchtext, L'\0', sel_stop-sel_start+1 );
						SendMessage(hwnd_searchedit,EM_GETSELTEXT,0,(LPARAM)tcharsearchtext );
					} else {
						SetStatusBarText(L"(no text to search for)");
						search_lock = false;
						break;
					}

					// Shave off all of the spaces at the end.
					// This is important for pasting & searching (because most copies keep a trailing space).
					while( wcslen(tcharsearchtext) > 1 && tcharsearchtext[wcslen(tcharsearchtext)-1] == L' ' ) {
						tcharsearchtext[wcslen(tcharsearchtext)-1] = L'\0';
					}
					Log(1,L"   Text exists: %s\n",tcharsearchtext);

					wstring searchtext = tcharsearchtext;
					// Clean up the old buffer:
					if( tcharsearchtext ) {  delete [] tcharsearchtext;  }
					// Is there a problem?
					if( searchtext.size() == 0 ) {
						SetStatusBarText(L"Search text error");
						search_lock = false;
						break;
					}


					// Keep the original text for reference:
					wstring searchtext_old = searchtext;


					// Get rid of end lines and carriage returns:
					while( true ) {
						int pos = searchtext.find(L'\n');
						if( pos >= 0 ) {  searchtext.erase( searchtext.begin()+pos );  }
						else {  break;  }
					}
					while( true ) {
						int pos = searchtext.find(L'\r');
						if( pos >= 0 ) {  searchtext.erase( searchtext.begin()+pos );  }
						else {  break;  }
					}


					// Thread search time!
					THREAD_SEARCH ts;
						ts.search_text = &searchtext;
						ts.mode = TS_NORMAL;
					// Start searching (thread):
					HANDLE hthread = Load_Thread_Search( &ts );
					Log(1,L"   Load_Thread_Search() called.\n");

					// Sit around for a bit until:
					// 1) The thread finishes, or
					// 2) He presses ESCAPE.
					enum {  SEARCH_ABORT, SEARCH_DONE, SEARCH_QUIT  };
					int wtf = -1;
					MSG msg;
					while( true ) {
						if( !thread_searching ) {  wtf = SEARCH_DONE;  break;  }

						// Check to see if we have an ESCAPE key message.
						PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
						TranslateMessage(&msg);
						if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  wtf = SEARCH_ABORT;  break;  }
						if( msg.message == WM_QUIT ) {  wtf = SEARCH_QUIT;  break;  }
						DispatchMessage(&msg);

						Sleep( 10L );
					}
					if( DestroyWindow(hwnd_searching) ) {
						hwnd_searching = NULL;
						Log(1,L"   hwnd_searching destroyed.\n");
					} else {
						int error = GetLastError();
						Log(L"ERROR: hwnd_searching NOT destroyed: Error Code %i.\n",error);
					}

					if( wtf != SEARCH_DONE ) {
					//////////////////////////
					///   SEARCH ABORTED   ///
					//////////////////////////
						TerminateThread( hthread, 0L );
						CloseHandle(hthread);
						thread_searching = false;

							// Kanji:
							kanjidic_results.clear();
							// Go through the edict results looking for kanji:
							for( int i = 0; i < edict_results.size(); i++ ) {
								KanjiWindow_SearchForKanjiIn( edict_results[i].word );
							}

						// Select the text for ease.
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ||
							 searchtext_old.find(L"JP:") == 0 || searchtext_old.find(L"jp:") == 0 ) {
							SendMessage( hwnd_searchedit, EM_SETSEL, 3,-1 );
						}

						// Set up & redraw the kanji window:
						// (this also sorts them).
						KanjiWindow_SetUpForNewResults();

						// Set up & redraw the result window:
						// (this also sorts them).
						ResultWindow_SetUpForNewResults();

						// Update the status bar text:
						TCHAR tsearchbar[400];
						swprintf(tsearchbar,L"Search aborted (%i results)",edict_results.size()	);
						SetStatusBarText( tsearchbar );


						if( wtf == SEARCH_QUIT ) {
							SendMessage(hwnd_mainwindow,WM_QUIT,0,0);
							search_lock = false;
							return(0);
						}
						search_lock = false;
						break;
					} else {
					/////////////////////////////
					///   SEARCH SUCCESSFUL   ///
					/////////////////////////////
						CloseHandle(hthread);
						Log(1,L"      SEARCH COMPLETE!\n");
						Log(1,L"      hthread :: Handle closed.\n");

						int search_status = thread_search_exitcode;
						//searchtext = thread_search_text;
						// We'll let searchtext be updated by the thread function...

						// Set up & redraw the kanji window:
						// (this also sorts them).
						KanjiWindow_SetUpForNewResults();
						Log(1,L"      Kanji Window :: Set up for new results.\n");

						// Set up & redraw the result window:
						// (this also sorts them).
						ResultWindow_SetUpForNewResults();
						Log(1,L"      Result Window :: Set up for new results.\n");


						if( !search_conversion_show ) {
							searchtext = searchtext_old;
						}
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ||
							 searchtext_old.find(L"JP:") == 0 || searchtext_old.find(L"jp:") == 0) {
							searchtext = searchtext_old;
						}


						// Replace the selection with the search text:
						SETTEXTEX st;
							st.flags = ST_KEEPUNDO|ST_SELECTION;
							st.codepage = 1200; // 1200 == unicode.
						SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)searchtext.c_str() );
						Log(1,L"      Search window text set (%s).\n",searchtext.c_str() );

						//	Select the text for ease.
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ||
							 searchtext_old.find(L"JP:") == 0 || searchtext_old.find(L"jp:") == 0) {
							SendMessage( hwnd_searchedit, EM_SETSEL, 3,-1 );
						} else {
							SendMessage( hwnd_searchedit, EM_SETSEL, sel_start,sel_stop );
						}

						// Update the status bar text:
						Uint16 tsearchbar[400];
						swprintf(tsearchbar,L"Search complete (%i results) %s",edict_results.size(),
							search_status&SEARCH_STATUS_TRUNCATED?L"-Results truncated-":L""
						);
						SetStatusBarText( tsearchbar );//*/
						Log(1,L"      Status bar text set.\n");

					}// (search successful)
					search_lock = false;
					break;
				}// case ID_SEARCH_GO
			}
			return(0);
		}//*/
/////////////////////
///               ///
///   P A I N T   ///
///               ///
/////////////////////
		case WM_ERASEBKGND: {
			return(TRUE);
		}
	}
/////////////////////////
///                   ///
///   D E F A U L T   ///
///                   ///
/////////////////////////
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateSearch_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS            wndClass; 
	   wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.lpfnWndProc    = WndProc_Search_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("SearchWindow");
   
   RegisterClass(&wndClass);
   
   HWND hWnd = CreateWindow(
      TEXT("SearchWindow"),   // window class name
      TEXT("(SearchWindow)"),  // window caption
      WS_CHILD|WS_VISIBLE,// |WS_BORDER/*|WS_OVERLAPPED*//*|WS_SIZEBOX*/,      // window style
		x,y,
		w,h,
		hwndParent, // parent window handle
      NULL, // window menu handle
      g_hinst, // program instance handle
      NULL // creation parameters
	);

	if( !hWnd ) {
		Log(L"ERROR: could not create Search Window!\n");
		MessageBox(NULL,L"Could not create Search Window",L"ERROR",MB_OK );
		exit(-1);
	}
	  
   //ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////