// xwcscmp.h | Compiled by Doug Manley
///////////////////////////////////////////////////////////
//    This is basically just the shell for whatever
// algorithm that I decide to use for string comparison.
// The last two were utter failures (yeah, you know who
// you are), so now we're on the third.
///////////////////////////////////////////////////////////
#ifndef XWCSCMP
#define XWCSCMP
///////////////////////////////////////////////////////////
// These are the characters that we use as wildcards.
#define WC_QUES 0xFF1F
#define WC_STAR 0xFF3F
///////////////////////////////////////////////////////////
//    This algorithm is from ROBODoc, whatever that may be,
// and I found it here:
// http://www.xs4all.nl/~rfsber/Robo/Manual/Multidoc/util_c.html
// Thanks for the good code, guys!
///////////////////////////////////////////////////////////
int RB_Match( Uint16* target, Uint16* wildcard_expression )
{
    if ( ( *wildcard_expression == L'\0' ) &&
         ( *target == L'\0' ) ) 
    {
        /* a match, since both strings are now "" */
        return TRUE;
    }
    else if ( *wildcard_expression == L'\0' ) 
    {
        /* we reached the end of the wildcard_expression,
         * but not the end of the target, this is not
         * a match.
         */
        return FALSE;
    }
    else if ( *target == L'\0' )
    {
        /* we reached the end of the target but not the end of the
         * wildcard_expression.  Only if the whole wildcard_expression
         * consists of * we have a match.
         */
        unsigned int i;
        for ( i = 0; i < wcslen( wildcard_expression ); ++i )
        {
            if ( wildcard_expression[ i ] != WC_STAR )
            {
                return FALSE;
            }
        }
        return TRUE;
    }
    else
    {
        /* There are wildcard_expression characters left
         * and target characters left.
         */
        Uint16 wildcard = wildcard_expression[ 0 ];

        if ( wildcard == WC_QUES )
        {
            /* Match a single character and see if the
             * rest of the target matches.
             */
            return RB_Match( target + 1, wildcard_expression + 1 );
        }
        else if ( wildcard == WC_STAR )
        {
            int match = FALSE;
            int l = wcslen( target );
            int i;
            /* First try to match all of the target string, and
             * then work back to the begin of the target string.
             * ( including the "" string. )
             */
            for ( i = l; i >= 0; --i )
            {
                if ( RB_Match( target + i, wildcard_expression + 1 ) )
                {
                    match = TRUE;
                    break;
                }
            }
            return match;
        }
        else 
        {
            int l_w = wcslen( wildcard_expression );
            int l_t = wcslen( target );
            /* The minimum of the length of the wildcard_expression
             * and target expression 
             */
            int l = ( l_w <= l_t ) ? l_w : l_t ;
            int i;
            for ( i = 0; i < l; ++i )
            {
                if( ( wildcard_expression[ i ] != WC_STAR ) &&
                    ( wildcard_expression[ i ] != WC_QUES ) )
                {
                    /* Some OS-es are not case-sensitive when it comes
                     * to file names, and consider Readme to be equal
                     * to README.  On these OS-es it can be handy if
                     * robodoc is also not case-sensitive.
                     */
#ifdef IGNORE_CASE_FILENAMES
                    if ( tolower( wildcard_expression[ i ] ) != tolower( target[ i ] ) )
#else
                    if ( wildcard_expression[ i ] != target[ i ] )
#endif
                    {
                        return FALSE;
                    }
                }
                else
                {
                    return RB_Match( target + i, wildcard_expression + i );
                }
            }
            /* The first l characters of the target and
             * wildcard_expression matched, now see if the rest
             * matches too.
             */
            return RB_Match( target + l, wildcard_expression + l );
        }
    }
}

#endif