// FuzzySearch.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef FUZZYSEARCH_H
#define FUZZYSEARCH_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
struct FuzzySearch_Info {
	wstring expression;
	vector<wstring> replace;
};
vector<FuzzySearch_Info> fuzzysearch_info;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This loads the fuzzy search info from the fuzzy search
   data file, "fuzzysearch.uni.dat".  If that file can't be
	opened, then it's okay.
*/
bool LoadFuzzySearchInfo() {
	Log(1,L"LoadFuzzySearchInfo()\n");
	FILE *f = fopen("fuzzysearch.uni.dat","rb");
	if( !f ) {
		Log(L"ERROR: LoadFuzzySearchInfo(): Couldn't open 'fuzzysearch.uni.dat'.\n");
		return(false);
	}

	Uint16 line[1024];
	while( !feof(f) ) {
		/////////////////////////////////////
		///   READ AND PROCESS THE LINE   ///
		/////////////////////////////////////
		// Set the line to be all L'\0's.
		wcsnset(line,L'\0',1024);
		// Read in a line:
		fgetws(line, 1024-1, f );
			//Log(1,L"   line: [%s]\n",line );
		if( feof(f) ) {  break;  }
			//Log(1,L"   It is not eof.\n");

		if( line[0] == L'#' || line[0] == 0xFEFF ) {  continue;  }
			//Log(1,L"   line has useful data.\n");

		if( line[wcslen(line)-1] == L'\n' ) {  line[wcslen(line)-1] = L'\0';  }
		if( line[wcslen(line)-1] == L'\r' ) {  line[wcslen(line)-1] = L'\0';  }

			//Log(1,L"   line: [%s]\n",line );

		///////////////////////////////////////
		///   GET THE INFORMATION FROM IT   ///
		///////////////////////////////////////
		wstring str = line;

		int pos = str.find(L':');
		if( pos > 0 ) {
			FuzzySearch_Info info;
			// Set its expression (the thing with the wildcards):
				info.expression = str.substr( 0, pos );
					//Log(1,L"   info.expression: %s\n",info.expression.c_str() );

			// Set it's replacements:
			str = str.substr( pos+1, -1 );
			while( true ) {
				pos = str.find(L',');

				wstring substr = str.substr( 0, pos );
				info.replace.push_back( substr );
					//Log(1,L"   info.replace[%i]: %s\n",info.replace.size()-1, info.replace[info.replace.size()-1].c_str() );

				if( pos < 0 ) {  break;  }

				str = str.substr( pos+1, -1 );
			}

			// Add it to the global vector:
			fuzzysearch_info.push_back( info );
		} else {
			Log(L"WARNING: LoadFuzzySearchInfo(): Couldn't find a ':' on this line: %s\n",line );
		}
	}

	fclose(f);
	Log(1,L"All done.\n");
	return(true);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*    This returns a vector of strings that are possible
   bases for the text that's passed to it.  You can use
	these strings for searching afterward.
*/
vector<wstring> FuzzySearch( wstring &searchtext ) {
	Log(L"FuzzySearch(%s)...\n",searchtext.c_str() );
	vector<wstring> strings;

	// One possible result is the text that we started with:
	strings.push_back( searchtext );

	// Go through every one of the fuzzy endings:
	for( int i = 0; i < fuzzysearch_info.size(); i++ ) {
		FuzzySearch_Info *info = &fuzzysearch_info[i];

		//Log(1,L"Checking: [%s] (%i) & [%s] (%i)...", searchtext.c_str(), searchtext.size(), info->expression.c_str(), info->expression.size() );
		if( StringsMatch( searchtext, info->expression ) ) {
				//Log(1,L"OKAY\n");
			if( info->expression[0] == WC_STAR ) {
					//Log(1,L"FUZZYSEARCH: TEXT: %s\n",searchtext.c_str() );
					//Log(1,L"FUZZYSEARCH: EXPRESSION: %s\n", info->expression.c_str() );
				wstring expr = info->expression.substr(1,-1);
					//Log(1,L"FUZZYSEARCH: EXPR: %s\n", expr.c_str() );

				// Find the final position where they match:
				int pos = -1;
				while( true ) {
					int test;
					if( pos == -1 ) {	test = searchtext.find( expr );  }
					else {				test = searchtext.find( expr, pos+1 );  }
					if( test == -1 ) {	break;  }
					else {					pos = test;  }
				}
				if( pos >= 0 ) {
					wstring part = searchtext.substr( 0, pos );
						//Log(1,L"FUZZYSEARCH: PART: %s\n",part.c_str() );

					for( int r = 0; r < info->replace.size(); r++ ) {
						if( info->replace[r][0] == WC_STAR ) {
							wstring newtext = part + info->replace[r].substr(1,-1);

							// We know that 'newtext' is a valid fuzzysearch result
							// of the search.  However, we also have to check IT for
							// fuzziness.
							vector<wstring> newstrings = FuzzySearch( newtext );
							for( int x = 0; x < newstrings.size(); x++ ) {
								strings.push_back( newstrings[x] );
								Log(1,L"   FUZZYSEARCH: Added: %s\n",newstrings[x].c_str() );
							}
						}
					}// for( r = 0 --> size )
				}// if( pos >= 0 )
			}
		} else {
			//Log(1,L"FAILED\n");
		}
	}

	return( strings );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
